# -*- coding:utf-8 -*-
import sys
if sys.version_info < (3,): memoryview = buffer
    
from qgis.PyQt.QtCore import (QAbstractTableModel, QDataStream, QEvent, QFileInfo, QIODevice, QObject, QPoint, QSize, QRect, QRegExp, Qt, pyqtSignal) 

from qgis.PyQt.QtCore import (QCoreApplication, QFileInfo, QItemSelectionModel, QLocale, QSettings, QSortFilterProxyModel, QStringListModel, QTranslator)

from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)

from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QCalendarWidget, QCheckBox, QCompleter,
                         QComboBox, QCommandLinkButton, QDialog, QDialogButtonBox, QDoubleSpinBox, QFileDialog, QGroupBox,
                         QHBoxLayout, QLabel, QLineEdit, QListView, QListWidget, QListWidgetItem, QMdiArea,
                         QMdiSubWindow, QMenu, QMessageBox, QProgressBar,QPushButton, QRadioButton, QSizePolicy,
                         QSlider, QSpinBox, QTabBar, QTabWidget, QTableView, QTableWidget, QTableWidgetItem, QTextBrowser,
                         QTextEdit, QTreeView, QTreeWidget, QTreeWidgetItem, QToolBar, QToolButton, QVBoxLayout, QWidget)

from qgis.PyQt.QtGui import (QBrush, QColor, QFont, QFontMetrics, QIcon, QImage, QKeySequence, QLinearGradient,
                         QMovie, QPainter, QPixmap, QRegExpValidator, QStandardItem, QStandardItemModel, QTextOption)

 
from qgis.core import (Qgis, QgsCoordinateReferenceSystem, QgsProject) 
from qgis.gui import (QgsMessageBar, QgsProjectionSelectionDialog)
from datetime import datetime, date
from qgis.PyQt.Qsci import (QsciAPIs, QsciLexerXML, QsciScintilla, QsciScintillaBase, QsciStyle)
from textwrap import fill
from ast import literal_eval
from configparser import ConfigParser
from urllib.parse import (quote, quote_plus)
from webbrowser import open as webbrowseropen
from os.path import dirname


#-------------------------
# CLASSES
#-------------------------
class BaseTableModel(QAbstractTableModel):
    def __init__(self, header=None, data=None, parent=None):
        QAbstractTableModel.__init__(self, parent)
        self._header = header if header else []
        self.resdata = data if data else []

    def headerToString(self, sep=u"\t"):
        header = self._header
        return sep.join(header)

    def rowToString(self, row, sep=u"\t"):
        text = u""
        for col in range(self.columnCount()):
            text += u"%s" % self.getData(row, col) + sep
        return text[:-1]

    def getData(self, row, col):
        try : return self.resdata[row][col]
        except : pass

    def columnNames(self):
        return list(self._header)

    def rowCount(self, parent=None):
        return len(self.resdata)

    def columnCount(self, parent=None):
        return len(self._header)

    def data(self, index, role):
        if role != Qt.DisplayRole and role != Qt.FontRole:
            return None

        val = self.getData(index.row(), index.column())

        if role == Qt.FontRole:  # draw NULL in italic
            if val is not None:
                return None
            f = QFont()
            f.setItalic(True)
            return f

        if val is None:
            return "NULL"
        elif isinstance(val, memoryview): 
            # hide binary data
            return None
        elif isinstance(val, (str, unicode)) and len(val) > 300:
            # too much data to display, elide the string
            return u"%s..." % val[:300]
        return unicode(val)  # convert to string
    """
    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
            return self._header[section]
        return QAbstractTableModel.headerData(self, section, orientation, role)
    """
    def headerData(self, section, orientation, role):
        """
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return self._header[section]
        """
        if role != Qt.DisplayRole:
            return None

        if orientation == Qt.Vertical:
            # header for a row
            return section + 1
        else:
            # header for a column
            return self._header[section]

class myMsgBoxDialog(QMessageBox):
    def __init__(self, parent, title, msg, icon, *args):
            QMessageBox.__init__(self, *args)
            from qsphere.qsphere_tools import (getThemeIcon, MakeWindowIcon)
            self.setWindowTitle(title)
            self.setIcon(QMessageBox.Question)
            MakeWindowIcon(self, "qsphere.png")

            groupBox1 = QGroupBox(self)
            groupBox1.setFixedSize(420, 180)
            groupBox1.setObjectName("groupBox1")

            labelText = QLabel(groupBox1)
            labelText.setText(msg)
            labelText.setAlignment(Qt.AlignCenter)
            labelText.setFixedSize(420, 50)
            labelText.setGeometry(5, 115, 420, 50)

            labelImage = QLabel(groupBox1)
            myImgPath = getThemeIcon(icon)
            carIcon = QImage(myImgPath)
            labelImage.setPixmap(QPixmap.fromImage(carIcon).scaled(128, 108))
            labelImage.setFixedSize(128, 108)
            labelImage.setGeometry(145, 10, 128, 108)

            zButtonAccept = QPushButton(self)
            zButtonAccept.setObjectName("SaveButton")
            zButtonAccept.setText(QApplication.translate("QSphere", "Apply"))
            zButtonAccept.setFixedSize(100, 25)

            zButtonReject = QPushButton(self)
            zButtonReject.setObjectName("CloseButton")
            zButtonReject.setText(QApplication.translate("QSphere", "Cancel"))
            zButtonReject.setFixedSize(100, 25)

            zButtonAccept.clicked.connect(self.accept)
            zButtonReject.clicked.connect(self.reject)

            self.layout().addWidget(groupBox1,0,1,1,2)
            self.layout().addWidget(zButtonAccept,4,1,1,1)
            self.layout().addWidget(zButtonReject,4,2,1,1)

            self.setStandardButtons(self.NoButton)

          
class mySubWindow(QMdiSubWindow):
     def __init__(self, parent, dialog, icon, *args):
         QMdiSubWindow.__init__(self, *args)
         self.setWidget(dialog)
         self.widget = self.children()[3]
         self.parent = parent
         self.setWindowIcon(icon)
         self.setMinimumSize(0, 0) 
         self.resize(self.widget.width()-100,  self.widget.height())

     def closeEvent(self,event):  self.parent.closeCurrentWindow()


class MyScintilla(QsciScintilla):
     def __init__(self, parent, nameObj, readOnly, boundsGeom):
        super(MyScintilla, self).__init__(parent)
        self.parent = parent
        self.setObjectName(nameObj)
        self.setReadOnly(readOnly)
        self.duration_info = self.parent.duration_info
        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed )
        self.barInfo.setGeometry(0, 0, boundsGeom[2]-10, 90) 
        self.setGeometry(boundsGeom[0], boundsGeom[1], boundsGeom[2], boundsGeom[3])
 
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.context_menu_requested)

        self.ARROW_MARKER_NUM = 8

        self.findDialog = FindDialog(self)
        font = QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(12)
        self.setFont(font)
        self.setMarginsFont(font)

        fontmetrics = QFontMetrics(font)
        self.setMarginsFont(font)
        self.setMarginWidth(0, fontmetrics.width("00000") + 6)
        self.setMarginLineNumbers(1, True)
        self.setMarginsBackgroundColor(QColor("#cccccc"))

        self.setMarginSensitivity(1, True)
        
        self.markerDefine(QsciScintilla.RightArrow, self.ARROW_MARKER_NUM)
        self.setMarkerBackgroundColor(QColor("#ee1111"), self.ARROW_MARKER_NUM)
        self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
        self.setCaretLineVisible(True)
        self.setCaretLineBackgroundColor(QColor("#ffffcc"))
        self.setUtf8(True)

        lexer = QsciLexerXML(self) 
        api = QsciAPIs(lexer)
        api.prepare()
        self.setAutoCompletionSource(QsciScintilla.AcsDocument)
        self.setAutoCompletionFillupsEnabled(True)
        self.setAutoCompletionThreshold(2)
        self.setLexer(lexer)
        
        font = QFont() 
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(12)
        self.setFont(font)
        
        font.setPointSize(8)
        self.setMarginsFont(font)
        large = '9999'
        self.setMarginWidth(0, large) 
        self.setMarginLineNumbers(0, True)
        self.setMarginWidth(1,1) 
        
        self.setMarginsBackgroundColor(QColor("#cccccc"))
        self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
        self.setCaretLineVisible(True)
        self.setCaretLineBackgroundColor(QColor("#ffe4e4"))
        
        self.setTabWidth(5) 
        self.setIndentationsUseTabs(True) 
        self.setIndentationWidth(5) 
        self.setTabIndents(True) 
        self.setBackspaceUnindents(True) 
        self.setAutoIndent( True )
        self.setIndentationGuides( True )
        self.setFolding(QsciScintilla.CircledTreeFoldStyle) 
        self.setFoldMarginColors(QColor("#99CC66"), QColor("#AAAAAA"))
        self.setMarginsBackgroundColor(QColor("#00b4FF"))
        self.setMarginsForegroundColor(QColor("#FFFFFF"))
        
        self.setWrapMode( QsciScintilla.WrapWord )
        self.setWrapIndentMode(QsciScintilla.WrapIndentSame )
        self.setAnnotationDisplay(QsciScintilla.AnnotationStandard) 


        self.find_text = ""
        self.point = None
        self.blockText = ""
        self.annotationStyle = QsciStyle()
        self.annotationStyle.setColor(QColor(255,255,255, 255))
        self.annotationStyle.setPaper(QColor(0,0,0, 125))
        font.setPointSize(18)
        self.annotationStyle.setFont(font)
        self.setWrapMode(QsciScintilla.WrapWord)


     def wheelEvent(self, ev):
           if Qt.ControlModifier & ev.modifiers():
               if ev.delta() > 0:
                   self.zoomIn()
               else:
                   self.zoomOut()
           else:
               try : return super(self, self).wheelEvent(ev)
               except : pass

     def mouseDoubleClickEvent(self, event):
         self.point = QPoint(event.pos().x(), event.pos().y())
         lineFrom = self.lineAt(self.point)
         if self.text()!= "" :
              lineFrom = self.lineAt(self.point)
              if lineFrom > 0 :
                 try : self.extractLine()
                 except : pass
         return
         

     def context_menu_requested(self, point):
            if self.sender().objectName()== "mActionXMLEditor" : return
            lineFrom = self.lineAt(point)
            if lineFrom <0 : return
            
            from qsphere.qsphere_tools import getThemeIcon

            self.point = point
            self.context_menu = QMenu() 

            menuIcon = getThemeIcon("undo.png")
            zText = (QApplication.translate("QSphere", "Undo"))
            self.myundo = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.myundo)
            self.myundo.triggered.connect(self.undo)
            self.myundo.setShortcut(QKeySequence("Ctrl+Z"))

            menuIcon = getThemeIcon("redo.png")
            zText = (QApplication.translate("QSphere", "Redo"))
            self.myredo = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.myredo)
            self.myredo.triggered.connect(self.redo)
            self.myredo.setShortcut(QKeySequence("Ctrl+R"))

            self.context_menu.addSeparator()
               
            menuIcon = getThemeIcon("copier.png")
            zText = (QApplication.translate("QSphere", "Copy"))
            self.copy = self.context_menu.addAction(QIcon(menuIcon), zText)
            self.ToCopy = unicode(self.selectedText())
            self.copy.triggered.connect(self.MakeToCopy)
            self.copy.setShortcut(QKeySequence("Ctrl+C"))

            self.copy.setEnabled(True) if unicode(self.selectedText()) else self.copy.setEnabled(False) 

            menuIcon = getThemeIcon("copier.png")
            zText = (QApplication.translate("QSphere", "Copy the xml block"))
            self.copyXmlBlock = self.context_menu.addAction(QIcon(menuIcon), zText)
            self.copyXmlBlock.triggered.connect(self.MakeToCopyXmlBlock)

            self.copyXmlBlock.setEnabled(True) if self.blockText!= "" else self.copyXmlBlock.setEnabled(False) 

            menuIcon = getThemeIcon("coller.png")
            zText = (QApplication.translate("QSphere", "Paste"))
            self.mypaste = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.mypaste)
            self.mypaste.triggered.connect(self.paste)
            self.mypaste.setShortcut(QKeySequence("Ctrl+V"))

            self.mypaste.setEnabled(True) if unicode(QApplication.clipboard().text()) else self.mypaste.setEnabled(False)


            menuIcon = getThemeIcon("cut.png")
            zText = (QApplication.translate("QSphere", "Cut"))
            self.mycut = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.mycut)
            self.mycut.triggered.connect(self.cut)
            self.mycut.setShortcut(QKeySequence("Ctrl+X"))

            self.mycut.setEnabled(True) if unicode(self.selectedText()) else self.mycut.setEnabled(False)

            self.context_menu.addSeparator()            
           
            menuIcon = getThemeIcon("selectall.png")
            zText = (QApplication.translate("QSphere", "Select all"))
            self.selall = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.selall)
            self.selall.triggered.connect(self.SelectAll)
            self.selall.setShortcut(QKeySequence("Ctrl+A"))

            
            self.context_menu.addSeparator()

            menuIcon = getThemeIcon("nodes_expand.png")
            zText = (QApplication.translate("QSphere", "Expand all nodes"))
            self.expandAllNodes = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.expandAllNodes)
            self.expandAllNodes.triggered.connect(self.expandAll)

            menuIcon = getThemeIcon("nodes_reduce.png")
            zText = (QApplication.translate("QSphere", "Close all nodes"))
            self.closeAllNodes = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.closeAllNodes)
            self.closeAllNodes.triggered.connect(self.closeAll)

            self.context_menu.addSeparator()


            menuIcon = getThemeIcon("notes.png")
            zText = (QApplication.translate("QSphere", "Get annotation"))
            self.getline = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.getline)
            self.getline.triggered.connect(self.extractLine)

            self.context_menu.addSeparator()

            menuIcon = getThemeIcon("delonenote.png")
            zText = (QApplication.translate("QSphere", "Delete this annotation"))
            self.delOneNota = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.delOneNota)
            self.delOneNota.triggered.connect(self.clearMyAnnotation)

            
            menuIcon = getThemeIcon("delallnotes.png")
            zText = (QApplication.translate("QSphere", "Delete all annotations"))
            self.delAllNota = QAction(QIcon(menuIcon), zText, self)
            self.context_menu.addAction(self.delAllNota)
            self.delAllNota.triggered.connect(self.clearMyAnnotations)


            self.getline.setEnabled(True) if unicode(self.selectedText()) else self.getline.setEnabled(False)

            self.context_menu.exec_(self.mapToGlobal(point))

     def MakeToCopy(self): QApplication.clipboard().setText(self.ToCopy)
     def MakeToCopyXmlBlock(self): QApplication.clipboard().setText(self.blockText)
     def SelectAll(self): self.selectAll(True)
     def expandAll(self): self.clearFolds()
          
     def closeAll(self):
         self.foldAll(True)
         self.foldLine(0)
     def clearMyAnnotation(self):
          if self.point!=None :
             lineFrom = self.lineAt(self.point)
             self.clearAnnotations(lineFrom)
     def clearMyAnnotations(self): self.clearAnnotations(-1)
     def extractLine(self):

         if self.point!=None :
            lineFrom = self.lineAt(self.point)
            lineOr = lineFrom
             
            selection = self.setSelection(lineFrom, 0, lineFrom, self.lineLength(lineFrom)-1)
            text = self.selectedText()
               
            baliseD = text
            #END BALISE
            if text.find("</")!=-1: return
            
            posD = text.find("<")
            if posD != -1 :
               posF = text.find(">", posD)

               if posF != -1 and posF > posD :
                  baliseD = "%s" % (text[posD:posF+1])

                  #TEMP - AJUST BALISE WITH PROPERTIES  
                  if baliseD.find(" ")!=-1 :
                     tempo = baliseD.split(" ")
                     baliseD = "%s>" % tempo[0]
                     
                  baliseF = baliseD.replace("<", "</")
                  
                  while text.find(baliseF, posF) == -1 :
                       lineFrom+= 1
                       text = "%s\n%s" % (text.strip(), self.text(lineFrom).strip())

            self.blockText = text
            self.annotate(lineOr, fill(self.blockText,50), self.annotationStyle)
            self.foldLine(lineOr)
 

class FindDialogToolSearchFiles(QDialog):
    findNext = pyqtSignal(str, int)
    findPrevious = pyqtSignal(str, int)
  
    def __init__(self, *args, **kwargs):
        super(FindDialogToolSearchFiles, self).__init__(*args, **kwargs)
        self.setMinimumSize(400, 200)
        self.setMaximumSize(400, 200)

        self.groupBoxSearch = QGroupBox(self)
        self.groupBoxSearch.setGeometry(5, 5, 390, 140)
        
        self.label = QLabel(self)
        self.label.setText("%s : " % QApplication.translate("QSphere", "Search for"))
        self.label.setGeometry(10, 15, 80, 25)
        self.label.setAlignment(Qt.AlignRight)
        self.lineEdit = QLineEdit(self)
        self.lineEdit.setGeometry(100, 10, 280, 20)
        self.ret = 0
  
        self.caseCheckBox = QCheckBox(self)
        self.caseCheckBox.setText(QApplication.translate("QSphere", "Match case"))
        self.caseCheckBox.setGeometry(100, 40, 180, 20)
        
        self.wholeWordCheckBox = QCheckBox(self)
        self.wholeWordCheckBox.setText(QApplication.translate("QSphere", "Match whole word"))
        self.wholeWordCheckBox.setGeometry(100, 60, 180, 20)

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.setColumnHidden(1, True)
        self.view.horizontalHeader().setDefaultSectionSize(500)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("""QTableView {selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 %s, stop: 1 %s);}""" % ("#FFFFFF", "#000000"))
        self.view.setShowGrid(False)
        self.view.setMinimumSize(500, 40)

        self.ComboLines = QComboBox(self)
        self.ComboLines.setGeometry(100, 90, 280, 40)
        self.ComboLines.setObjectName("ComboLines")
        self.ComboLines.setAccessibleName("ComboLines")
        self.ComboLines.setModel(self.model)
        self.ComboLines.setView(self.view)

        self.countButton = QPushButton(self)
        self.countButton.setText("&%s" % (QApplication.translate("QSphere", "Find")))
        self.countButton.setGeometry(10, 90, 80, 25)
        self.countButton.setEnabled(False)
  
        self.closeButton = QPushButton(self)
        self.closeButton.setText("&%s" % (QApplication.translate("QSphere", "Close")))
        self.closeButton.setGeometry(290, 160, 100, 25)
  
        self.lineEdit.textChanged.connect(self.enableFindButton)
        self.ComboLines.currentIndexChanged.connect(self.gotoLine)
        self.countButton.clicked.connect(self.countFindClicked)
        self.closeButton.clicked.connect(self.closeme)

        self.setWindowTitle(QApplication.translate("QSphere", "Search word"))
        self.lineEdit.setFocus()
        

    def enableFindButton(self, text): self.countButton.setEnabled(not text=="")
    def closeme(self): self.close()
    
    def gotoLine(self):
        if self.parent().TabViewResults.currentIndex() != 0 : self.parent().TabViewResults.setCurrentIndex(0)
        if self.ComboLines.currentIndex() == -1 : return
        oldItem = self.parent().listFiles.currentItem()
        self.parent().listFiles.setCurrentItem(self.Items[self.ComboLines.currentIndex()])


    def countFindClicked(self):
        from qsphere.qsphere_tools import SendMessage
        
        zText, ignoreCase, wholeWord = self.getFindText(), self.isCaseSensitive(), self.isWholeWord()
        line = index = 0
        lstPos = []
        self.ComboLines.clear()

        Flags = Qt.MatchExactly  if self.isWholeWord() else Qt.MatchContains
        if self.isCaseSensitive() : Flags = Flags  & Qt.MatchCaseSensitive

        listResults = []
        self.Items = self.parent().listFiles.findItems(zText, Flags)
        
        if self.Items != [] :
           listResults = [ item.text() for item in self.Items] 
           self.ComboLines.addItems(listResults)
           self.ComboLines.setCurrentIndex(0)
        
        zTitle = QApplication.translate("QSphere", "Information")
        zMsg = QApplication.translate("QSphere","Count")
        SendMessage(self.parent(), zTitle , "%s : <u><i>%s</i></ul>" % (zMsg, len(listResults)), QgsMessageBar.INFO, 2)

      
    def getFindText(self): return self.lineEdit.text()
    def isCaseSensitive(self): return self.caseCheckBox.isChecked()
    def isWholeWord(self): return self.wholeWordCheckBox.isChecked()



class FindDialog(QDialog):
    findNext = pyqtSignal(str, int)
    findPrevious = pyqtSignal(str, int)
  
    def __init__(self, *args, **kwargs):
        super(FindDialog, self).__init__(*args, **kwargs)
        self.setMinimumSize(400, 200)
        self.setMaximumSize(400, 200)

        self.groupBoxSearch = QGroupBox(self)
        self.groupBoxSearch.setGeometry(5, 5, 390, 160)
        
        self.label = QLabel(self)
        self.label.setText(QApplication.translate("QSphere", "Search for:"))
        self.label.setGeometry(10, 15, 80, 25)
        self.label.setAlignment(Qt.AlignRight)
        self.lineEdit = QLineEdit(self)
        self.lineEdit.setGeometry(100, 10, 280, 20)
        self.ret = 0
  
        self.caseCheckBox = QCheckBox(self)
        self.caseCheckBox.setText(QApplication.translate("QSphere", "Match case"))
        self.caseCheckBox.setGeometry(100, 40, 180, 20)
        
        self.wholeWordCheckBox = QCheckBox(self)
        self.wholeWordCheckBox.setText(QApplication.translate("QSphere", "Match whole word"))
        self.wholeWordCheckBox.setGeometry(100, 60, 180, 20)

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.setColumnHidden(1, True)
        self.view.horizontalHeader().setDefaultSectionSize(400)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("""QTableView {selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 %s, stop: 1 %s);}""" % ("#FFFFFF", "#000000"))
        self.view.setShowGrid(False)
        self.view.setMinimumSize(500, 40)

        self.ComboLines = QComboBox(self)
        self.ComboLines.setGeometry(100, 90, 280, 40)
        self.ComboLines.setObjectName("ComboLines")
        self.ComboLines.setAccessibleName("ComboLines")
        self.ComboLines.setModel(self.model)
        self.ComboLines.setView(self.view)

        self.lblCounter = QLabel(self)
        self.lblCounter.setText("0")
        self.lblCounter.setGeometry(30, 120, 40, 25)
        self.lblCounter.setAlignment(Qt.AlignCenter)
        self.lblCounter.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.countButton = QPushButton(self)
        self.countButton.setText("&%s" % (QApplication.translate("QSphere", "Find")))
        self.countButton.setGeometry(10, 90, 80, 25)
        self.countButton.setEnabled(False)
  
        self.closeButton = QPushButton(self)
        self.closeButton.setText("&%s" % (QApplication.translate("QSphere", "Close")))
        self.closeButton.setGeometry(290, 160, 100, 25)
  
        self.lineEdit.textChanged.connect(self.enableFindButton)
        self.ComboLines.currentIndexChanged.connect(self.gotoLine)
        self.countButton.clicked.connect(self.countFindClicked)
        self.closeButton.clicked.connect(self.closeme)

        self.setWindowTitle(QApplication.translate("QSphere", "Search word"))
        self.lineEdit.setFocus()
        

    def enableFindButton(self, text): self.countButton.setEnabled(not text=="")
    def closeme(self): self.close()
    
    def gotoLine(self):
        if self.ComboLines.currentIndex() == -1 : return
        model = self.ComboLines.model()
        try : pos = model.item(self.ComboLines.currentIndex(),1).text()
        except : pos = "0|0"
        coord = pos.split("|")
        try : line = int(coord[0])
        except : line = 0
        try : index = int(coord[1])
        except : index = 0

        zText = self.getFindText()
        self.parent().setCursorPosition(line-1,index-len(zText))
        self.parent().findFirst(self.getFindText(), True, self.isCaseSensitive(), self.isWholeWord(), True, True, line-1, index-len(zText))

    def countFindClicked(self):
        zText, ignoreCase, wholeWord = self.getFindText(), self.isCaseSensitive(), self.isWholeWord()
        line = index = 0
        lstPos = []
        self.ComboLines.clear()

        self.parent().setCursorPosition(line,index)
        
        bFind = self.parent().findFirst(zText, True, ignoreCase, wholeWord, True, True, line, index)
        line, index = self.parent().getCursorPosition()
        
        while bFind and not (line, index) in lstPos :
            lstPos.append((line, index))
            self.AppendItemInModel(self.model, self.parent().text(line), "%s|%s" % (line+1, index))
            bFind = self.parent().findFirst(zText, True, ignoreCase, wholeWord, True, True, line, index)
            line, index = self.parent().getCursorPosition()

        from qsphere.qsphere_tools import SendMessage
        self.parent().setCursorPosition(0,0)

        self.lblCounter.setText("%s" % len(lstPos))
             
        zTitle = QApplication.translate("QSphere", "Information")
        zMsg = QApplication.translate("QSphere","Count")
        SendMessage(self.parent().parent, zTitle , "%s : <u><i>%s</i></ul>" % (zMsg, len(lstPos)), QgsMessageBar.INFO, self.parent().duration_info)

        self.gotoLine()
        self.ret = len(lstPos)

        
    def getFindText(self): return self.lineEdit.text()
    def isCaseSensitive(self): return self.caseCheckBox.isChecked()
    def isWholeWord(self): return self.wholeWordCheckBox.isChecked()
    def AppendItemInModel(self, zModel, line, elt):
        if (elt == "") : return
        zModel.appendRow([QStandardItem(fill(line.strip(),50)), QStandardItem(elt)])


class FilteredComboBox(QComboBox):
    def __init__(self, parent=None):
        super(FilteredComboBox, self).__init__(parent)
        self.setFocusPolicy(Qt.StrongFocus)
        self.setEditable(True)

        self.pFilterModel = QSortFilterProxyModel(self)
        self.pFilterModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
        self.pFilterModel.setSourceModel(self.model())

        self.completer = QCompleter(self.pFilterModel, self)
        self.completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
        self.setCompleter(self.completer)

        def filter(text): self.pFilterModel.setFilterFixedString("%s" % (text))

        self.lineEdit().textEdited[unicode].connect(filter)
        self.completer.activated.connect(self.on_completer_activated)

    def on_completer_activated(self, text):
        if text:
            index = self.findText("%s" % (text))
            self.setCurrentIndex(index)

    def setModel(self, model):
        super(FilteredComboBox, self).setModel(model)
        self.pFilterModel.setSourceModel(model)
        self.completer.setModel(self.pFilterModel)

    def setModelColumn(self, column):
        self.completer.setCompletionColumn(column)
        self.pFilterModel.setFilterKeyColumn(column)
        super(FilteredComboBox, self).setModelColumn(column)

class editor(QTextEdit):
       def __init__(self, parent):
           super(editor, self).__init__(parent)
           self.setWindowTitle("QTextEdit Standard Output Redirection")
           self.setStyleSheet("color: white; background-color: black;")
           self.setObjectName(parent.objectName())
           
class LusTREDialog(QDialog):
       def  __init__(self, parent, serverLusTRE, isModal):
         QDialog.__init__(self)
         self.setWindowTitle("LusTRE %s ..." % (QApplication.translate("QSphere","Web services")))
         self._W, self._H = 580, 400
         self.setMinimumSize(self._W,self._H)
         self.setMaximumSize(self._W*2,self._H*2)
         self.resize(self._W,self._H)

         self.parent = parent
         self.iface = self.parent.iface
         self.langueTR = self.parent.langueTR
         self.styleSheetPushButton = self.parent.styleSheetPushButton
         self.serverLusTRE = serverLusTRE
         self.isModal = isModal
         self.countkeywords = 0
         self.groupLusTREInfo = QGroupBox(self)


         from qsphere.qsphere_tools import (getThemeIcon, makeGetOptions, MakeWindowIcon)

         MakeWindowIcon(self, "lustre_wizard.png")
         makeGetOptions(self)

         
         zIcon = getThemeIcon("servers.png")
         carIcon = QImage(zIcon)
         self.ImgServerLusTRE = QLabel(self.groupLusTREInfo)
         self.ImgServerLusTRE.setObjectName("ImgServerLusTRE")
         self.ImgServerLusTRE.setAlignment(Qt.AlignCenter)
         self.ImgServerLusTRE.setPixmap(QPixmap.fromImage(carIcon).scaled(42,42))
         

         self.labelServerLusTRE = QLabel(self.groupLusTREInfo)
         self.labelServerLusTRE.setObjectName("labelServerLusTRE")
         self.labelServerLusTRE.setAlignment(Qt.AlignLeft)
         self.labelServerLusTRE.setWordWrap(True)
         self.labelServerLusTRE.setText(self.serverLusTRE)

         self.labelKeywordForLusTRE = QLabel(self.groupLusTREInfo)
         self.labelKeywordForLusTRE.setText("%s : " % (QApplication.translate("QSphere","Keyword")))
         self.labelKeywordForLusTRE.setObjectName("labelUserServerCSWT")
         self.labelKeywordForLusTRE.setAlignment(Qt.AlignRight)    

         self.keywordForLusTRE = MyWidgetLineEdit(self.groupLusTREInfo)
         self.keywordForLusTRE.initType(6)
         self.keywordForLusTRE.setObjectName("keywordForLusTRE")
         self.keywordForLusTRE.setAccessibleName("keywordForLusTRE")
         self.keywordForLusTRE.textChanged.connect(self.keywordForLusTRE.VerifExpReg)

         zIcon = getThemeIcon("find.png") 
         zToolTip = QApplication.translate("QSphere","Get suggestion from the server")
         self.getSuggestionButton = MyPushButton(self.groupLusTREInfo) 
         self.getSuggestionButton.initPushButton(24, 24, 50, 50, "getSuggestionButton", "", "%s %s" % (zToolTip, "LusTRE"), True, zIcon, 24, 24, True)  


         self.labelParamNbKeywords = QLabel(self.groupLusTREInfo)
         self.labelParamNbKeywords.setObjectName("labelParamNbKeywords")
         self.labelParamNbKeywords.setAlignment(Qt.AlignRight)
         self.labelParamNbKeywords.setWordWrap(True)
         self.labelParamNbKeywords.setText("%s :" % (QApplication.translate("QSphere","Num max results")))
         self.labelParamNbKeywords.setGeometry(100, 75, 120, 25)

         self.paramNbKeywords = QSpinBox(self.groupLusTREInfo)
         self.paramNbKeywords.setGeometry(220, 70, 40, 25)
         self.paramNbKeywords.setMinimum(10)
         self.paramNbKeywords.setMaximum(100)
         self.paramNbKeywords.setValue(10)
         self.paramNbKeywords.setSingleStep(5)
         self.paramNbKeywords.setObjectName("paramNbKeywords")

         self.labelParamLangKeywords = QLabel(self.groupLusTREInfo)
         self.labelParamLangKeywords.setObjectName("labelParamLangKeywords")
         self.labelParamLangKeywords.setAlignment(Qt.AlignRight)
         self.labelParamLangKeywords.setWordWrap(True)
         self.labelParamLangKeywords.setText("%s :" % (QApplication.translate("QSphere","Language")))
         self.labelParamLangKeywords.setGeometry(260, 75, 80, 25)

         self.ParamLangKeywords = QComboBox(self.groupLusTREInfo)
         self.ParamLangKeywords.setGeometry(340, 70, 40, 25)
         self.ParamLangKeywords.addItems(sorted(self.parent.MainPlugin.dicoLangs))
         self.ParamLangKeywords.setEditable(True)
         self.ParamLangKeywords.setInsertPolicy(QComboBox.InsertAlphabetically)
         self.ParamLangKeywords.setObjectName("ParamLangKeywords")

         self.allChildsActive = QCheckBox(self.groupLusTREInfo)
         self.allChildsActive.setGeometry(390, 70, 150, 25)
         self.allChildsActive.setText(QApplication.translate("QSphere","All children are activated"))
         self.allChildsActive.setObjectName("allChildsActive")
         self.allChildsActive.setChecked(False)         

         self.treeWidgetExpand = MyPushButton(self.groupLusTREInfo)
         self.treeWidgetExpand.initPushButton(40, 24, 5, 70, "treeWidgetExpand", "", QApplication.translate("QSphere", "Expand all"), True, getThemeIcon("tree_expand.png"), 40, 24, True)
        
         self.treeWidgetReduce = MyPushButton(self.groupLusTREInfo)
         self.treeWidgetReduce.initPushButton(40, 24, 45, 70, "treeWidgetReduce", "", QApplication.translate("QSphere", "Reduce all"), True, getThemeIcon("tree_reduce.png"), 40, 24, True)

         zLanguage = self.parent.MainPlugin.dicoLangs[self.parent.MainPlugin.indexLang]
         zIndex = self.ParamLangKeywords.findText(zLanguage) if self.ParamLangKeywords.findText(zLanguage) !=-1 else 0
         self.ParamLangKeywords.setCurrentIndex(zIndex)

         self.treeKeywords = QTreeWidget(self.groupLusTREInfo)
         self.treeKeywords.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self.treeKeywords.setObjectName("treeKeywords")
         self.treeKeywords.headerItem().setText(0, "1")
         self.treeKeywords.header().setVisible(False)
         self.treeKeywords.header().setDefaultSectionSize(200)

         self.treeKeywords.setStyleSheet("* { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                     "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")

         self.labelResultKeywords = QLabel(self)
         self.labelResultKeywords.setAlignment(Qt.AlignLeft)
         self.labelResultKeywords.setObjectName("labelResultKeywords")
         self.labelResultKeywords.setAccessibleName("labelResultKeywords")
         self.labelResultKeywords.setText("")

         self.status_txt = QLabel(self)
         self.movie = QMovie(getThemeIcon("sablier.gif"))
         self.status_txt.setMovie(self.movie)
         self.status_txt.setLayout(QHBoxLayout())
         self.status_txt.layout().addWidget(QLabel(''))
         self.status_txt.setVisible(False)         

         self.barInfo = QgsMessageBar(self)
         self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

         self.quitButton = MyPushButton(self)
         self.quitButton.initPushButton(80, 25, 10,  5, "close", QApplication.translate("QSphere","Close"), \
                                        QApplication.translate("QSphere","Close"), True, "", 22, 22, False)
         self.quitButton.clicked.connect(self.closeme)

         self.initTreeKeywords()
         self.refreshCounter(self.countkeywords)

         self.getSuggestionButton.clicked.connect(self.getSuggestion)
         self.treeWidgetExpand.clicked.connect(self.expandTreeWidget)
         self.treeWidgetReduce.clicked.connect(self.expandTreeWidget)         
         self.treeKeywords.itemDoubleClicked.connect(self.addKeyword)
         
       def changeActivateWindow(self, int) :  self.parent.changeActivateWindow(int)   

       def resizeEvent(self, ev):
                zSize = ev.size()

                self.groupLusTREInfo.setGeometry(10, 10, zSize.width()-20, zSize.height()-60)

                self.ImgServerLusTRE.setGeometry(40, 2, 48, 48)
                self.labelServerLusTRE.setGeometry(90, 10, zSize.width()-110, 40)
                         
                self.labelKeywordForLusTRE.setGeometry(10, 45, 80, 25)
                self.keywordForLusTRE.setGeometry(90, 40, zSize.width()-150, 25)
                self.getSuggestionButton.setGeometry(zSize.width()-55, 40, 24, 24)

                self.treeKeywords.setGeometry(10, 100, self.groupLusTREInfo.width()-20, self.groupLusTREInfo.height()-110)
                
                self.status_txt.setGeometry(int(zSize.width()/2)-64, int(zSize.height()/2)-64, 64, 64)
                self.barInfo.setGeometry(0, 0, zSize.width(), 90)

                self.labelResultKeywords.setGeometry(25, zSize.height()-45, 200, 25)
                self.quitButton.setGeometry(zSize.width()-90, zSize.height()-40, 80, 25)

       def closeEvent(self,event):
          self.parent.dialogLUSTRE = None
          self.parent.windowsNoMDIChilds.remove(self)


       def expandTreeWidget(self):
               if self.sender() == self.treeWidgetExpand : self.treeKeywords.expandAll()
               else :
                   self.treeKeywords.collapseAll()
                   self.treeKeywords.expandItem(self.treeKeywords.itemAt(0, 0))

       def refreshCounter(self, countkeywords):
               sKeywords = QApplication.translate("QSphere", "keywords") if countkeywords > 1 else QApplication.translate("QSphere", "keyword") 
               self.labelResultKeywords.setText("%s %s %s %s [%s]." % (countkeywords, sKeywords, \
                                                             QApplication.translate("QSphere", "in"), QApplication.translate("QSphere", "language"), \
                                                             self.ParamLangKeywords.currentText()))  

       def startMovie(self):
              self.status_txt.setVisible(True)
              self.movie.start()
              self.status_txt.repaint()

       def stopMovie(self):
              self.movie.stop()
              self.status_txt.setVisible(False)


       def initTreeKeywords(self):
               self.results_items = None
               self.results_items = QTreeWidgetItem()
               self.results_items.setText(0, QApplication.translate("QSphere", "Results"))

               self.treeKeywords.addTopLevelItem(self.results_items)
               self.treeKeywords.expandAll()
               self.treeKeywords.resizeColumnToContents(0)
               self.treeKeywords.resizeColumnToContents(1)
               self.countkeywords = 0

       def testURL(self, ucase):
              from qsphere.qsphere_tools import getThemeIcon, multi_get 
              import urllib

              if self.keywordForLusTRE.text() == "" :
                 self.MakeMsgNotValid(self.keywordForLusTRE.text())
                 return    

              self.startMovie()       
              libelleDict = {1: "suggestions"}
              libelleSubDicts = {1: ("title", "language", "conceptUri", "source")}
              zLanguage = self.parent.MainPlugin.dicoLangs[self.parent.MainPlugin.indexLang]

              try : searchkeywords = quote(self.keywordForLusTRE.text())
              except : searchkeywords = self.keywordForLusTRE.text().encode('utf-8')
       
              if ucase == 0 : zUrl = self.serverLusTRE
              elif ucase == 1 : zUrl = "%stfes/rest/GetSuggestions?keyword=%s&maxCount=%s&languages=%s&source=true" % (self.serverLusTRE, searchkeywords, self.paramNbKeywords.value(), self.ParamLangKeywords.currentText())
                     
              sites = [('%s' % (zUrl), 'GET', {'Content-Type': 'application/html'}, 'None')]
              isValidURL = multi_get(self.movie, self.status_txt, sites, 10, 1)[0][1]
              
              mycodelang = self.parent.languesDico[zLanguage]['bibliographic']               
              menuIcon = getThemeIcon("dkeywords.png")
              menuIconMetadata = getThemeIcon("metadata.png")
              menuIconSource = getThemeIcon("source.png")
              menuIconLanguage = getThemeIcon("ressources/images/%s.png" % (mycodelang))

              if isValidURL != None :

                 if ucase > 0 :

                        zTest = True
                        isValidURL = isValidURL.decode("utf-8", "replace")
                        try : mydict = literal_eval(isValidURL)
                        except :
                              self.MakeMsgNotValid(zUrl)
                              zTest = False

                        if zTest :     
                               self.treeKeywords.clear()
                               self.initTreeKeywords()

                               zListSubDicts = mydict[libelleDict[ucase]]
                               for dicts in zListSubDicts :
                                   keywordItem = None 
                                   for i in range(len(libelleSubDicts[ucase])):
                                       if i == 0 :  
                                          keywordItem = QTreeWidgetItem()
                                          keywordItem.setIcon(0, QIcon(menuIcon))
                                          if libelleSubDicts[ucase][0] in dicts : keywordItem.setText(0, "%s" % (dicts[libelleSubDicts[ucase][0]]))
                                          if libelleSubDicts[ucase][2] in dicts : keywordItem.setToolTip(0,  "%s" % (dicts[libelleSubDicts[ucase][2]]))
                                          
                                       else :
                                          if keywordItem != None and libelleSubDicts[ucase][i] in dicts :    
                                             keywordItemMeta = QTreeWidgetItem()

                                             if libelleSubDicts[ucase][i] in ("language", "source") :
                                                if libelleSubDicts[ucase][i]== "language" : keywordItemMeta.setIcon(0, QIcon(menuIconLanguage))
                                                else : keywordItemMeta.setIcon(0, QIcon(menuIconSource))
                                                keywordItemMeta.setText(0, "%s : %s" % (QApplication.translate("QSphere", libelleSubDicts[ucase][i]), dicts[libelleSubDicts[ucase][i]]))
                                             else :
                                                keywordItemMeta.setIcon(0, QIcon(menuIconMetadata))    
                                                keywordItemMeta.setText(0, "%s" % (dicts[libelleSubDicts[ucase][i]]))
                                             keywordItem.addChild(keywordItemMeta)
                                   self.results_items.addChild(keywordItem)
                                   self.countkeywords+= 1
                            
              else :
                self.MakeMsgNotValid(zUrl)
 
              self.refreshCounter(self.countkeywords)
              self.stopMovie()
      
              return isValidURL

       def MakeMsgNotValid(self, zUrl):
           from qsphere.qsphere_tools import SendMessage  
           zTitle = QApplication.translate("QSphere", "Warning" ) 
           zMsg = QApplication.translate("QSphere", "Not valid")
           zPicto = QgsMessageBar.WARNING
           zDuration = self.parent.duration_warning
           zMsg = "<b><u>%s</u></b><br><i>%s</i>." % (zUrl, zMsg)
           SendMessage(self, zTitle, zMsg, zPicto, zDuration)

       def getCapabilities(self): self.testURL(0)
       def getSuggestion(self) : self.testURL(1)

       def addKeyword(self, Item):
           from qsphere.qsphere_tools import SendMessage  
           if Item == None : return

           if Item.toolTip(0)=="":
              language =  "%s : " % (QApplication.translate("QSphere", "language"))
              source =  "%s : " % (QApplication.translate("QSphere", "source"))
              if not self.allChildsActive.isChecked():
                 if not (Item.text(0).startswith(language) or Item.text(0).startswith(source)) : self.parent.loadNavigator(Item.text(0)) 
              else :   
                 if Item.text(0).startswith(language) : zUrl = "https://www.google.fr/?gws_rd=ssl#q=%s+language+wiki" % (Item.text(0).replace(language,""))
                 elif Item.text(0).startswith(source) : zUrl = "https://www.google.fr/?gws_rd=ssl#q=%s+thesaurus+wiki" % (Item.text(0).replace(source,""))
                 else : zUrl = Item.text(0)
                 self.parent.loadNavigator(zUrl)
              return

           dialogMDD = None
           
           if self.parent.mdi.subWindowList(QMdiArea.CreationOrder)!= [] :
              dialog = self.parent.mdi.activeSubWindow()
              if dialog == None : return
              from qsphere.doUI import DialogMetadata
              for child in dialog.children():
                   if type(child)==DialogMetadata :
                      dialogMDD = child
                      break  
           else :
                 zTitle = QApplication.translate("QSphere", "Warning" ) 
                 zMsg = QApplication.translate("QSphere","No Windows")
                 zPicto = QgsMessageBar.WARNING
                 zDuration = self.parent.duration_warning
                 SendMessage(self, zTitle, "%s !" % (zMsg), zPicto, zDuration)
                 return


           if dialogMDD == None :
                 zTitle = QApplication.translate("QSphere", "Warning" ) 
                 zMsg = QApplication.translate("QSphere","No Metadata Windows")
                 zPicto = QgsMessageBar.WARNING
                 zDuration = self.parent.duration_warning
                 SendMessage(self, zTitle, "%s !" % (zMsg), zPicto, zDuration)
                 return

           zState = Item.isExpanded()          

           zCommand = dialogMDD.findChild(QAction,"%s_6_%s" % ("Ajouter", "tablemotsclefsf"))
           if zCommand != None : zCommand.triggered.emit(True)

           zObj = dialogMDD.findChild(MyTableWidget,"tablemotsclefsf")
           if zObj != None :

              if self.parent.tabWidget.currentIndex()!=1 : self.parent.tabWidget.setCurrentIndex(1)
              if dialogMDD.tabWidget.currentIndex()!= 1 : dialogMDD.tabWidget.setCurrentIndex(1)
                
              ObjWidget = zObj.cellWidget(zObj.rowCount()-1, 0)
              ObjWidget.setText(Item.text(0))
              ObjWidget = zObj.cellWidget(zObj.rowCount()-1, 1)
              ObjWidget.setChecked(True)
              ObjWidget = zObj.cellWidget(zObj.rowCount()-1, 2)
              zTextSource = Item.child(2).text(0)
              zTextToReplace = "%s : " % (QApplication.translate("QSphere", "source"))
              ObjWidget.setText(zTextSource.replace(zTextToReplace, ""))
              ObjWidget = zObj.cellWidget(zObj.rowCount()-1, 3)
              ObjWidget.setText(datetime.now().replace(microsecond=0).strftime("%Y-%m-%d"))
              zObj.selectRow(zObj.rowCount()-1)
              dialogMDD.passChangeMode()
              
           Item.setExpanded(not zState)


       def closeme(self):
           self.parent.dialogLUSTRE = None   
           self.close()

    
class toolSearchFilesDialog(QDialog):
      def  __init__(self, parent):
            QDialog.__init__(self)
            self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Search files")))
            self._W, self._H = 800, 800
            self.parent = parent
            self.iface = self.parent.iface
            self.setMinimumSize(self._W,self._H)
            self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
            layout = QVBoxLayout(self)

            from qsphere.qsphere_tools import getThemeIcon, LoadSimpleData, makeGetOptions, MakePropertiesForWindow, MakeWindowIcon, html

            self.initDirSearchFiles = ""
            self.listFiltersSearch = LoadSimpleData(self, "file:/ressources/filters_search.csv") 
            if self.listFiltersSearch == [] : self.listFiltersSearch = ["*.csv", "*.htm*", "*.odp", "*.pdf", "*.qsp", "*.qsr", "*.xml"]
            MakeWindowIcon(self, "toolssearchfiles.png")
            makeGetOptions(self)
            MakePropertiesForWindow(self, self.parent)

            self.html = html
            self.lastSavedFileQSR = ""

            self.labelPath = QLabel(self)
            self.labelPath.setAlignment(Qt.AlignRight)
            self.labelPath.setObjectName("labelPath")
            self.labelPath.setGeometry(10, 10, 45, 25)
            self.labelPath.setText("%s :" % QApplication.translate("QSphere", "Folder"))

            self.txtPath = QComboBox(self)
            self.txtPath.setEditable(False)
            self.txtPath.setEnabled(True)
            self.txtPath.setObjectName("txtPath")
            self.txtPath.setAccessibleName("txtPath")

            self.labelFilters = QLabel(self)
            self.labelFilters.setText("%s :" % QApplication.translate("QSphere", "Pattern"))
            self.labelFilters.setAlignment(Qt.AlignRight)
            self.labelFilters.setGeometry(10, 40, 45, 25)

            self.saveFilters = MyPushButton(self)
            self.saveFilters.initPushButton(24, 24, 50, 35, "saveFilters", "", QApplication.translate("QSphere", "Save filters"), True, getThemeIcon("fastsave.png"), 24, 24, True)

            self.Filters = QComboBox(self)
            self.listFiltersSearch = LoadSimpleData(self, "file:/ressources/filters_search.csv") 
            if self.listFiltersSearch == [] : self.listFiltersSearch = ["*.csv", "*.htm*", "*.odp", "*.pdf", "*.qsp", "*.qsr", "*.xml"]

            self.Filters.addItems(self.listFiltersSearch)
            self.Filters.setCurrentIndex(0)
            self.Filters.setEditable(True)
            self.Filters.setInsertPolicy(QComboBox.InsertAlphabetically)

            self.labelOptions = QLabel(self)
            self.labelOptions.setText("%s :" % QApplication.translate("QSphere", "Options"))
            self.labelOptions.setAlignment(Qt.AlignRight)
            self.labelOptions.setGeometry(10, 70, 45, 25)


            self.chkSensibleCase = QCheckBox(self)
            self.chkSensibleCase.setObjectName("chkSensibleCase")
            self.chkSensibleCase.setText("aaa = AAA")
            self.chkSensibleCase.setToolTip(QApplication.translate("QSphere", "Case sensitive"))
            self.chkSensibleCase.setChecked(False)

            self.chkIncludePath = QCheckBox(self)
            self.chkIncludePath.setObjectName("chkIncludePath")
            self.chkIncludePath.setText(QApplication.translate("QSphere", "Include path"))
            self.chkIncludePath.setToolTip(QApplication.translate("QSphere", "Include path for filtering"))
            self.chkIncludePath.setChecked(False)

            self.labelSearchFILE = QLabel(self)
            self.labelSearchFILE.setAlignment(Qt.AlignCenter)
            self.labelSearchFILE.setObjectName("labelSearchFILE")
            self.labelSearchFILE.setAccessibleName("labelSearchFILE")
            self.labelSearchFILE.setText("")


            self.LoadPath = MyPushButton(self)
            self.LoadPath.initPushButton(24, 24, 220, 27, "ChoosePath", "", QApplication.translate("QSphere", "Choose path ..."), True, getThemeIcon("folder.png"), 24, 24, True)

            self.SearchFiles = MyPushButton(self)
            self.SearchFiles.initPushButton(64, 64, 220, 55, "SearchFiles", "", QApplication.translate("QSphere", "Search files in the path ..."), True, getThemeIcon("searchfiles.png"), 64, 64, True)


            self.TabViewResults = QTabWidget(self)
            self.TabViewResults.setIconSize(QSize(32, 32))

            self.listFiles = QListWidget(self)
            self.listFiles.setObjectName("listFiles")
            self.listFiles.setIconSize(QSize(32, 32))
            zIcon = getThemeIcon("liste.png")
            self.TabViewResults.addTab(self.listFiles, QIcon(zIcon), QApplication.translate("QSphere","List"))

            self.treeFiles = QTreeWidget(self)
            self.treeFiles.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
            self.treeFiles.setObjectName("treeFiles")
            self.treeFiles.headerItem().setText(0, "1")
            self.treeFiles.header().setVisible(False)
            self.treeFiles.header().setDefaultSectionSize(200)
            self.treeFiles.setStyleSheet("* { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                     "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")        
            self.treeFiles.setVisible(True)
            zIcon = getThemeIcon("folder.png")
            self.TabViewResults.addTab(self.treeFiles, QIcon(zIcon), QApplication.translate("QSphere","Organized view"))

            self.labelResultFiles = QLabel(self)
            self.labelResultFiles.setAlignment(Qt.AlignLeft)
            self.labelResultFiles.setObjectName("labelResultFiles")
            self.labelResultFiles.setText("")


            self.infosFile = QTextBrowser(self.TabViewResults)
            self.infosFile.setStyleSheet("QTextBrowser {font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black }")
            self.printInfosFile = MyPushButton(self.infosFile)
            self.printInfosFile.initPushButton(32, 32, 340, 10, "printInfosFile", "", QApplication.translate("QSphere","Print"), True, getThemeIcon("print.png"), 32, 32, True)
            self.printInfosFile.setEnabled(False)

            self.movie = QMovie(getThemeIcon("sablier.gif"))
            self.counter = self.movie.frameCount()

            self.status_txt = QLabel(self)
            self.status_txt.setMovie(self.movie)
            self.status_txt.setLayout(QHBoxLayout())
            self.status_txt.layout().addWidget(QLabel(''))
            self.status_txt.setVisible(False)

            self.barInfo = QgsMessageBar(self)
            self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )


            self.toolBar = QToolBar(self)
            self.toolBar.setObjectName("toolBar")
            self.toolBar.setGeometry(240, 100, 360, 40)
            self.toolBar.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
            self.toolBar.setWindowTitle(QApplication.translate("QSphere","Results actions"))
            self.actionsForToolBar(self.toolBar)

            self.toolBar.setStyleSheet("QToolBar {icon-size: 48px, 48px; background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);  border-width: 2px; border-radius: 10px; border-color: beige;}")
                                        
            self.LoadPath.clicked.connect(self.fixePath)
            self.SearchFiles.clicked.connect(self.makeListFiles)
            self.TabViewResults.currentChanged.connect(self.activeConnect)
            self.printInfosFile.clicked.connect(self.printInfosForTheFile)

            self.treeFiles.clear()
            self.initTreeFiles()
            self.activeConnect(0)

            self.listFiles.itemDoubleClicked.connect(self.openFile)
            self.treeFiles.itemDoubleClicked.connect(self.openFile)

            self.treeFiles.expandAll()
            self.FixeEnabled()

      def actionsForToolBar(self, zButtonAction):
          from qsphere.qsphere_tools import getThemeIcon

          menuIcon = getThemeIcon("open.png")
          self.loadSearchFiles = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Load search files results"), self)
          self.loadSearchFiles.setObjectName("loadSearchFiles")
          self.loadSearchFiles.setShortcut(QKeySequence("Ctrl+O"))
          zButtonAction.addAction(self.loadSearchFiles)
          self.loadSearchFiles.triggered.connect(self.loadResultsFiles)

          menuIcon = getThemeIcon("save.png")
          self.saveSearchFiles = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Save search files results"), self)
          self.saveSearchFiles.setObjectName("saveSearchFiles")
          self.saveSearchFiles.setShortcut(QKeySequence("Ctrl+S"))
          zButtonAction.addAction(self.saveSearchFiles)
          self.saveSearchFiles.triggered.connect(self.saveResultsFiles)

          zButtonAction.addSeparator()

          menuIcon = getThemeIcon("tree_expand.png")
          self.treeWidgetExpand = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Expand all nodes"), self)
          self.treeWidgetExpand.setObjectName("treeWidgetExpand")
          self.treeWidgetExpand.setShortcut(Qt.Key_Plus) 
          zButtonAction.addAction(self.treeWidgetExpand)
          self.treeWidgetExpand.triggered.connect(self.expandTreeWidget)

          menuIcon = getThemeIcon("tree_reduce.png")
          self.treeWidgetReduce = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Reduce all nodes"), self)
          self.treeWidgetReduce.setObjectName("treeWidgetReduce")
          self.treeWidgetReduce.setShortcut(Qt.Key_Minus)
          zButtonAction.addAction(self.treeWidgetReduce)
          self.treeWidgetReduce.triggered.connect(self.expandTreeWidget)


          zButtonAction.addSeparator()

          menuIcon = getThemeIcon("search.png")
          self.findInListResults = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Find in results"), self)
          self.findInListResults.setObjectName("findInListResults")
          self.findInListResults.setShortcut(QKeySequence("Ctrl+Shift+F"))
          zButtonAction.addAction(self.findInListResults)
          self.findInListResults.triggered.connect(self.findInResults)


          zButtonAction.addSeparator()

          menuIcon = getThemeIcon("print.png")
          self.printListResults = QAction(QIcon(menuIcon), QApplication.translate("QSphere", "Print search files results"), self)
          self.printListResults.setObjectName("printListResults")
          self.printListResults.setShortcut(QKeySequence("Ctrl+Shift+P"))
          zButtonAction.addAction(self.printListResults)
          self.printListResults.triggered.connect(self.printResults)
             

      def resizeEvent(self, ev):
          zSize = ev.size()

          self.barInfo.setGeometry(0, 0, self.width(), 45)  
          self.status_txt.setGeometry(self.width()/2-32, self.height()/2-32, 64, 64)

          self.txtPath.setGeometry(75, 5, self.width()-195, 25)
          self.LoadPath.setGeometry(self.txtPath.x()+self.txtPath.width()+10, 5, 24, 24)
          self.Filters.setGeometry(75, 35, self.width()-195, 25)
          self.saveFilters.setGeometry(self.Filters.x()+self.Filters.width()+10, 35, 24, 24)
          
          self.chkSensibleCase.setGeometry(self.Filters.x(), 65, 120, 25)
          self.chkIncludePath.setGeometry(self.chkSensibleCase.x()+self.chkSensibleCase.width()+10, 65, 120, 25)
          
          self.SearchFiles.setGeometry(self.width()-70, 5, 64, 64)

          
          self.toolBar.setGeometry(self.width()/2-130, 100, 360, 40)

          self.labelSearchFILE.setGeometry(5, 80, self.width()-10, 25)
          self.TabViewResults.setGeometry(10, 100, self.width()-20, self.height()-115)
          self.listFiles.setGeometry(5, 5, self.TabViewResults.width()-10, self.TabViewResults.height()-40)
          self.treeFiles.setGeometry(5, 5, self.TabViewResults.width()-10, self.TabViewResults.height()-40)
          self.infosFile.setGeometry(self.TabViewResults.width()-430, 55, 400, 440)


      def closeEvent(self,event):
          self.parent.dialogSearchFiles = None
          self.parent.windowsNoMDIChilds.remove(self)

      def startMovie(self):
          self.status_txt.setVisible(True)
          self.movie.start()
          self.status_txt.repaint()

      def stopMovie(self):
          self.movie.stop()
          self.status_txt.setVisible(False)


      def fixePath(self):
          from qsphere.qsphere_tools import FixeLabelsFileDialog 
          from os.path import (basename, dirname, exists, splitext)
          
          InitDir = dirname(__file__) if self.txtPath.currentText()=="" else dirname("%s" % (self.txtPath.currentText()))
          zTitle = QApplication.translate("QSphere", "Select the folder to search files")
          MyFileDialog = QFileDialog(self, zTitle)
          MyFileDialog.setDirectory(InitDir)
          MyFileDialog.setFileMode(QFileDialog.DirectoryOnly)

          FixeLabelsFileDialog(self, MyFileDialog, 0, False)          
          
          if MyFileDialog.exec_():
             inputDir = MyFileDialog.selectedFiles()[0] 
             zCible = "%s/" % (inputDir.replace("\\","/"))
             if self.txtPath.findText(zCible)==-1 : self.txtPath.addItem("%s" % (zCible))
             self.txtPath.setCurrentIndex(self.txtPath.model().rowCount()-1)
             self.initDir = inputDir
          self.FixeEnabled()

      def initTreeFiles(self):
          self.layers_item = None
          self.layers_item = QTreeWidgetItem()
          self.layers_item.setText(0, QApplication.translate("QSphere", "Results"))

          self.treeFiles.addTopLevelItem(self.layers_item)
          self.treeFiles.expandAll()
          self.treeFiles.resizeColumnToContents(0)
          self.treeFiles.resizeColumnToContents(1)

      def FixeEnabled(self):
          self.txtPath.setEnabled(False) if self.txtPath.count()<=0 else self.txtPath.setEnabled(True)
          self.SearchFiles.setEnabled(False) if self.txtPath.currentText()=="" else self.SearchFiles.setEnabled(True)
          self.saveSearchFiles.setEnabled(False) if self.txtPath.currentText()=="" else self.saveSearchFiles.setEnabled(True)


      def makeListFiles(self):
          from os.path import exists
          from qsphere.qsphere_tools import SendMessage
          if not exists(self.txtPath.currentText()) :
                 zTitle = QApplication.translate("QSphere","Information")
                 zMsg = QApplication.translate("QSphere","Error")
                 zMsg1 = QApplication.translate("QSphere","folder")
                 SendMessage(self, zTitle , "%s %s :<br>%s" % (zMsg, zMsg1, self.txtPath.currentText()), QgsMessageBar.WARNING, self.duration_warning)
                 self.txtPath.removeItem(self.txtPath.currentIndex())
                 self.countfiles, self.countdirs = 0, 0
                 self.refreshCounter(self.countfiles, self.countdirs)        
                 try : self.txtPath.setCurrentIndex(0)
                 except : pass
                 self.FixeEnabled()
                 return 

          self.treeFiles.clear()
          self.initTreeFiles()
                
          self.countfiles, self.countdirs = 0, 1
          self.refreshCounter(self.countfiles, self.countdirs)        
          self.filter = QRegExp("*%s" % (self.Filters.currentText()))
          self.filter.setCaseSensitivity(Qt.CaseInsensitive) if not self.chkSensibleCase.isChecked() else self.filter.setCaseSensitivity(Qt.CaseSensitive)
          self.filter.setPatternSyntax(QRegExp.Wildcard)
          self.listFiles.clear()
          self.listFiles.update()
          self.listFiles.setEnabled(False)
          self.startMovie()

          from qsphere.qsphere_tools import listdirectoryWeb
          
          self.countfiles, self.countdirs = listdirectoryWeb(self, self.txtPath.currentText(), self.listFiles, self.counter, self.chkIncludePath.isChecked())
          self.stopMovie()
          if self.listFiles.count()> 0 :
             self.listFiles.setCurrentRow(0)
             self.loadFile(self.listFiles.currentItem())
          self.refreshCounter(self.countfiles, self.countdirs)
          self.movie.stop()   
          self.listFiles.update()
          self.listFiles.setEnabled(True)

      def refreshCounter(self, countfiles, countdirs):
         sFile = QApplication.translate("QSphere", "files") if self.countfiles > 1 else QApplication.translate("QSphere", "file") 
         sFolder = QApplication.translate("QSphere", "folders") if self.countdirs > 1 else QApplication.translate("QSphere", "folder")
         self.labelSearchFILE.setText("%s %s %s %s %s." % (countfiles, sFile, \
                                                      QApplication.translate("QSphere", "in"), \
                                                      countdirs, sFolder))   


      def activeConnect(self, index):
          if index == 0 :
            try :
                self.treeFiles.itemClicked.disconnect(self.loadFile)
                self.treeFiles.currentItemChanged.disconnect(self.loadFile)
            except : pass
            self.listFiles.itemClicked.connect(self.loadFile)
            self.listFiles.currentItemChanged.connect(self.loadFile)
          elif index == 1 :
            try :
                self.listFiles.itemClicked.disconnect(self.loadFile)
                self.listFiles.currentItemChanged.disconnect(self.loadFile)
            except : pass
            self.treeFiles.itemClicked.connect(self.loadFile)
            self.treeFiles.currentItemChanged.connect(self.loadFile)     


      def loadFile(self, Item):
          fileName = ""
          if self.sender() == self.listFiles :
            if self.listFiles.count()==0 : return
            if Item == None : return
            if Item.type()==1 :
                 self.infosFile.setText("")
                 self.printInfosFile.setEnabled(False)
                 return
            if not Item : return
            fileName = Item.toolTip()
          elif self.sender() == self.treeFiles :
            if Item == None : return
            if Item.toolTip(0)=="":
                 self.infosFile.setText("")
                 self.printInfosFile.setEnabled(False)
                 return
            fileName = Item.toolTip(0)
          if fileName == "": return
          self.synchronize(self.sender())

          from qsphere.qsphere_tools import MakeFileInfosToHTML     
          infosHtml = MakeFileInfosToHTML(self, QFileInfo(fileName))
          self.infosFile.setText(self.html % (infosHtml))
          self.printInfosFile.setEnabled(True)


      def synchronize(self, zSender):
          if zSender  == self.treeFiles :
            zList = literal_eval(zSender.currentItem().whatsThis(0))
            Elt = int(zList[0])
            Path = zList[1]
            self.listFiles.setCurrentRow(Elt)
          elif zSender == self.listFiles :
            zList = literal_eval(zSender.currentItem().whatsThis())
            Elt = zList[0]#.encode('utf-8')
            Path = zList[1]
            if Elt == "" : return
            zItems = self.treeFiles.findItems("%s" % Elt, Qt.MatchFixedString | Qt.MatchRecursive)
            if zItems != [] :
               for item in zItems :
                   zList = literal_eval(item.whatsThis(0))
                   pathTarget = zList[1]
                   if pathTarget ==  Path : 
                      self.treeFiles.setCurrentItem(item)
                      break
          else : return 

      def openFile(self, Item):
             from os.path import (basename, dirname, exists, splitext)
             from qsphere.qsphere_tools import SendMessage
             fileName = ""
             if self.sender() == self.listFiles :
                 if self.listFiles.count()==0 : return
                 if Item == None : return
                 if Item.type()==1 : return
                 if not Item : return
                 fileName = Item.toolTip()
             elif self.sender() == self.treeFiles :
                 if Item == None : return
                 if Item.toolTip(0)=="": return
                 fileName = Item.toolTip(0)
             if fileName == "": return

             if exists(fileName) :
                 zFile = QFileInfo(fileName)
                 extension = zFile.suffix().lower()

                 if extension == "xml" :  self.parent.loadNavigator("filexml:%s" % fileName)
                 elif extension == "gbmeta" :  self.parent.loadNavigator("filexml:%s.xml" % fileName)
                 elif extension == "qsp" :  self.parent.loadNavigator("file:%s" % fileName)
                 elif extension in ("qsr", "csv", "gif", "htm", "html", "jpeg", "jpg", "png", "qsp") : self.parent.loadNavigator("file:%s" % fileName)
                 else :      
                     try : webbrowseropen(fileName)
                     except :
                            zTitle = QApplication.translate("QSphere","Information")
                            zMsg = QApplication.translate("QSphere","Try to open Web navigator for https protocol ...")
                            SendMessage(self, zTitle , zMsg, QgsMessageBar.WARNING, 3)
                     


      def expandTreeWidget(self):
          if self.sender() == self.treeWidgetExpand : self.treeFiles.expandAll()
          else :
             self.treeFiles.collapseAll()
             self.treeFiles.expandItem(self.treeFiles.itemAt(0, 0))

      def findInResults(self):
          self.findDialog = FindDialogToolSearchFiles(self)
          ret = self.findDialog.exec_()
          self.findDialog.lineEdit.setFocus()
          if self.findDialog.ret > 0 : pass


      def saveResultsFiles(self):
        from qsphere.qsphere_tools import (CorrigePath, FileNameWithExtension, FixeLabelsFileDialog, SendMessage)
        from os.path import (basename, dirname, exists, splitext)

        InitDir = dirname(__file__) if self.initDirSearchFiles == "" else self.initDirSearchFiles
        InitDir = CorrigePath(InitDir)
        zTitle = QApplication.translate("QSphere","Save search files results")
        MyFileDialog = QFileDialog(self, zTitle)
        zElt0 = QApplication.translate("QSphere","files").title()
        MyFileDialog.setNameFilters(("%s QSphere (*.qsr)" % (zElt0),)) 
        MyFileDialog.setDefaultSuffix("qsr")
        MyFileDialog.setViewMode(QFileDialog.Detail)
        MyFileDialog.setAcceptMode(QFileDialog.AcceptSave)

        FixeLabelsFileDialog(self, MyFileDialog, 1, True)

        self.lastSavedFileQSR
        if self.lastSavedFileQSR != "" :
             if exists(self.lastSavedFileQSR):
                zFile = QFileInfo(self.lastSavedFileQSR)
                if zFile.suffix().lower() == "qsr" :
                   InitName = basename(self.lastSavedFileQSR).split(".")[0]
                   InitDir = dirname(self.lastSavedFileQSR)
                   MyFileDialog.selectFile(InitName)
              
        MyFileDialog.setDirectory(InitDir)
        
        if MyFileDialog.exec_():
            fileName = FileNameWithExtension(self, MyFileDialog.selectedFiles()[0], MyFileDialog.selectedNameFilter())
            self.initDirSearchFiles = dirname(fileName)
            
            zLOG = open(fileName, "w")
            Config = ConfigParser()
            NameSection = "QSphere Search Results"
            Config.add_section(NameSection)
            Config.set(NameSection,'date', datetime.now().strftime("%d/%m/%Y %Hh%Mm%Ss"))
            Config.set(NameSection,'mandatoryPlugin',"QSphere")
            Config.set(NameSection,'version', self.version)

            NameSection = "Root"
            Config.add_section(NameSection)
            Config.set(NameSection, 'Path', self.txtPath.currentText())
            Config.set(NameSection, 'filtre', self.Filters.currentText())
            Config.set(NameSection, 'files', self.countfiles)
            counter=0
            for i in range(self.listFiles.count()):
                if self.listFiles.item(i).type()==0 :
                    Config.set(NameSection,'File_%s' % (counter),self.listFiles.item(i).toolTip().replace("\\","/").encode("cp1252"))
                    counter+= 1                                 
                elif self.listFiles.item(i).type()==1 :
                    NameSection = self.listFiles.item(i).toolTip().encode("cp1252")
                    Config.add_section(NameSection)
        
            Config.write(zLOG)
            if zLOG != None : zLOG.close()

            zTitle = QApplication.translate("QSphere","Save as a file") 
            zMsg = QApplication.translate("QSphere","The metadata record was saved as")
            SendMessage(self, zTitle , "%s :<br><u><i>%s</i></ul>" % (zMsg, fileName), QgsMessageBar.INFO, self.duration_info)        


      def loadResultsFiles(self):
        from qsphere.qsphere_tools import (CorrigePath, DicoHasKey, FileNameWithExtension,
                                   FixeLabelsFileDialog, getThemeIcon, SendMessage)
        from os.path import (basename, dirname, exists, splitext)
        
        zTitle = QApplication.translate("QSphere","Load search files results")
        InitDir = dirname(__file__) if self.initDirSearchFiles == "" else self.initDirSearchFiles
        MyFileDialog = QFileDialog(self, zTitle)
        zElt0 = QApplication.translate("QSphere","files").title()        
        MyFileDialog.setNameFilters(("%s QSphere (*.qsr)" % (zElt0),)) 
        MyFileDialog.setViewMode(QFileDialog.Detail)
        MyFileDialog.setDirectory(InitDir)
        MyFileDialog.setFileMode(QFileDialog.ExistingFile) 
        MyFileDialog.setAcceptMode(QFileDialog.AcceptOpen)

        FixeLabelsFileDialog(self, MyFileDialog, 0, True)
        
        if MyFileDialog.exec_():
            fileName = "%s" % (MyFileDialog.selectedFiles()[0])
            if fileName!="" :

               self.startMovie()
               
               self.initDirSearchFiles = dirname(fileName)
               
               config = ConfigParser()
               config.read(fileName)
               zSections = config.sections()

               fontDirs = QFont()
               fontDirs.setPointSize(8) 
               fontDirs.setWeight(10) 
               fontDirs.setBold(True)

               self.countfiles, self.countdirs = 0, 1

               self.listFiles.clear()
               self.listFiles.update()

               self.treeFiles.clear()
               self.initTreeFiles()
               self.infosFile.setText("")
               self.printInfosFile.setEnabled(False)
               
               menuIcon = getThemeIcon("mime/file/xml.png")
               icondir = getThemeIcon("open.png")
               pos, countfiles, countdirs, oldirs = 0, 0, 1, []
               position = 0

               for section in zSections :
                   if section == "Root" :
                       zPath = config.get(section,'path')
                       if self.txtPath.findText(zPath) == -1: self.txtPath.addItem(zPath)
                       else : self.txtPath.setCurrentIndex(self.txtPath.findText(zPath))

                       self.listdirectories = {}
                       self.listdirectories["%s" % (zPath)]= (self.layers_item, [])
                       groupItems = self.layers_item
                       refpath = zPath
                       
                       zFiltre = config.get(section,'filtre')

                       if zFiltre not in self.listFiltersSearch :
                           self.listFiltersSearch.append(zFiltre)
                           self.Filters.addItem(zFiltre)
                           
                       index = self.Filters.findText(zFiltre)
                       
                       if index < 0 : index = 0
                       menuIcon = getThemeIcon("mime/file/%s.png" % (zFiltre.replace("*","").replace(".","")))
                       if menuIcon == "" : menuIcon = getThemeIcon("mime/file/unknow.png")
                       self.Filters.setCurrentIndex(index)
                       self.countfiles = int(config.get(section,'files'))

                   if not section in ("QSphere Search Results", "Root") :
                      if exists("%s%s/" % (zPath,section)):
                            zItem = QListWidgetItem(QIcon(icondir), "%s" % (section), None, 1)
                            zItem.setFont(fontDirs)
                            zItem.setBackground(QBrush(QColor(200,200,200,125),Qt.SolidPattern))
                            zItem.setFlags(Qt.ItemIsEnabled)
                            zItem.setToolTip(section)
                            zItem.setWhatsThis('("%s", "%s")' % (position, "%s%s/" % (zPath,section)))
                            self.listFiles.addItem(zItem)
                            position+= 1  
                            countdirs+= 1

                            groupItems = self.layers_item
                            abspath = dirname("%s%s/" % (zPath, section))
                            tpath = abspath.split("/")
                            dirNameRoot = "%s/" % (abspath.replace(zPath, ""))
                            tdirname = dirNameRoot.split("/")
                            compath = ""

                              
                            for  i in range(len(tdirname)):
                                if tdirname[i]!= "" :
                                    compath+=  tdirname[i]+"/"
                                    zkey = ("%s%s" % (refpath, compath))
                                    if not DicoHasKey(self.listdirectories, zkey) :
                                        groupItemsSub = QTreeWidgetItem()
                                        groupItemsSub.setIcon(0, QIcon(icondir))
                                        groupItemsSub.setText(0, "%s" % (tdirname[i]))
                                        groupItemsSub.setToolTip(0, "")
                                        groupItems.addChild(groupItemsSub)
                                        self.listdirectories[zkey] = (groupItemsSub, [])
                                        groupItems = groupItemsSub
                                    else:
                                        groupItems = self.listdirectories[zkey][0]
                        
                      else : oldirs.append("%s%s/" % (zPath,section))
                            
                            
                   if section != "QSphere Search Results" :
                       c = 0
                       for i in range(pos, self.countfiles, 1):
                           try : zFile = config.get(section,'file_%s' % (i))
                           except : break

                           if exists(zFile):
                               zItem = QListWidgetItem(QIcon(menuIcon), "%s" % (basename(zFile)), None, 0)
                               zItem.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
                               
                               zItem.setToolTip("%s" % (zFile))
                               zItem.setWhatsThis('("%s", "%s")' % (basename(zFile), dirname(zFile)))
                               self.listFiles.addItem(zItem)
                               countfiles+= 1

                               if groupItems!= None :
                                  FileItem = QTreeWidgetItem()
                                  FileItem.setIcon(0, QIcon(menuIcon))
                                  FileItem.setText(0, "%s" % (basename(zFile)))
                                  FileItem.setToolTip(0, zFile)
                                  FileItem.setWhatsThis(0, '("%s", "%s")' % (position, dirname(zFile)))
                                  groupItems.addChild(FileItem)
                                  position+= 1
                               
                           pos+= 1
                           c = c+1 if c < (self.counter-1) else 0
                           self.movie.jumpToFrame(c)
                           self.status_txt.repaint()
                           
                   self.refreshCounter(self.countfiles, countdirs)

               self.countdirs = countdirs - len(oldirs)

               if (self.countfiles != countfiles) or (self.countdirs != countdirs) :    
                   zTitle = QApplication.translate("QSphere","Warning")
                   zEltFiles = QApplication.translate("QSphere","files").title()
                   zEltIn = QApplication.translate("QSphere","in").title()
                   zEltFolders = QApplication.translate("QSphere","folders").title()
                   zEltReload = QApplication.translate("QSphere","Read from download stream ")
                   zEltFile = QApplication.translate("QSphere","File")
                   
                   SendMessage(self, zTitle , "%s ... : %s %s %s %s %s<br>%s : %s %s %s %s %s<br>%s" % (zEltFile, self.countfiles, zEltFiles, zEltIn, self.countdirs, zEltFolders, zEltReload, countfiles, zEltFiles, zEltIn, countdirs, zEltFolders, oldirs), QgsMessageBar.WARNING, self.duration_warning)    

               self.countfiles, self.countdirs = countfiles, countdirs

               self.stopMovie()
               self.refreshCounter(countfiles, countdirs)
               self.FixeEnabled()
               self.lastSavedFileQSR = fileName

               if self.listFiles.count()> 0 :
                  self.listFiles.setCurrentRow(0)
                  self.loadFile(self.listFiles.currentItem())        
            

      def printInfosForTheFile(self):
          from qsphere.qsphere_tools import MakeWindowIcon
                    
          printer = QPrinter() 
          printer.setPageSize(QPrinter.A4)
          printer.setOrientation(QPrinter.Portrait)
          printer.setPageMargins(5, 10, 5, 10, QPrinter.Millimeter)
          printer.setOutputFormat(QPrinter.NativeFormat)
          printDialog = QPrintPreviewDialog(printer)
          MakeWindowIcon(printDialog, "print.png")
          printDialog.setWindowTitle(QApplication.translate("QSphere", "Print informations for the file"))
          printDialog.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
                
          printDialog.paintRequested.connect(self.infosFile.print_)
          printDialog.exec_()


          
      def printResults(self):
          from qsphere.qsphere_tools import getThemeIcon, MakeWindowIcon
                    
          printer = QPrinter() 
          printer.setPageSize(QPrinter.A4)
          printer.setOrientation(QPrinter.Portrait)
          printer.setPageMargins(5, 10, 5, 10, QPrinter.Millimeter)
          printer.setOutputFormat(QPrinter.NativeFormat)
          printDialog = QPrintPreviewDialog(printer)
          MakeWindowIcon(printDialog, "print.png")
          printDialog.setWindowTitle(QApplication.translate("QSphere", "Print list of results"))
          printDialog.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)

          infosResults = QTextBrowser(self)
         
          infosHtml = "<div class='container'>"
          infosHtml+= "<table border=1 cellspacing=0 cellpadding=0 width='100%'>"
          infosHtml+= '<tr><td><img src="%s"></td><td>' % getThemeIcon("searchfiles.png")
          infosHtml+= "<table border=0 cellspacing=0 cellpadding=0 width='100%'>"
          infosHtml+= "<tr><td>%s</td><td><b>%s</b></td></tr>" % ("<u>%s</u> :" % QApplication.translate("QSphere", "Folder"), self.txtPath.currentText())
          infosHtml+= "<tr><td>%s</td><td><b>%s</b></td></tr>" % ("<u>%s</u> :" % QApplication.translate("QSphere", "Filters"), self.Filters.currentText())
          infosHtml+= "<tr><td>%s</td><td><b>%s</b></td></tr>" % ("<u>%s</u> :" % QApplication.translate("QSphere", "Results"), self.labelSearchFILE.text())
          infosHtml+= "</table></td></tr>"
          infosHtml+= "</table><hr>"
          infosHtml+= "<table border=0 cellspacing=0 cellpadding=0 width='100%'>"
          nbsubfolders = 0
          for i in range(self.listFiles.count()):
              if self.listFiles.item(i).type()==0 :
                 try : infosHtml+= "<tr><td>%s : %s</td><td>%s</td></tr>"  % (QApplication.translate("QSphere", "File"), i-nbsubfolders, self.listFiles.item(i).toolTip().encode("cp1252"))
                 except : infosHtml+= "<tr><td>%s : %s</td><td>%s</td></tr>"  % (QApplication.translate("QSphere", "File"), i-nbsubfolders, self.listFiles.item(i).toolTip())
              elif self.listFiles.item(i).type()==1 :
                 infosHtml+= "<tr><td colspan=2><hr></td></tr>"
                 try : infosHtml+= '<tr valign="middle"><td><img src="%s"></td><td><h2><b><i>%s</i></b></h2></td></tr>'  %   (getThemeIcon("toolssearchfiles.png"), self.listFiles.item(i).toolTip().encode("cp1252"))
                 except : infosHtml+= "<tr><td colspan=2><b><i>%s</i></b></td></tr>"  %   self.listFiles.item(i).toolTip()
                 infosHtml+= "<tr><td colspan=2><hr></td></tr>"
                 nbsubfolders+= 1
          infosHtml+= "</table><br></div>"

          infosResults.setText(self.html % (infosHtml))
          printDialog.paintRequested.connect(infosResults.print_)
          printDialog.exec_() 


                         
class LoadFromLayersDialog(QDialog):

       def  __init__(self, iface, parent):
             QDialog.__init__(self)
             self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Load metadata file for layers")))
             self._W, self._H = 600, 500
             self.iface = iface
             self.parent = parent
             self.setMinimumSize(self._W,self._H)
             self.setMaximumSize(self._W*2,self._H*2)
             layout = QVBoxLayout(self)

             self.groupLayersInfo = QGroupBox(self)

             self.suffixe = "qsp"

             from qsphere.qsphere_tools import getThemeIcon
             zIcon = getThemeIcon("layers.png")
             carIcon = QImage(zIcon)
             self.imgQGISLayers = QLabel(self.groupLayersInfo)
             self.imgQGISLayers.setObjectName("imgQGISLayers")
             self.imgQGISLayers.setAlignment(Qt.AlignCenter)
             self.imgQGISLayers.setPixmap(QPixmap.fromImage(carIcon).scaled(48,48))        

             self.labelMDDFilters = QLabel(self.groupLayersInfo)
             self.labelMDDFilters.setObjectName("labelMDDFilters")
             self.labelMDDFilters.setAlignment(Qt.AlignLeft)
             self.labelMDDFilters.setWordWrap(True)
             self.labelMDDFilters.setText("%s :" % QApplication.translate("QSphere","Filters"))

             self.groupBoxObj = QGroupBox(self.groupLayersInfo)
             self.groupBoxObj.setObjectName("groupBoxObj")

             self.radioTypeObj0 = QRadioButton(self.groupBoxObj)
             self.radioTypeObj0.setGeometry(QRect(10,5,70,23))
             self.radioTypeObj0.setObjectName("radioTypeObj0")
             self.radioTypeObj0.setText("QSP")

             self.radioTypeObj1 = QRadioButton(self.groupBoxObj)
             self.radioTypeObj1.setGeometry(QRect(80,5,70,23))
             self.radioTypeObj1.setObjectName("radioTypeObj1")
             self.radioTypeObj1.setText("XML")
             
             self.radioTypeObj2 = QRadioButton(self.groupBoxObj)
             self.radioTypeObj2.setGeometry(QRect(150,5,80,23))
             self.radioTypeObj2.setObjectName("radioTypeObj2")
             self.radioTypeObj2.setText("All")

             self.radioTypeObj0.clicked.connect(self.FixeFilter)
             self.radioTypeObj1.clicked.connect(self.FixeFilter)
             self.radioTypeObj2.clicked.connect(self.FixeFilter)

             self.radioTypeObj0.setChecked(True)

             self.labelQGISLayers = QLabel(self.groupLayersInfo)
             self.labelQGISLayers.setObjectName("labelQGISLayers")
             self.labelQGISLayers.setAlignment(Qt.AlignLeft)
             self.labelQGISLayers.setWordWrap(True)
             self.labelQGISLayers.setText("%s :" % QApplication.translate("QSphere","Layers"))         

             self.listQGISLayers = QListWidget(self.groupLayersInfo)
             self.listQGISLayers.setObjectName("listQGISLayers")
             self.listQGISLayers.setStyleSheet("* { font-size:12px; background-color: #5D5D5D; padding: 4px ; color: rgb(255,255,255)}"
                                               "QListWidget::item:selected {background-color: rgb(255,255,255); color: black;}")
             self.listQGISLayers.setSelectionMode(QAbstractItemView.MultiSelection)
             self.listQGISLayers.setEditTriggers(QAbstractItemView.NoEditTriggers)
             self.listQGISLayers.setIconSize(QSize(80, 26))

             self.FixeFilter() 

             self.buttonBox = QDialogButtonBox()
             self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
             self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
             self.buttonBox.accepted.connect(self.accept)
             self.buttonBox.rejected.connect(self.reject)
             self.listQGISLayers.itemDoubleClicked.connect(self.loadOnDblClick)
             
             layout.addWidget(self.groupLayersInfo) 
             layout.addWidget(self.buttonBox) 
             self.setLayout(layout)


       def loadOnDblClick(self, Item):
            if Item.toolTip()!= "" and Item.toolTip().find("|")>-1 :
                 infos = Item.toolTip().split("|")
                 if len(infos)== 2:
                      fileName, layerid = infos[0], infos[1]
                      if fileName!="" :
                         self.setWindowOpacity(0.5) 
                         self.parent.loadMDDFile(fileName, True, layerid)     
                         self.parent.InitDir = dirname(fileName)
                         self.setWindowOpacity(1.0)

       def FixeFilter(self):
           if self.radioTypeObj1.isChecked() : self.suffixe = ("xml", "meta.xml")
           elif  self.radioTypeObj2.isChecked() : self.suffixe = ("qsp", "xml", "meta.xml")
           else :  self.suffixe = "qsp"
           self.updateListLayers()
                
       def updateListLayers(self):
            from qsphere.qsphere_tools import getThemeIcon 

            font = QFont()
            font.setPointSize(8) 
            font.setWeight(10) 
            font.setBold(True)

            qDefaultIconLayer = QIcon(getThemeIcon("modenone.png"))
            self.listQGISLayers.clear()

            root = QgsProject.instance().layerTreeRoot()
            if root != None :
               for layer in root.children():
                  xmlFiles = self.getLayerXMLorQSP(layer.layer(), "", self.suffixe)
                  if xmlFiles != [] :
                       for xmlFile in xmlFiles :
                            qIconLayer = QIcon(getThemeIcon("mode%s.png" % xmlFile[1]))
                            zItem = QListWidgetItem(qIconLayer, "%s" % (fill(layer.name(), 50)), None, 1)
                            zItem.setFont(font)
                            zItem.setBackground(QBrush(QColor(0,255,0,125),Qt.SolidPattern))
                            zItem.setToolTip("%s|%s" % (xmlFile[0], xmlFile[2]))
                            self.listQGISLayers.addItem(zItem)
                  else :
                       zItem = QListWidgetItem(qDefaultIconLayer, "%s" % (fill(layer.layer().name(), 50)), None, 1)
                       zItem.setFlags(Qt.ItemIsEnabled)
                       zItem.setBackground(QBrush(QColor(255,0,0,125),Qt.SolidPattern))
                       zItem.setToolTip("")
                       self.listQGISLayers.addItem(zItem)

       def getLayerXMLorQSP(self, layer, WithPrefix, WithSuffix):
           from qsphere.qsphere_tools import CorrigePath
           from os.path import (basename, dirname, exists, splitext)
           
           xmlFiles = [] 
           if layer != None :
               fileName = layer.source()
               layerid = layer.id()
               fileDir = CorrigePath(dirname(fileName))
               fileOnlyName = basename(fileName)
               textension = splitext(fileName)
               extension = textension[len(textension)-1].lower()

               if type(WithSuffix)== str:
                  baseName = "%s.%s" % (textension[0], WithSuffix)
                  if exists(baseName): xmlFiles.append((baseName, WithSuffix, layerid))
               elif type(WithSuffix)== tuple:
                  FileNameWithoutExt = splitext(fileOnlyName)[0]
                  for suffix in WithSuffix :
                      basenames = ("%s.%s" % (textension[0], suffix), \
                                   "%smd_%s.%s" % (fileDir, FileNameWithoutExt, suffix), \
                                   "%s%s.qsp.%s" % (fileDir, FileNameWithoutExt, suffix), \
                                   "%smd_%s.qsp.%s" % (fileDir, FileNameWithoutExt, suffix))

                      for elt in basenames :
                          if exists(elt): xmlFiles.append((elt, suffix, layerid))
           return xmlFiles

       def qgisLayersList(self):
         return [item.toolTip() for item in self.listQGISLayers.selectedItems()]


       def resizeEvent(self,ev):
         zSize = ev.size()

         self.groupLayersInfo.setGeometry(10, 10, zSize.width()-20, zSize.height()-50)

         self.imgQGISLayers.setGeometry(5, 1, 40, 40)
         self.labelMDDFilters.setGeometry(60, 10, 80, 20)
         self.groupBoxObj.setGeometry(140, 5, zSize.width()-200, 30)
         self.labelQGISLayers.setGeometry(60, 40, zSize.width()-120, 20)
         self.listQGISLayers.setGeometry(10, 60, zSize.width()-50, zSize.height()-90)

         
class AuthCSWTDialog(QDialog):
       def  __init__(self, user, pwd, serverCSWT, pgConnect):
         QDialog.__init__(self)
         if not pgConnect : self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Authentification for the CSW-T Server")))
         else : self.setWindowTitle("%s ..." % (QApplication.translate("QSphere","Authentification for PostgreSQL Server")))
         self._W, self._H = 400, 160
         self.setMinimumSize(self._W,self._H)
         self.setMaximumSize(self._W*1.5,self._H)
         layout = QVBoxLayout(self)

         self.groupCSWTInfo = QGroupBox(self)

         from qsphere.qsphere_tools import getThemeIcon
         zIcon = getThemeIcon("cswt.png")
         carIcon = QImage(zIcon)
         self.ImgServerCSWT = QLabel(self.groupCSWTInfo)
         self.ImgServerCSWT.setObjectName("ImgServerCSWT")
         self.ImgServerCSWT.setAlignment(Qt.AlignCenter)
         self.ImgServerCSWT.setPixmap(QPixmap.fromImage(carIcon).scaled(42,42))        

         self.labelServerCSWT = QLabel(self.groupCSWTInfo)
         self.labelServerCSWT.setObjectName("labelServerCSWT")
         self.labelServerCSWT.setAlignment(Qt.AlignLeft)
         self.labelServerCSWT.setWordWrap(True)
         self.labelServerCSWT.setText(serverCSWT)

         self.labelUserServerCSWT = QLabel(self.groupCSWTInfo)
         self.labelUserServerCSWT.setText("%s : " % (QApplication.translate("QSphere","User for CSW-T")))
         self.labelUserServerCSWT.setObjectName("labelUserServerCSWT")
         self.labelUserServerCSWT.setAlignment(Qt.AlignRight)    

         self.UserCSWT = MyWidgetLineEdit(self.groupCSWTInfo)
         self.UserCSWT.initType(6)
         self.UserCSWT.setObjectName("UserCSWT")
         self.UserCSWT.setAccessibleName("UserCSWT")
         self.UserCSWT.textChanged.connect(self.UserCSWT.VerifExpReg)

         self.UserCSWT.setText(user)

         self.labelPwdUserServerCSWT = QLabel(self.groupCSWTInfo)
         self.labelPwdUserServerCSWT.setText("%s : " % (QApplication.translate("QSphere","Password for the connexion")))
         self.labelPwdUserServerCSWT.setObjectName("labelPwdUserServerCSWT")
         self.labelPwdUserServerCSWT.setAlignment(Qt.AlignRight)  

         self.pwdUserCSWT = MyWidgetLineEdit(self.groupCSWTInfo)
         self.pwdUserCSWT.initType(9)
         self.pwdUserCSWT.setObjectName("pwdUserCSWT")
         self.pwdUserCSWT.setAccessibleName("pwdUserCSWT")
         self.pwdUserCSWT.textChanged.connect(self.pwdUserCSWT.VerifExpReg)

         self.pwdUserCSWT.setText(pwd)

         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)

         if (serverCSWT == "" or serverCSWT == None) and not pgConnect : self.acceptButton.setEnabled(False)
         
         layout.addWidget(self.groupCSWTInfo) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)


       def authInfos(self): return (self.UserCSWT.text(), self.pwdUserCSWT.text())


       def resizeEvent(self,ev):
         zSize = ev.size()

         self.groupCSWTInfo.setGeometry(10, 10, zSize.width()-20, zSize.height()-50)

         self.ImgServerCSWT.setGeometry(5, 1, 40, 40)
         self.labelServerCSWT.setGeometry(60, 10, zSize.width()-80, 40)
                  
         self.labelUserServerCSWT.setGeometry(10, 45, 180, 25)
         self.UserCSWT.setGeometry(190, 40, zSize.width()-220, 25)
     
         self.labelPwdUserServerCSWT.setGeometry(10, 75, 180, 25)
         self.pwdUserCSWT.setGeometry(190, 70, zSize.width()-220 , 25)

class SRSDialog(QgsProjectionSelectionDialog): 
       def __init__(self, parent, ztext): 
         QgsProjectionSelectionDialog.__init__(self) 
         self.parent = parent
         self.setCrs(QgsCoordinateReferenceSystem(ztext))
         self._W, self._H = 500, 600
         self.setMinimumSize(self._W,self._H)


class FormatDialog(QDialog):
       def  __init__(self, listformats, zItemText):
         QDialog.__init__(self)
         self.setWindowTitle(QApplication.translate("QSphere","GDAL/OGR formats"))
         self._W, self._H = 500, 300
         self.setMinimumSize(self._W,self._H)

         from qsphere.qsphere_tools import MakeWindowIcon
         MakeWindowIcon(self, "format.png")
         
         layout = QVBoxLayout(self)
         self.gdalogrformats = listformats
         self.listformats = QListView(self)
         self.listformats.setGeometry(QRect(10, 10, 200, 400))
         self.modele = QStringListModel()
         self.modele.setStringList(self.gdalogrformats)
         self.listformats.setModel(self.modele)
         self.listformats.setSelectionMode(QAbstractItemView.SingleSelection)
         self.listformats.setEditTriggers(QAbstractItemView.NoEditTriggers)
         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Close"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Select Item"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
         
         layout.addWidget(self.listformats) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)

         if zItemText != "" :
            selection = self.listformats.model() 
            zIndexItem = self.gdalogrformats.index(zItemText) if zItemText in self.gdalogrformats else -1
            if zIndexItem != -1 :
                   index = selection.index(zIndexItem,0)
                   self.listformats.selectionModel().setCurrentIndex(index, QItemSelectionModel.Select)

         self.listformats.doubleClicked.connect(self.accept)

       def format(self):
         selection = self.listformats.selectionModel()
         indexElementSelectionne = int(selection.currentIndex().row())
         return "%s" % (self.gdalogrformats[indexElementSelectionne])     

class TopologyDialog(QDialog):
       def  __init__(self, dictinfosTopology):
         QDialog.__init__(self)
         self.setWindowTitle(QApplication.translate("QSphere","Topology informations :"))
         layout = QVBoxLayout(self)
         self._W, self._H = 400, 160
         self.setMinimumSize(self._W,self._H)   
         self.zoneInfos = QTableWidget(0, 2, self)
         self.zoneInfos.setObjectName("zoneInfos")
         self.zoneInfos.setAccessibleName("zoneInfos")
         self.zoneInfos.setGeometry(10, 10, 300, 130)

         from qsphere.qsphere_tools import MakeWindowIcon
         MakeWindowIcon(self, "topological.png")

         self._listTopo = ("abstract", "full planar graph", "full surface graph", "full topology 3D", "geometryOnly", "planar graph", "surface graph", "topology1D", "topology3D", "unknow")
         self._listTypeVect = ("complex", "composite", "curve", "point", "solid", "surface","unknow")
         
         try : mydict = literal_eval("%s" % (dictinfosTopology))
         except : mydict = None

         from qsphere.qsphere_tools import DicoHasKey

         zIndexTopo, zIndexTypeVect = 0, 0
         if mydict!= None and type(mydict)== dict :
            zIndexTopo = self._listTopo.index(mydict['TopologyLevelCode']) if DicoHasKey(mydict, 'TopologyLevelCode') else 0
            zIndexTypeVect = self._listTypeVect.index(mydict['GeometricObjectTypeCode']) if DicoHasKey(mydict, 'GeometricObjectTypeCode') else 0    

         zDim = (160, 200)
         for i in range(len(zDim)): self.zoneInfos.setColumnWidth(i, zDim[i])         
         self.zoneInfos.verticalHeader().setVisible(False)
         self.zoneInfos.horizontalHeader().setVisible(False)

         self.zoneInfos.insertRow(0)

         ObjWidget = QLineEdit()
         ObjWidget.setMinimumSize(160, 30)
         ObjWidget.setMaximumSize(160, 30)
         ObjWidget.setText("TopologyLevelCode")
         ObjWidget.setEnabled(False)
         self.zoneInfos.setCellWidget(0, 0, ObjWidget)

         ObjWidget = QComboBox()
         ObjWidget.setMinimumSize(200, 30)
         ObjWidget.setMaximumSize(200, 30)
         ObjWidget.addItems(self._listTopo)
         ObjWidget.setCurrentIndex(zIndexTopo)
         self.zoneInfos.setCellWidget(0, 1, ObjWidget)

         self.zoneInfos.insertRow(1)

         ObjWidget = QLineEdit()
         ObjWidget.setMinimumSize(160, 30)
         ObjWidget.setMaximumSize(160, 30)
         ObjWidget.setText("GeometricObjectTypeCode")
         ObjWidget.setEnabled(False)
         self.zoneInfos.setCellWidget(1, 0, ObjWidget)

         ObjWidget = QComboBox()
         ObjWidget.setMinimumSize(200, 30)
         ObjWidget.setMaximumSize(200, 30)
         ObjWidget.addItems(self._listTypeVect)
         ObjWidget.setCurrentIndex(zIndexTypeVect)
         self.zoneInfos.setCellWidget(1, 1, ObjWidget)

         self.buttonBox = QDialogButtonBox()
         self.quitButton = self.buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
         self.acceptButton = self.buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
         self.buttonBox.accepted.connect(self.accept)
         self.buttonBox.rejected.connect(self.reject)
         
         layout.addWidget(self.zoneInfos) 
         layout.addWidget(self.buttonBox) 
         self.setLayout(layout)


       def infosTopology(self):
         return "{'TopologyLevelCode':'%s', 'GeometricObjectTypeCode':'%s'}" % (self.zoneInfos.cellWidget(0, 1).currentText(), self.zoneInfos.cellWidget(1, 1).currentText())

class MyComboBox(QComboBox):
      def __init__(self, *args):  QComboBox.__init__(self, *args)

      def focusInEvent(self, event):
          self.setStyleSheet("""QComboBox {color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)}"""
                             """QComboBox QAbstractItemView {border: 2px solid darkgray; selection-background-color: darkgray;}"""
                             )
          if self.parent().objectName()!= "formTableWidget" :
              zTableWidget = self.parent().parent()
              
              if (self.objectName().find("tablecontacts")!=-1 or self.objectName().find("tableroles")!=-1):
                     zInfos = self.objectName().split("_")
                     CurrentRow = int(zInfos[2])
                     CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0 
                     zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                     zTableWidget.selectRow(CurrentRow)

                     if zTableWidget.objectName()== "tablecontacts" : zTableWidget.parent().FixeCurrentContact()
              else :
                     if zTableWidget.parent().objectName()== "ViewerTableWidget"  :
                        zInfos = self.objectName().split("_")
                        CurrentRow = int(zInfos[2])
                        CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0 
                        zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                        zTableWidget.selectRow(CurrentRow)
                        zTableWidget.parent().FixeCurrentItem()
                 
      def focusOutEvent(self, event):
             self.setStyleSheet("""QComboBox {color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)}"""
                                """QComboBox QAbstractItemView {border: 2px solid darkgray; selection-background-color: darkgray;}"""
                               )



class MyAction(QAction):
      def __init__(self, zName, zText, zIcon, parent): 
         QAction.__init__(self, parent)
         self.setIcon(QIcon(zIcon))
         self.setText(zText)
         self.setObjectName(zName)

       
class MySlider(QSlider):
      def __init__(self, *args): QSlider.__init__(self, *args)

      def initSlider(self, SizeW, SizeH, zPosX, zPosY, zName, zToolTip, zGeom, zResizable, orientation, valueMin, valueMax, value):
            self.setMinimumSize(SizeW, SizeH)
            if not zResizable : self.setMaximumSize(SizeW, SizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            self.setOrientation(orientation)
            self.setToolTip("%s" % (zToolTip))
            self.setMinimum(valueMin)
            self.setMaximum(valueMax)
            self.setValue(value)


class MyUserComboBox(QComboBox): 
      def __init__(self, *args):
          QComboBox.__init__(self, *args) 
          self.setStyleSheet("background-color:red;")
          self.setAcceptDrops(True)
          self.view().setDragDropMode(QAbstractItemView.DragDrop)
          
      def VerifExpReg(self):          
          self.setStyleSheet("background-color:red;")
          if self.currentText()!="" : self.setStyleSheet("background-color:#AEEE00;")    

class MyWebComboBox(FilteredComboBox): 
      def __init__(self, *args):
          FilteredComboBox.__init__(self, *args) 
          self.regex = QRegExp(r"^(http|https|ftp)\://[a-zA-Z0-9\-\.]+.[\w]{2,4}(:[a-zA-Z0-9]*)?/?([a-zA-Z0-9\-\._\?\,\'/\\\+&amp;%\$#\=~])*$", Qt.CaseSensitive)
          self.setStyleSheet("background-color:red;")
          self.setAcceptDrops(True)
          self.view().setDragDropMode(QAbstractItemView.DragDrop)
          
      def keyPressEvent(self, event):
          if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter :
             if self.parent().objectName() == "DialogViewer" :    
                zDialog = self.parent()
                if zDialog.eURL.findText(zDialog.eURL.currentText())==-1 : zDialog.eURL.addItem(zDialog.eURL.currentText())
                zDialog.go_url()
                return
          FilteredComboBox.keyPressEvent(self, event) 


      def VerifExpReg(self):          
          self.setStyleSheet("background-color:red;")
          if self.currentText().find(" ")!= -1 : self.setEditText(self.currentText().replace(" ","%20"))

          tagLocalFile = ["file:", "filexml:", "fileHTML:",  "about:"]
          cond = False
          for elt in tagLocalFile :
              if self.currentText().startswith(elt): cond = True ; break ;

          if cond : self.setStyleSheet("background-color:#AEEE00;")
          else :
               if self.regex.exactMatch(self.currentText()) : self.setStyleSheet("background-color:#AEEE00;")
               elif self.currentText()=="#blank" : self.setStyleSheet("background-color:#AEEE00;")
  

class MySimpleWidgetLineEditST(QLineEdit):
      def __init__(self, *args):
          QLineEdit.__init__(self, *args)
          self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)")
          self.setAutoFillBackground(True)
          
      def focusInEvent(self, event):
          try :
                 zInfos = self.objectName().split("_")
                 zTableWidget = self.parent().parent()
                 CurrentRow = int(zInfos[2])
                 CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0
                 zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                 zTableWidget.selectRow(CurrentRow)
                 if zTableWidget.objectName()== "tablecontacts" : zTableWidget.parent().FixeCurrentContact()
                 elif zTableWidget.parent().objectName()== "ViewerTableWidget" : zTableWidget.parent().FixeCurrentItem()
                 self.setStyleSheet("")
          except : pass             

      def focusOutEvent(self, event):
             self.setStyleSheet("color: rgb(0, 0, 0);background-color: rgba(131, 131, 131, 125)")

      def keyPressEvent(self, event):
          QLineEdit.keyPressEvent(self,event)

class MySimpleWidgetLineEdit(QLineEdit):
      def __init__(self, *args):
          QLineEdit.__init__(self, *args)
          self.setDragEnabled(True)

      def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):

                  if event.mimeData().hasFormat('text/plain'):
                         event.accept()
                         try : self.setText(event.mimeData().text())
                         except : pass

                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     index = self.indexAt(event.pos())
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)
                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                          if len(map.values())==1:
                                zText = "%s" % (map.values()[0])
                                if  index.row()>=0 and index.column()>=0 :
                                       try : self.item(index.row(), index.column()).setText(zText)
                                       except :
                                             try : self.cellWidget(index.row(), index.column()).setText(zText)
                                             except :
                                                 try : self.cellWidget(index.row(), index.column()).setValue(float(zText))
                                                 except :
                                                     try : self.cellWidget(index.row(), index.column()).setValue(int(zText))
                                                     except :
                                                          try :
                                                              zIndex = self.cellWidget(index.row(), index.column()).findText(zText, Qt.MatchFixedString)
                                                              if zIndex == -1 : zIndex = 0
                                                              self.cellWidget(index.row(), index.column()).setCurrentIndex(zIndex)
                                                          except : pass
                     event.accept()
          else :
                 zTitle = QApplication.translate("QSphere","Information")
                 zMsg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, zTitle, zMsg) 
                 event.ignore()
          
      def focusInEvent(self, event):
          try :
                 zInfos = self.objectName().split("_")
                 zTableWidget = self.parent().parent()
                 CurrentRow = int(zInfos[2])
                 CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0 
                 zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                 zTableWidget.selectRow(CurrentRow)
                 if zTableWidget.objectName()== "tablecontacts" : zTableWidget.parent().FixeCurrentContact()
                 elif zTableWidget.parent().objectName()== "ViewerTableWidget" : zTableWidget.parent().FixeCurrentItem()
          except : pass

class MyCheckBox(QCheckBox):
      def __init__(self, *args):
          QCheckBox.__init__(self, *args)
          
      def focusInEvent(self, event):
          try :
                 zInfos = self.objectName().split("_")
                 zTableWidget = self.parent().parent()
                 CurrentRow = int(zInfos[2])
                 CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0 
                 zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                 zTableWidget.selectRow(CurrentRow)
                 if zTableWidget.objectName()== "tablecontacts" : zTableWidget.parent().FixeCurrentContact()
                 elif zTableWidget.parent().objectName()== "ViewerTableWidget" : zTableWidget.parent().FixeCurrentItem()
          except : pass
     
class MyWidgetLineEdit(QLineEdit):
      def __init__(self, *args):
          QLineEdit.__init__(self, *args)
          self.setDragEnabled(True)
          self.setAcceptDrops(True)
 
      def initType(self, iType):
          if iType == 0 : self.regex = QRegExp(r"(^[\w.-]+@[\w.-]+\.[a-zA-Z]{2,6}$)", Qt.CaseSensitive)
          elif iType == 2 : self.regex = QRegExp(r"(^(([0-8][0-9])|(9[0-5]))[0-9]{3}$)", Qt.CaseSensitive)
          elif iType == 3 : self.regex = QRegExp(r"(^EPSG:+[0-9]{4,6}$)", Qt.CaseSensitive)
          elif iType in (4, 40, 41) : self.regex = QRegExp(r"^(http|https|ftp)\://[a-zA-Z0-9\-\.]+.[\w]{2,4}(:[a-zA-Z0-9]*)?/?([a-zA-Z0-9\-\._\?\,\'/\\\+&amp;%\$#\=~])*$", Qt.CaseSensitive)
          elif iType == 9 : self.setEchoMode(QLineEdit.Password)
          self.setStyleSheet("background-color:red;") if iType != 3 else self.setStyleSheet("background-color:#AEEE00;")
          self.iType = iType

      def dragMoveEvent(self, event): pass
       
      def dragEnterEvent(self, event):
           if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'): event.accept()
           else : event.ignore()

      def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):
                  zObjSource = event.source()
                  first, zeroMode = False, False
                  
                  if event.mimeData().hasFormat('text/plain'):
                         event.accept()
                         try : self.setText(event.mimeData().text())
                         except : pass
                  
                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     try : index = self.indexAt(event.pos())
                     except : index = None 
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)

                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                            
                          if len(map.values())==1: zText = "%s" % (map.values()[0])
                          elif zObjSource.metaObject().className() =="QTableView" :
                               if not first :
                                  zText = zObjSource.model().item(row, column).text()
                                  first, zeroMode = True, True
                          
                          if index == None :
                             self.setText(zText)
                          else :       
                                if  index.row()>=0 and index.column()>=0 :
                                       try : self.item(index.row(), index.column()).setText(zText)
                                       except :
                                             try : self.cellWidget(index.row(), index.column()).setText(zText)
                                             except :
                                                 try : self.cellWidget(index.row(), index.column()).setValue(float(zText))
                                                 except :
                                                     try : self.cellWidget(index.row(), index.column()).setValue(int(zText))
                                                     except :
                                                          try :
                                                              zIndex = self.cellWidget(index.row(), index.column()).findText(zText, Qt.MatchFixedString)
                                                              if zIndex == -1 : zIndex = 0
                                                              self.cellWidget(index.row(), index.column()).setCurrentIndex(zIndex)
                                                          except : pass
                          if zeroMode : break   
                     event.accept()
          else :
                 zTitle = QApplication.translate("QSphere","Information")
                 zMsg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, zTitle, zMsg) 
                 event.ignore()

      def focusInEvent(self, event):
          try :
                 zInfos = self.objectName().split("_")
                 zTableWidget = self.parent().parent()
                 CurrentRow = int(zInfos[2])
                 CurrentCol = zTableWidget.currentColumn() if len(zInfos) == 4 else 0 
                 zTableWidget.setCurrentCell(CurrentRow, CurrentCol)
                 zTableWidget.selectRow(CurrentRow)
                 if zTableWidget.objectName()== "tablecontacts" : zTableWidget.parent().FixeCurrentContact()
                 elif zTableWidget.parent().objectName()== "ViewerTableWidget" : zTableWidget.parent().FixeCurrentItem()
          except : pass
          
      def keyPressEvent(self, event):
          if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter :
             if self.parent().objectName() == "DialogViewer" :    
                zDialog = self.parent()
                zDialog.go_url()
                return
          QLineEdit.keyPressEvent(self,event)

      def VerifExpReg(self):          
          self.setStyleSheet("background-color:red;")
          if self.iType in (0, 3, 4, 40, 41):
             zCond = False    
             if self.iType in (0, 4, 40, 41):
                zCond = True
                if self.iType in (4, 40, 41) and self.text().find(" ")!= -1 : self.setText(self.text().replace(" ","%20"))
                if self.iType == 40 : zCond = (self.text().lower().endswith("csw-publication") or self.text().lower().endswith("csw-all") or self.text().lower().endswith("csw")) 
             else :
                 zText = self.text().replace("EPSG:","")
                 if zText != "" :
                        zId = long(zText)
                        zCond = QgsCoordinateReferenceSystem().createFromSrid(zId)
             if self.regex.exactMatch(self.text()) and zCond : self.setStyleSheet("background-color:#AEEE00;")
          elif self.iType in (6,9) :
             if self.text()!= "" : self.setStyleSheet("background-color:#AEEE00;")   
          elif self.iType == 5 :
             zDate = self.text()
             zTestDate1 = verifdatechaine(zDate, "-")
             if zTestDate1 : self.setStyleSheet("background-color:#AEEE00;")
          elif self.iType == 1 :              
             zDateInf = self.text().split(" ")[0]
             zDateSup = self.text().split(" ")[1]
             zTestDate1 = verifdatechaine(zDateInf, "-")
             zTestDate2 = verifdatechaine(zDateSup, "-")
             if zTestDate1 and zTestDate2:
                L = zDateInf.split("-")
                a, m, j = int(L[0]), int(L[1]), int(L[2])
                d0 = date(a, m, j)
                L = zDateSup.split("-")
                a, m, j = int(L[0]), int(L[1]), int(L[2])
                d1 = date(a, m, j)
                delta = d1 - d0
                if (delta.days)>=0 : self.setStyleSheet("background-color:#AEEE00;")

                
          if self.iType == 2 :
              if self.regex.exactMatch(self.text()):
                 if self.regex.pattern() == "(^(([0-8][0-9])|(9[0-5]))[0-9]{3}$)" :
                    if self.text()!="00000" : self.setStyleSheet("background-color:#AEEE00;") 
                 else : self.setStyleSheet("background-color:#AEEE00;")

class MyLabel(QLabel):
     def __init__(self, *args):
             QLabel.__init__(self, *args)
             self.setStyleSheet("QLabel { border-top : 1px solid yellow;  border-bottom : 1px solid yellow;  border-left: 1px solid yellow;  border-right : 1px solid yellow }")
             self.setWindowOpacity(0.5)
             self.setAutoFillBackground(False)
             self.setUpdatesEnabled(True)

class MyLabelMap(QLabel):
     def __init__(self, *args): QLabel.__init__(self, *args)
            
     def initLabel(self, SizeW, SizeH, zPosX, zPosY, zName, zText, zGeom, zAlign):
            self.setMinimumSize(QSize(SizeW, SizeH))
            self.setMaximumSize(QSize(SizeW, SizeH))
            if zGeom : self.setGeometry(QRect(zPosX, zPosY, SizeW, SizeH))
            self.setAlignment(zAlign)     
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            self.setText(zText)

class MyButton(QPushButton):
     def __init__(self, *args):  QPushButton.__init__(self, *args)
            
     def initButton(self, SizeW, SizeH, zPosX, zPosY, zName, zText, zToolTip, zEnable, zGeom):
            self.setMinimumSize(SizeW, SizeH)
            self.setMaximumSize(SizeW, SizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            self.setToolTip("%s" % (zToolTip))
            self.setText(zText)
            self.setEnabled(zEnable)

class MyPushButton(QPushButton):
     def __init__(self, *args):  QPushButton.__init__(self, *args)
            
     def initPushButton(self, zSizeW, zSizeH, zPosX, zPosY, zName, zText, zToolTip, zGeom, zIcon, zSizeWIcon, zSizeHIcon, zStyleSheet):
            self.setMinimumSize(zSizeW, zSizeH)
            self.setMaximumSize(zSizeW, zSizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, zSizeW, zSizeH)
            if zIcon != "" :
               self.setIcon(QIcon(zIcon))
               self.setIconSize(QSize(zSizeWIcon, zSizeHIcon))
            self.setToolTip(zToolTip)
            if zStyleSheet : self.setStyleSheet(""" QPushButton {border: none;}""")
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            if zText != "" : self.setText(zText)

class MyPushButtonFunny(QPushButton):
     def __init__(self, *args):  QPushButton.__init__(self, *args)
            
     def initPushButton(self, zSizeW, zSizeH, zPosX, zPosY, zName, zText, zToolTip, zGeom, zIcon, zSizeWIcon, zSizeHIcon, zStyleSheet):
            self.setMinimumSize(zSizeW, zSizeH)
            self.setMaximumSize(zSizeW, zSizeH)
            self.sizeWIcon = zSizeWIcon
            self.sizeHIcon = zSizeHIcon
            self.selfFocused = False
            self.subMenuVisble = False
            if zGeom : self.setGeometry(zPosX, zPosY, zSizeW, zSizeH)
            if zIcon != "" :
               self.setIcon(QIcon(zIcon))
               self.setIconSize(QSize(self.sizeWIcon , self.sizeHIcon))
            self.setToolTip(zToolTip)
            if zStyleSheet : self.setStyleSheet(" QPushButton {border-width: 0px; border-radius: 10px;  border-color: beige;}")
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            if zText != "" : self.setText(zText)

     def enterEvent(self, event):
          if not self.selfFocused :
             self.setIconSize(QSize(self.sizeWIcon *0.8, self.sizeHIcon *0.8))
             self.selfFocused = True
          
     def leaveEvent(self, event):
        if self.subMenuVisble == False : self.restoreButton()
     
     def restoreButton(self):
           self.selfFocused = False    
           self.setIconSize(QSize(self.sizeWIcon, self.sizeHIcon))

class MyPushButtonMappable(QPushButton):
     def __init__(self, *args):  QPushButton.__init__(self, *args)
            
     def initPushButton(self, parent, zSizeW, zSizeH, zPosX, zPosY, zName, zText, zToolTip, zGeom, zIcon, zSizeWIcon, zSizeHIcon, zStyleSheet):
            self.parent = parent
            self.setMinimumSize(zSizeW, zSizeH)
            self.setMaximumSize(zSizeW, zSizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, zSizeW, zSizeH)
            if zIcon != "" :
               self.setIcon(QIcon(zIcon))
               self.setIconSize(QSize(zSizeWIcon, zSizeHIcon))
            self.setToolTip(zToolTip)
            if zStyleSheet : self.setStyleSheet(""" QPushButton {border: none;}""")
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            self.isMappable, self.MainPlugin = False, None
            if zText != "" : self.setText(zText)

     def mousePressEvent(self, event):
            if self.isMappable :
               if event.button() == Qt.LeftButton:
                  pos, index = event.pos().x(), -1
                  if 65 <= pos <= 90     : index = 0 #("fr")
                  elif 91 <= pos <= 110  : index = 1 #("en")
                  elif 111 <= pos <= 130 : index = 2 #("it")
                  elif 131 <= pos <= 155 : index = 3 #("de")
                  elif 156 <= pos <= 175 : index = 4 #("es")
                  elif 176 <= pos <= 200 : index = 5 #("fi")

                  if index != -1 :
                     if self.MainPlugin != None :    
                            self.MainPlugin.indexLang = index
                            self.parent.indexLang = index
                            self.parent.changeLang()
                     else :
                            from qsphere.qsphere_tools import SendMessage      
                            zTitle = QApplication.translate("QSphere", "Warning" ) 
                            zMsg = QApplication.translate("QSphere","Can only change the language by MainPlugin !")
                            SendMessage(self.parent, zTitle, "<font color='#ff0000'><b>%s</b></font>" % (zMsg), QgsMessageBar.WARNING, self.parent().duration_warning)

                   

class MySpinBox(QDoubleSpinBox):
     def __init__(self, *args):  QDoubleSpinBox.__init__(self, *args)
     
     def initDoubleSpinBox(self, parent, gui):
            self.parent = parent
            self.gui = gui 
            self.setFocusPolicy(Qt.StrongFocus)
            self.installEventFilter(self)

     def eventFilter(self, object, event):
          if not event : return False
          if event == None : return False
          if event.type() == QEvent.FocusIn:
             zIndexEmp = int(object.accessibleName().split("_")[2])
             try :
                 self.parent.setToolTip("%s" % (zIndexEmp))
                 if self.parent.metaObject().className() in ("QTableWidget", "MyTableWidget") :
                    self.parent.selectRow(zIndexEmp)
                    if self.parent.parent().objectName() == "ViewerTableWidget" : self.parent.parent().FixeCurrentItem()
                 return True
             except : return False
          else : return False


class MyTextEdit(QTextEdit):           
     def __init__(self, *args): QTextEdit.__init__(self, *args)
     
     def initTextEdit(self, SizeW, SizeH, zPosX, zPosY, zName, zGeom, zUndoRedo, zReadOnly, zRichText, zResizable):
            self.setMinimumSize(SizeW, SizeH)
            if not zResizable : self.setMaximumSize(SizeW, SizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setUndoRedoEnabled(zUndoRedo)
            self.setReadOnly(zReadOnly)
            self.setAcceptRichText(zRichText)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            self.setAcceptDrops(True)

     def dragEnterEvent(self, event): event.accept()
     def dragLeaveEvent(self, event): event.accept()
     def dragMoveEvent(self, event): event.accept()
     def dropEvent(self, event):
          if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'):
                  cursor = self.textCursor()
                  if event.mimeData().hasFormat('text/plain'): cursor.insertText(event.mimeData().text())
                  elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
                     encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
                     stream = QDataStream(encoded, QIODevice.ReadOnly)
                     while not stream.atEnd():
                          row = stream.readInt()
                          column = stream.readInt()
                          map = stream.readQVariantMap()
                          if len(map.values())==1:
                                zText = "%s" % (map.values()[0])
                                try : cursor.insertText(zText) 
                                except : pass
                     event.accept()
          else :
                 zTitle = QApplication.translate("QSphere","Information")
                 zMsg = QApplication.translate("QSphere","Bad format !")
                 QMessageBox.information(None, zTitle, zMsg) 
                 event.ignore()
  

class MyTableWidget(QTableWidget):           
       def __init__(self, *args):
              QTableWidget.__init__(self, *args)
              self.setDragEnabled(True)
              self.setAcceptDrops(True)
              self.setDragDropOverwriteMode(False)
              self.setDropIndicatorShown(True)
              self.setSelectionMode(QAbstractItemView.SingleSelection)
              self.setSelectionBehavior(QAbstractItemView.SelectRows) 
              self.setDragDropMode(QAbstractItemView.DragDrop)
              #BUGS
              #self.verticalHeader().setMovable(True)
              self.originalWidth, self.dims = 0, 0
         
       def fixeMenu(self, menu): self.menu = menu

       def dropEvent(self, event):
           if event.mimeData().hasFormat('text/plain'):
              index = self.indexAt(event.pos())
              if  index.row()>=0 and index.column()>=0 :
                  try : self.item(index.row(), index.column()).setText(zText)
                  except :                     
                         try : self.cellWidget(index.row(), index.column()).setText(event.mimeData().text())
                         except :       
                                   try : self.cellWidget(index.row(), index.column()).setValue(float(event.mimeData().text()))
                                   except :
                                       try : self.cellWidget(index.row(), index.column()).setValue(int(event.mimeData().text()))
                                       except :
                                            try :
                                                zIndex = self.cellWidget(index.row(), index.column()).findText(event.mimeData().text(), Qt.MatchFixedString)
                                                if zIndex == -1 : zIndex = 0
                                                self.cellWidget(index.row(), index.column()).setCurrentIndex(zIndex)
                                            except :
                                                 try:
                                                      zItemComment = QTableWidgetItem()
                                                      zItemComment.setText("%s" % (event.mimeData().text()))
                                                      self.setItem(index.row(), index.column(), zItemComment)
                                                      self.item( index.row(), index.column() ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled|Qt.ItemIsEditable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled)
                                                 except : pass
              else :
                 zIndex = self.refDialog.tabWidget.currentIndex()  
                 self.refDialog.AddDragDropLine(self, event.mimeData().text())
                 self.refDialog.tabWidget.setCurrentIndex(zIndex) 
                        
              event.accept()
          
           elif event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist"):
              zObjSource = event.source()
              first, zeroMode = False, False
              
              index = self.indexAt(event.pos())
              encoded =  event.mimeData().data("application/x-qabstractitemmodeldatalist")
              stream = QDataStream(encoded, QIODevice.ReadOnly)
              
              while not stream.atEnd():
                   row = stream.readInt()
                   column = stream.readInt()
                   map = stream.readQVariantMap()

                   if len(map.values())==1: zText = "%s" % (map.values()[0])
                   elif zObjSource.metaObject().className() =="QTableView" :
                        if not first :
                           zText = zObjSource.model().item(row, column).text()
                           first, zeroMode = True, True

                   if  index.row()>=0 and index.column()>=0 :
                         try : self.item(index.row(), index.column()).setText(zText)
                         except :
                               try : self.cellWidget(index.row(), index.column()).setText(zText)
                               except :
                                   try : self.cellWidget(index.row(), index.column()).setValue(float(zText))
                                   except :
                                       try : self.cellWidget(index.row(), index.column()).setValue(int(zText))
                                       except :
                                            try :
                                                zIndex = self.cellWidget(index.row(), index.column()).findText(zText, Qt.MatchFixedString)
                                                if zIndex == -1 : zIndex = 0
                                                self.cellWidget(index.row(), index.column()).setCurrentIndex(zIndex)
                                            except : pass

                   else :
                        zIndex = self.refDialog.tabWidget.currentIndex() 
                        self.refDialog.AddDragDropLine(self, zText)
                        self.refDialog.tabWidget.setCurrentIndex(zIndex) 

                   if zeroMode : break
                   
              event.accept()
               
           else : event.ignore()
               
       def dragMoveEvent(self, event): pass
       
       def dragEnterEvent(self, event):
           if event.mimeData().hasFormat("application/x-qabstractitemmodeldatalist") or event.mimeData().hasFormat('text/plain'): event.accept()
           else : event.ignore()
     
       def contextMenuEvent(self, event): action = self.menu.exec_(self.mapToGlobal(event.pos()))



class MyProgressBar(QProgressBar):
     def __init__(self, *args): QProgressBar.__init__(self, *args)

     def initTextEdit(self, SizeW, SizeH, zPosX, zPosY, zName, zGeom, zInitValue, zViewText, zAlign, zStyleSheet):
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setProperty("value", zInitValue)
            self.setAlignment(zAlign)
            self.setTextVisible(zViewText)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            if zStyleSheet == 0 :
                 self.setStyleSheet(
                      """QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"""
                      """QProgressBar::chunk {background-color: #6C96C6; width: 10px; margin: 0.5px;}"""
                  )

class MyListView(QListView):           
     def __init__(self, *args): QListView.__init__(self, *args)
            
     def initListView(self, SizeW, SizeH, zPosX, zPosY, zName, zGeom):
            self.setMinimumSize(SizeW, SizeH)
            self.setMaximumSize(SizeW, SizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            zFontStyle = "font: 9pt \"%s\";" % (QFont().defaultFamily())
            self.setStyleSheet(zFontStyle)

class MyListWidget(QListWidget):           
     def __init__(self, *args): QListWidget.__init__(self, *args)
            
     def initListWidget(self, SizeW, SizeH, zPosX, zPosY, zName, zGeom):
            self.setMinimumSize(SizeW, SizeH)
            self.setMaximumSize(SizeW, SizeH)
            if zGeom : self.setGeometry(zPosX, zPosY, SizeW, SizeH)
            self.setObjectName(zName)
            self.setAccessibleName(zName)
            zFontStyle = "font: 9pt \"%s\";" % (QFont().defaultFamily())
            self.setStyleSheet(zFontStyle)

#----------------
# FUNCTIONS DATE 
#----------------
def bissextile(an):
    if (an % 4)==0:
        if (an % 100)==0 and (an % 400) != 0 : return False
        else : return True
    else:
        return False
 
def verifdate(j,m,a):
    # correction for short date year (eg.: '09' ald '2009')
    if a<100:
        if a<0: return False  
        if a<50: a += 2000  
        else: a += 1900  
    if m<1 or m>12 or j<1: return False
    if j>(31,29,31,30,31,30,31,31,30,31,30,31)[m-1]: return False
    if (not bissextile(a)) and m==2 and j>28: return False
    return True
 
def verifdatechaine(D, sep="/"):
    L = D.split(sep)
    if len(L)!=3: return False
    try: a, m, j = int(L[0]), int(L[1]), int(L[2])
    except: return False
    return verifdate(j,m,a)
