import psycopg2
from psycopg2 import (connect, pool as psycopg2pool, OperationalError)
from psycopg2.extensions import (ISOLATION_LEVEL_AUTOCOMMIT, POLL_OK, POLL_WRITE, POLL_READ)

from qgis.PyQt.QtCore import (QObject, QEvent, QPoint, QRect, QSize, QSizeF, Qt, QTime, QUrl)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.Qsci import (QsciAPIs, QsciScintilla)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDesktopWidget, QDockWidget,
                                 QFileDialog, QGroupBox, QHBoxLayout, QInputDialog, QLCDNumber, QLabel, QLineEdit, QListView, QMdiSubWindow,
                                 QMenu, QMenuBar, QPushButton, QSizePolicy, QTableView, QTabWidget, QTextBrowser, QToolBar,
                                 QTreeWidget, QTreeWidgetItem, QWidget)
from qgis.PyQt.QtGui import (QColor, QCursor, QFont, QFontMetrics, QIcon, QImage, QKeySequence, QMovie, QPainter, QPixmap, QBrush,
                             QRegion, QStandardItemModel, QTextDocument, QTextCursor)

from qgis.core import (QgsDataSourceUri, QgsMapLayer, QgsProject, QgsRectangle, QgsStyle, QgsVectorLayer)
from qgis.gui import (QgsCollapsibleGroupBox, QgsCodeEditorSQL, QgsColorDialog, QgsMapCanvas, QgsRendererPropertiesDialog, 
                      QgsMapToolIdentify, QgsMapToolPan, QgsMapToolZoom, QgsMessageBar)

from PyQt5.QtWebKitWidgets import (QWebInspector, QWebPage, QWebView)
from PyQt5.QtWebKit import QWebSettings

from qsphere.qsphere_objmaker import (BaseTableModel, editor, FilteredComboBox, FindDialog, MyLabelMap, myMsgBoxDialog, MyProgressBar, MyPushButton, MyPushButtonFunny, MyTextEdit)
from qsphere.qsphere_tools import (CorrigePath, DicoHasKey, fixeRanges, FileNameWithExtension, FixeLabelsFileDialog, FixeLayerType, GetzLayerCombo, MakeWindowIcon,
                           netForHTML, QGISKeyDataSources, getParamsConnexionQGIS, getThemeIcon, makeGetOptions, SendMessage)
from qsphere.importexport import ExportToXML
from qsphere.ui_editorXML import xmlEditor

from qsphere.sql_templates import (GET_ALL_PRIMARYKEYS_DATABASE, GET_ALL_PRIVILEGES_FOR_USER, GET_ALL_SCHEMAS_SYSTEM_DATABASE,
                                   GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, GET_ALL_SCHEMAS_TABLES_DATABASE,
                                   GET_ALL_SERVER_USERS, GET_PG_KEYWORDS, GET_PG_PROCEDURES,
                                   GET_SERVER_NUMERIC_TYPES, GET_SERVER_BASES, GET_SERVER_SYSTEM_SCHEMAS,
                                   GET_TABLE_COUNT_RECORDS, GET_TABLE_FIELDS, GET_TABLE_GEOMETRY_COLUMN,
                                   GET_TABLE_JOINS, GET_TABLE_NAMES, GET_TABLE_RECORDS, GET_TABLE_STATISTICS,
                                   GET_VIEW_SQL, GET_VIEW_SQL_PLUS, TERMINATE_BACKEND_BASE)

from datetime import datetime as thedatetime
from datetime import timedelta
from math import floor
from os import remove as osremove
from os.path import (dirname, exists, join, splitext)
from os import walk
from urllib.parse import (quote, unquote)
import textwrap


def myMdiSubWindow(parent, name, title, width, height):
        self = QMdiSubWindow(parent)
        self.setObjectName(name)
        self.setWindowTitle(title)
        self.setWindowFlags(Qt.WindowMinimizeButtonHint)
        self.setFixedSize(width, height)
        self.setGeometry(10, 10, width, height)
        self.setStyleSheet("QMdiSubWindow {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")
        self.oldPos = self.pos()

        return self

#-------------------------------------
# FONCTIONS CREATE GROUP CONTROLS MAP
#-------------------------------------
def CTRLCarte(self, conteneur, iX, iY, iW, iH):

         self.groupMap = QGroupBox(conteneur)
         self.groupMap.setGeometry(10, 30, 260, 35)

         zIcon = getThemeIcon("mapview/zoomin.png")
         zToolTip = QApplication.translate("QSphere", "Zoom in the map window")
         self.buttonZoomIn = MyPushButton(self.groupMap)
         self.buttonZoomIn.initPushButton(30, 30, 10, 2, "buttonZoomIn", "", zToolTip, True, zIcon, 30, 30, True)

         zIcon = getThemeIcon("mapview/zoomout.png")
         zToolTip = QApplication.translate("QSphere", "Zoom in the map window")
         self.buttonZoomOut = MyPushButton(self.groupMap)
         self.buttonZoomOut.initPushButton(30, 30, 40, 2, "buttonZoomOut", "", zToolTip, True, zIcon, 30, 30, True)

         zIcon = getThemeIcon("mapview/zoomall.png")
         zToolTip = QApplication.translate("QSphere", "Zoom all the map window")
         self.buttonZoomAll = MyPushButton(self.groupMap)
         self.buttonZoomAll.initPushButton(30, 30, 70, 2, "buttonZoomAll", "", zToolTip, True, zIcon, 30, 30, True)
         
         zIcon = getThemeIcon("mapview/zoomprev.png")
         zToolTip = QApplication.translate("QSphere", "Previous zoom")
         self.buttonZoomPrevious = MyPushButton(self.groupMap)
         self.buttonZoomPrevious.initPushButton(30, 30, 100, 2, "buttonZoomPrevious", "", zToolTip, True, zIcon, 30, 30, True)

         zIcon = getThemeIcon("mapview/zoomnext.png")
         zToolTip = QApplication.translate("QSphere", "Next zoom")
         self.buttonZoomNext = MyPushButton(self.groupMap)
         self.buttonZoomNext.initPushButton(30, 30, 130, 2, "buttonZoomNext", "", zToolTip, True, zIcon, 30, 30, True)

         zIcon = getThemeIcon("mapview/pan.png")
         zToolTip = QApplication.translate("QSphere", "Move the map window")
         self.buttonPan = MyPushButton(self.groupMap)
         self.buttonPan.initPushButton(25, 25, 160, 5, "buttonPanLeftMap", "", zToolTip, True, zIcon, 25, 25, True)

         zIcon = getThemeIcon("mapview/identify.png")
         zToolTip = QApplication.translate("QSphere", "Identify for the layer")
         self.buttonIdentify = MyPushButton(self.groupMap)
         self.buttonIdentify.initPushButton(25, 25, 190, 5, "buttonIdentify", "", zToolTip, True, zIcon, 25, 25, True)

         zIcon = getThemeIcon("mapview/properties.png")   
         zToolTip = QApplication.translate("QSphere", "Open properties widget for the layer")
         self.buttonPropertiesLayer = MyPushButton(self.groupMap)
         self.buttonPropertiesLayer.initPushButton(25, 25, 220, 5, "buttonPropertiesLayer", "", zToolTip, True, zIcon, 25, 25, True)
        

class Ui_Dialog_Console(object):
    
    def setupUi(self):
        self.setObjectName("dialogSGDBR")
        self.minwidth, self.minheight = 900, 660
        self.setMinimumSize(self.minwidth, self.minheight)
        self.resize(QSize(QRect(0,0,self.minwidth,self.minheight).size()).expandedTo(self.minimumSizeHint()))
        self.ractitle = "%s" % self.contextText
        self.setWindowTitle(self.ractitle)
        makeGetOptions(self)

        self.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
        self.startdate, self.connexion = None, None
        self.counterFolders = 0
        self.currentRecord = -1
        self.changeSource, self.oldSource = True, ""
        self.currentLayer = None
        self.startTime = thedatetime.now()
        self.listFiltres = ("%s SQL (*.qry)" %  QApplication.translate("QSphere","File"), "%s SQL (*.sql)" %  QApplication.translate("QSphere","File"))
        self.description, self.numericTypesCodes = [], []

        self.titleConfirmation = QApplication.translate("QSphere","Confirmation")
        self.yes = QApplication.translate("QSphere", "Yes")
        self.no = QApplication.translate("QSphere", "No")
        self.SQLCommands = QApplication.translate("QSphere","SQL Commands")
        self.failed = QApplication.translate("QSphere","Failed")
        self.successfull = QApplication.translate("QSphere","Successfull")
        self.completed = QApplication.translate("QSphere","Completed")
        self.labelRecords = QApplication.translate("QSphere","records")
        

        self.pucebase = getThemeIcon("sgbdr/pucebase.png")
        self.pucebaseopen = getThemeIcon("sgbdr/pucebaseopen.png")
        self.ipuceschema = "sgbdr/puceschema.png"
        self.puceschema = getThemeIcon(self.ipuceschema)
        self.ipucetable = "sgbdr/pucetable.png"
        self.pucetable = getThemeIcon(self.ipucetable)
        self.ipuceview = "sgbdr/puceview.png"
        self.puceview = getThemeIcon(self.ipuceview)
        self.pucekey = getThemeIcon("sgbdr/pucekey.png")
        self.pucerecords = getThemeIcon("sgbdr/pucerecords.png")
        self.pucequery = getThemeIcon("sgbdr/pucequery.png")
        self.ipucesuccessfull = "sgbdr/validate.png"
        self.pucesuccessfull = getThemeIcon(self.ipucesuccessfull)
        self.ipucewarning = "sgbdr/warning.png"
        self.pucewarning = getThemeIcon(self.ipucewarning)
        self.pucearrow = getThemeIcon("sgbdr/arrow.png")
        self.puceparami = getThemeIcon("sgbdr/puceparami.png")
        self.puceparamc =getThemeIcon("sgbdr/puceparamc.png")
        self.ipucestartChrono = "sgbdr/chronoplay.png"
        self.pucestartChrono = getThemeIcon(self.ipucestartChrono)
        self.pucestopChrono = getThemeIcon("sgbdr/chronostop.png")
        self.sphery = getThemeIcon("sgbdr/sphery.gif")
        self.deltaChrono = 0
        self.path = CorrigePath(dirname(__file__)).replace("\\", "/")
        
        self.path = join(self.path, "icons/sgbdr/")
        self.baseurl = QUrl('file:///')
        
        self.dicSchemas = {}
        self.sqlDictionnary = {}
        self.metadatatodownload = {}
        self.listBases, self.listSchemas, self.listTables, self.listFunctions = [], [], [], []
        self.offset = -1
        self.targetDB, self.itemQSPMetadata, self.currentItemData = None, None, None
        self.infosChild = False
        self.changedMDDFiles = True
        self.mode = "normal"
        self.cursor = QCursor( Qt.ArrowCursor )
        self.styledlg = None

        self.html = """<HTML><BODY><style>
            table {border-spacing: 0px; width: 100%;}
            table.sql {background-color: white; border-color:black; width: 100%; cellspacing: 0px;}
            th, td {padding: 5px 30px 5px 10px;border-spacing: 0px;font-size: 90%;margin: 0px;}
            th {color: #FFFFFF;text-align: left;background-color: #2C76B4;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            th.title {color: #FFFFFF;text-align: left;background-color: #5D5D5D;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            td {color: #737476;text-align: left;background-color: #e0e9f0;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            tr.comment th {color: #90b4d6;background-color: #FFF; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: left;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em;}
            tr.head th {color: #fff;background-color: #90b4d6; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: left;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em;}
            tr.head th:first-child {-webkit-border-top-left-radius: 5px;-moz-border-radius-topleft: 5px;border-top-left-radius: 5px;}
            tr.head th:last-child {-webkit-border-top-right-radius: 5px;-moz-border-radius-topright: 5px;border-top-right-radius: 5px;}
            tr.headTitle th {font-size: 150%; color: #fff;background-color: #00b4FF; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: center;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em; vertical-align: middle;}
            tr.headTitle th:first-child {-webkit-border-top-left-radius: 10px;-moz-border-radius-topleft: 10px;border-top-left-radius: 10px; -webkit-border-bottom-left-radius: 10px;-moz-border-radius-bottomleft: 10px;border-bottom-left-radius: 10px;}
            tr.headTitle th:last-child {-webkit-border-top-right-radius: 10px;-moz-border-radius-topright: 10px;border-top-right-radius: 10px; -webkit-border-bottom-right-radius: 10px;-moz-border-radius-bottomright: 10px;border-bottom-right-radius: 10px;}

            tr.even td, tr.even th {background-color: #e8eff5;}

            table.browse td.col0 {width: 20%; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px;text-align:right;}
            table.browse td.cols {width: 15%; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 10px;}
            table.browse th {font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px;background-color: #2C76B4;border-bottom: 3px solid #336699;border-right: 1px solid #336699;cursor:pointer;padding: 6px 15px 6px 6px;text-align:left;white-space: nowrap;}
            table.browse td img{padding:0 4px 0 0;margin:0 0 -1px 0;}
            table.browse td.input{border: solid 1px black}
            table.browse td.gel{font-weight:bold; color:336699 ; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px; background-color: #E4EDFC;border-bottom: 3px solid #336699;border-right: 1px solid #336699;cursor:pointer;padding: 6px 15px 6px 6px;text-align:left;white-space: nowrap;}
            table.browse td{border-bottom: 1px solid #ddd; border-right: 1px solid #ddd; color: #222222; font-size:.8em; margin:0;padding: 1px 6px 2px 7px; white-space: nowrap; text-align:left;vertical-align:top;}
        """
        self.html+= "table.browse th.descend{background: #2C76B4 95%% 85%% url('file:///%scaret_up.gif') no-repeat;}" % self.path
        self.html+= "table.browse th.ascend{color: black;background: #2C76B4 95%% 85%% url('file:///%scaret_down.gif') no-repeat;}" % self.path
        self.html+= "</style>"
   
        self.groupParams = QgsCollapsibleGroupBox(self)
        self.groupParams.setMinimumHeight(190)
        self.groupParams.setMaximumHeight(190)
        self.groupParams.setMaximumWidth(self.width()-10)
        self.groupParams.setMinimumWidth(self.width()-10)
        self.groupParams.setTitle("%s : " % (QApplication.translate("QSphere","Params")))


        self.groupParamConnexion = QgsCollapsibleGroupBox(self.groupParams)
        self.groupParamConnexion.setMinimumHeight(160)
        self.groupParamConnexion.setMaximumHeight(160)
        self.groupParamConnexion.setMaximumWidth(225)
        self.groupParamConnexion.setMinimumWidth(225)
        self.groupParamConnexion.setTitle("%s : " % (QApplication.translate("QSphere","Connection params")))
        self.groupParamConnexion.setGeometry(10, 20, 225, 160)
        self.groupParamConnexion.setCollapsed(False)

        self.labelQGISConnexion = QLabel(self.groupParamConnexion)
        self.labelQGISConnexion.setGeometry(5, 22, 90, 20)
        self.labelQGISConnexion.setText("%s : " % (QApplication.translate("QSphere","QGIS Con.")))
        self.labelQGISConnexion.setObjectName("labelQGISConnexion")
        self.labelQGISConnexion.setAlignment(Qt.AlignRight)
        self.labelQGISConnexion.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.QGISConnexion = QComboBox(self.groupParamConnexion)
        self.QGISConnexion.setObjectName("QGISConnexion")
        self.QGISConnexion.setGeometry(100, 20, 120, 25)
        self.QGISConnexion.addItems(QGISKeyDataSources())

        self.labelService = QLabel(self.groupParamConnexion)
        self.labelService.setGeometry(5, 50, 90, 20)
        self.labelService.setText("%s : " % (QApplication.translate("QSphere","Service")))
        self.labelService.setObjectName("labelService")
        self.labelService.setAlignment(Qt.AlignRight)
        self.labelService.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.nameService = QLineEdit(self.groupParamConnexion)
        self.nameService.setObjectName("nameService")
        self.nameService.setText("localhost")

        self.labelPort = QLabel(self.groupParamConnexion)
        self.labelPort.setGeometry(5, 75, 90, 20)
        self.labelPort.setText("%s : " % (QApplication.translate("QSphere","Port")))
        self.labelPort.setObjectName("labelPort")
        self.labelPort.setAlignment(Qt.AlignRight)
        self.labelPort.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.namePort = QLineEdit(self.groupParamConnexion)
        self.namePort.setObjectName("namePort")
        self.namePort.setText("5433")

        self.labelUser = QLabel(self.groupParamConnexion)
        self.labelUser.setGeometry(5, 100, 90, 20)
        self.labelUser.setText("%s : " % (QApplication.translate("QSphere","User")))
        self.labelUser.setObjectName("labelUser")
        self.labelUser.setAlignment(Qt.AlignRight)
        self.labelUser.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.nameUser = QLineEdit(self.groupParamConnexion)
        self.nameUser.setEchoMode(QLineEdit.Password)
        self.nameUser.setObjectName("nameUser")
        self.nameUser.setText("postgres")

        zToolTip = QApplication.translate("QSphere","Show or hide the user")
        self.viewUser = MyPushButton(self.nameUser) 
        self.viewUser.initPushButton(20, 20, 95, 0, "viewUser", "", zToolTip, True, getThemeIcon("show.png"), 20, 20, True)

        self.labelPwdUser = QLabel(self.groupParamConnexion)
        self.labelPwdUser.setGeometry(5, 130, 90, 20)
        self.labelPwdUser.setText("%s : " % (QApplication.translate("QSphere","Password")))
        self.labelPwdUser.setObjectName("labelPwdUser")
        self.labelPwdUser.setAlignment(Qt.AlignRight)
        self.labelPwdUser.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.pwdUser = QLineEdit(self.groupParamConnexion)
        self.pwdUser.setEchoMode(QLineEdit.Password)
        self.pwdUser.setObjectName("pwdUser")
        self.pwdUser.setText("postgres")

        zToolTip = QApplication.translate("QSphere","Show or hide the user")
        self.viewPwd = MyPushButton(self.pwdUser) 
        self.viewPwd.initPushButton(20, 20, 95, 0, "viewPwd", "", "%s [%s]" % (zToolTip, QApplication.translate("QSphere","Password")), True, getThemeIcon("show.png"), 20, 20, True)        
       
        zIcon = getThemeIcon("sgbdr/connect.png")
        zToolTip = QApplication.translate("QSphere","Get the connection")
        self.getConButton = MyPushButton(self.groupParams) 
        self.getConButton.initPushButton(80, 80, 220, 80, "getConButton", "", zToolTip, True, zIcon, 80, 80, True)

        self.groupOptions = QgsCollapsibleGroupBox(self.groupParams)
        self.groupOptions.setMinimumHeight(120)
        self.groupOptions.setMaximumHeight(120)
        self.groupOptions.setMaximumWidth(225)
        self.groupOptions.setMinimumWidth(225)
        self.groupOptions.setTitle("%s : " % (QApplication.translate("QSphere","Options")))
        self.groupOptions.setGeometry(10, 40, 225, 120)
        self.groupOptions.setCollapsed(False)


        self.checkQSphereAsk = QCheckBox(self.groupOptions)
        self.checkQSphereAsk.setObjectName("checkQSphereAsk")
        zText = "%s (QSphere %s)" % (QApplication.translate("QSphere","Silent mode"), QApplication.translate("QSphere","Actions").lower()) 
        self.checkQSphereAsk.setText(zText)
        self.checkQSphereAsk.setToolTip(zText)
        self.checkQSphereAsk.setChecked(True)

        self.checkSQLComments = QCheckBox(self.groupOptions)
        self.checkSQLComments.setObjectName("checkQSphereAsk")
        zText = "%s (%s)" % (QApplication.translate("QSphere","Comment"), QApplication.translate("QSphere","SQL Commands"))
        self.checkSQLComments.setText(zText)
        self.checkSQLComments.setToolTip(zText)
        self.checkSQLComments.setChecked(False)

        self.checkSystemObjects = QCheckBox(self.groupOptions)
        self.checkSystemObjects.setObjectName("checkSystemObjects")
        zText = QApplication.translate("QSphere","Include system objects")
        self.checkSystemObjects.setText(zText)
        self.checkSystemObjects.setToolTip(zText)
        self.checkSystemObjects.setChecked(False)


        self.myDockSQL = QDockWidget("SQL", self.groupParams)
        self.myDockSQL.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
        self.myDockSQL.setWindowTitle("SQL :")
        self.myDockSQL.setFloating(False)
        self.myDockSQL.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
        self.myDockSQL.setAllowedAreas(Qt.NoDockWidgetArea)
        self.myDockSQL.setStyleSheet(
                    "QDockWidget { color:rgb(255,255,255); font-size:12px; font-weight: bold; border: 1px solid lightgray;}"
                    "QDockWidget::title {  text-align: center; background: #5D5D5D; padding-left: 5px;}")        

        MakeWindowIcon(self.myDockSQL, "layers-database.png")

        font = QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(10)

        self.editorSQL = QgsCodeEditorSQL(self.myDockSQL)
        self.editorSQL.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.editorSQL.setMarginVisible(True)
        self.editorSQL.setWrapMode(QsciScintilla.WrapWord)

        self.editorSQL.findDialog = FindDialog(self.editorSQL)

        self.editorSQL.setFont(font)
        fontmetrics = QFontMetrics(font)
        self.editorSQL.setMarginsFont(font)
        self.editorSQL.setMarginLineNumbers(0, True)
        self.editorSQL.setMarginWidth(0, fontmetrics.width("000000000"))
        self.editorSQL.setMarginLineNumbers(1, True)
        self.editorSQL.setMarginWidth(1,1)

        self.widgetEditorSQL = myMdiSubWindow(self.editorSQL,"widgetEditorSQL", "%s :" % QApplication.translate("QSphere","Actions ..."), 300, 80 )
        self.groupEditorSQL = QGroupBox(self.widgetEditorSQL)
        self.groupEditorSQL.setGeometry(10, 30, 280, 35)
        self.editorSQL.duration_info = self.duration_info

        self.searchbutton = MyPushButton(self.groupEditorSQL)
        self.searchbutton.initPushButton(25, 25, 5, 5, "searchbutton", "", QApplication.translate("QSphere", "Search word"), True, getThemeIcon("find.png"), 25, 25, True)
        self.searchbutton.setShortcut(QKeySequence("Ctrl+F"))
        self.BAddParam = MyPushButton(self.groupEditorSQL)
        self.BAddParam.initPushButton(25, 25, 40, 5, "BAddParam", "", QApplication.translate("QSphere","Insert individual param"), True, self.puceparami, 25, 25, True)
        self.BAddPARAM = MyPushButton(self.groupEditorSQL)
        self.BAddPARAM.initPushButton(25, 25, 75, 5, "BAddPARAM", "", QApplication.translate("QSphere","Insert common param"), True, self.puceparamc, 25, 25, True)
        self.checkQuote = QCheckBox(self.groupEditorSQL)
        self.checkQuote.setObjectName("checkQuote")
        zText = QApplication.translate("QSphere","With quote")
        self.checkQuote.setText(zText)
        self.checkQuote.setToolTip(zText)
        self.checkQuote.setChecked(True)
        self.checkQuote.setGeometry(115, 5, 120, 25)
        self.makeMenuAddParamButton(self.BAddParam)
        self.makeMenuAddParamButton(self.BAddPARAM)

        zToolTip = QApplication.translate("QSphere","Start the robot")
        self.chronoSQL = MyPushButton(self.groupEditorSQL) 
        self.chronoSQL.initPushButton(25, 25, 235, 5, "chronoSQL", "", zToolTip, True, self.pucestartChrono, 25, 25, True)
        self.isStartChronoSQL = False
        self.chronoSQLtoRefresh, self.lastSQL = [], []

        self.InitDir = join(dirname(__file__), "qry")

        self.sqlToolBar = QToolBar(self.myDockSQL)
        self.sqlToolBar.setOrientation(Qt.Vertical)
        self.sqlToolBar.setGeometry(10, 25, 32, 140)
        self.sqlToolBar.setStyleSheet("QToolBar {border: 0px solid grey; border-radius: 1px; color: white; spacing:1px;}")

        zIcon = getThemeIcon("new.png")
        zToolTip = "%s SQL" % QApplication.translate("QSphere","New sheet")
        self.eraseSQLButton = MyPushButtonFunny(self.editorSQL) 
        self.eraseSQLButton.initPushButton(24, 24, 370, 390, "eraseSQLButton", "", zToolTip, True, zIcon, 24, 24, True)
        self.eraseSQLButton.setShortcut(QKeySequence("Ctrl+N"))
        self.sqlToolBar.addWidget(self.eraseSQLButton)

        zIcon = getThemeIcon("sgbdr/ressourcessql.png")
        self.getSQLButton = MyPushButtonFunny(self.groupParams) 
        self.getSQLButton.initPushButton(24, 24, 370, 390, "getSQLButton", "", "SQL", True, zIcon, 24, 24, True)
        self.getSQLButton.setShortcut(QKeySequence("Ctrl+G"))
        self.sqlToolBar.addWidget(self.getSQLButton)

        self.sqlToolBar.addSeparator()
        
        zIcon = getThemeIcon("sgbdr/savesql.png")
        zToolTip = "%s SQL" % QApplication.translate("QSphere","Save")
        self.saveSQLFilesButton = MyPushButtonFunny(self.groupParams) 
        self.saveSQLFilesButton.initPushButton(24, 24, 370, 390, "saveSQLFilesButton", "", zToolTip, True, zIcon, 24, 24, True)
        self.saveSQLFilesButton.setShortcut(QKeySequence("Ctrl+S"))
        self.sqlToolBar.addWidget(self.saveSQLFilesButton)

        zIcon = getThemeIcon("sgbdr/savesqlas.png")
        zToolTip = "%s SQL" % QApplication.translate("QSphere","Save as ...")
        self.saveSQLFilesAsButton = MyPushButtonFunny(self.groupParams) 
        self.saveSQLFilesAsButton.initPushButton(24, 24, 370, 390, "saveSQLFilesAsButton", "", zToolTip, True, zIcon, 24, 24, True)
        self.saveSQLFilesAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
        self.sqlToolBar.addWidget(self.saveSQLFilesAsButton)

        self.sqlToolBar.addSeparator()
       
        zIcon = getThemeIcon("sgbdr/sqlplay.png")
        zToolTip = QApplication.translate("QSphere","Execute SQL")
        self.getSQLResultsButton = MyPushButtonFunny(self.groupParams) 
        self.getSQLResultsButton.initPushButton(24, 24, 370, 390, "getSQLResultsButton", "", zToolTip, True, zIcon, 24, 24, True)
        self.getSQLResultsButton.setShortcut(QKeySequence("F5"))
        self.sqlToolBar.addWidget(self.getSQLResultsButton)

        self.makeMenuGetSQLButton(self.getSQLButton)

        self.nameService.setGeometry(100, 50, 120, 20)
        self.namePort.setGeometry(100, 75, 60, 20)
        self.nameUser.setGeometry(100, 100, 120, 20)
        self.viewUser.setGeometry(95, 0, 25, 25)
        self.viewPwd.setGeometry(95, 0, 25, 25)        
        self.pwdUser.setGeometry(100, 130, 120, 20)
        self.checkQSphereAsk.setGeometry(5, 20, 200, 20)
        self.checkSQLComments.setGeometry(5, 50, 200, 20)
        self.checkSystemObjects.setGeometry(5, 80, 200, 20)
        self.getConButton.setGeometry(240, 20, 80, 80)

        self.groupBoxResults = QGroupBox(self)
        self.groupBoxResults.setObjectName("groupBoxResults")

        self.LCDNumber = QLCDNumber(self.groupBoxResults)
        self.LCDNumber.setObjectName("LCDNumber")
        self.LCDNumber.setSegmentStyle(QLCDNumber.Filled)
        self.LCDNumber.setStyleSheet("QLCDNumber {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: yellow;   text-align: center;}")
        self.LCDNumber.setDigitCount(8)
        self.LCDNumber.display(QTime.currentTime().toString("hh:mm:ss"))
        self.idTimer = self.startTimer(1000) 

        zIcon = getThemeIcon("closewindow.png")
        zToolTip = QApplication.translate("QSphere","Close")
        self.CloseButton = MyPushButton(self.groupBoxResults) 
        self.CloseButton.initPushButton(48, 48, 370, 390, "CloseButton", "", zToolTip, True, zIcon, 48, 48, True)

        zIcon = getThemeIcon("print.png")
        zToolTip = QApplication.translate("QSphere","Print")
        self.BPrint = MyPushButton(self.groupBoxResults) 
        self.BPrint.initPushButton(48, 48, 50, 10, "BPrint", "", zToolTip, True, zIcon, 48, 48, True)

        zIcon = getThemeIcon("sgbdr/reload.png")
        zToolTip = QApplication.translate("QSphere","Reload page")
        self.BReload = MyPushButton(self.groupBoxResults) 
        self.BReload.initPushButton(48, 48, 50, 10, "BReload", "", zToolTip, True, zIcon, 48, 48, True)

        zIcon = getThemeIcon("sgbdr/delconsole.png")
        zToolTip = QApplication.translate("QSphere","Clear content of log console")
        self.BDeleteLog = MyPushButton(self.groupBoxResults) 
        self.BDeleteLog.initPushButton(48, 48, 50, 10, "BDeleteLog", "", zToolTip, True, zIcon, 48, 48, True)
        
        self.tabs = QTabWidget(self.groupBoxResults)
        self.tabs.setTabsClosable(False)
        self.tabs.setDocumentMode(True)
        self.tabs.setTabShape(QTabWidget.Rounded)

        self.groupBoxViewsDataBase = QGroupBox()
        self.groupBoxViewsDataBase.setObjectName("groupBoxViewsDataBase")
        
        self.TreeView = QTreeWidget(self.groupBoxViewsDataBase)
        self.TreeView.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.TreeView.setObjectName("TreeView")
        self.TreeView.headerItem().setText(0, "1")
        self.TreeView.header().setVisible(False)
        self.TreeView.header().setDefaultSectionSize(200)
        self.TreeView.setStyleSheet("QTreeWidget { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                     "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")

        self.widgetTreeView = myMdiSubWindow(self.TreeView,"widgetTreeView", "%s :" % QApplication.translate("QSphere","Actions ..."), 140, 80 )
        self.groupTreeView = QGroupBox(self.widgetTreeView)
        self.groupTreeView.setGeometry(10, 30, 120, 35)

        self.BServer = MyPushButton(self.groupTreeView)
        self.BServer.initPushButton(25, 25, 5, 5, "BServer", "",  QApplication.translate("QSphere","Actions..."), True, getThemeIcon("sgbdr/server.png"), 25, 25, True)
        self.BDataBase = MyPushButton(self.groupTreeView)
        self.BDataBase.initPushButton(25, 25, 40, 5, "BDataBase", "", QApplication.translate("QSphere","Actions..."), True, getThemeIcon("sgbdr/database.png"), 25, 25, True)
        self.BTableView = MyPushButton(self.groupTreeView)
        self.BTableView.initPushButton(25, 25, 75, 5, "BTableView", "", QApplication.translate("QSphere","Actions..."), True, getThemeIcon("sgbdr/pucetable.png"), 25, 25, True)        
        self.makeMenuAddParamButton(self.BServer)
        self.makeMenuAddParamButton(self.BDataBase)
        self.makeMenuAddParamButton(self.BTableView)
      
      
        self.tabsResults = QTabWidget(self.groupBoxViewsDataBase)
        self.tabsResults.setTabsClosable(False)
        self.tabsResults.setDocumentMode(True)
        self.tabsResults.setTabShape(QTabWidget.Rounded)

        self.textBrowser = QWebView(self.groupBoxViewsDataBase)
        self.textBrowser.setObjectName("textBrowser")
        self.textBrowser.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.textBrowser.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        self.textBrowser.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
        self.textBrowser.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        self.textBrowser.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.textBrowser.page().setForwardUnsupportedContent(True)
        self.textBrowser.page().linkClicked.connect(self.goToTabResults)

        zIcon = getThemeIcon("sgbdr/tableinfo.png")
        index = self.tabsResults.addTab(self.textBrowser, QIcon(zIcon), "Infos")
        self.tabsResults.setTabToolTip(index, "Infos")

        self.tableview = QTableView(self.groupBoxViewsDataBase)
        self.tableview.setAccessibleDescription("")

        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(3)
        sizePolicy.setHeightForWidth(self.tableview.sizePolicy().hasHeightForWidth())
        self.tableview.setSizePolicy(sizePolicy)
        self.tableview.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.tableview.setSelectionMode(QAbstractItemView.SingleSelection)
        self.tableview.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.tableview.setVisible(False)
        self.tableview.setAlternatingRowColors(True)
        self.tableview.setStyleSheet("QTableView {alternate-background-color: cyan;background-color: white;}")

        zIcon = getThemeIcon("sgbdr/tabledata.png")
        index = self.tabsResults.addTab(self.tableview, QIcon(zIcon), self.labelRecords.title())
        self.tabsResults.setTabToolTip(index, "Data")

        self.mapview = QgsMapCanvas(self.groupBoxViewsDataBase)
        self.mapview.setVisible(False)
        self.mapview.setAccessibleDescription("")

        self.printMapview = QTextDocument() 

        self.widgetMapView = myMdiSubWindow(self.mapview, "widgetConsole", "%s :" % QApplication.translate("QSphere","Actions ..."), 280, 80 )
        CTRLCarte(self, self.widgetMapView, 5, 5, 340, 35)

        self.toolPan = QgsMapToolPan(self.mapview)
        self.toolZoomIn = QgsMapToolZoom(self.mapview, False) 
        self.toolZoomOut = QgsMapToolZoom(self.mapview, True)

        zIcon = getThemeIcon("sgbdr/tablegeom.png")
        index = self.tabsResults.addTab(self.mapview, QIcon(zIcon), QApplication.translate("QSphere","Location : "))
        self.tabsResults.setTabToolTip(index, "Map")

        self.textBrowserChilds = QWebView(self.groupBoxViewsDataBase)
        self.textBrowserChilds.setObjectName("textBrowserChilds")
        self.textBrowserChilds.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.textBrowserChilds.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        self.textBrowserChilds.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
        self.textBrowserChilds.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        self.textBrowserChilds.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.textBrowserChilds.page().setForwardUnsupportedContent(True)
        self.textBrowserChilds.page().linkClicked.connect(self.goToLinkURL)

        zIcon = getThemeIcon("windowmdiarea_false.png")
        zToolTip = QApplication.translate("QSphere","Sub mainwindow's")
        index = self.tabsResults.addTab(self.textBrowserChilds, QIcon(zIcon), zToolTip)
        self.tabsResults.setTabToolTip(index, zToolTip)
        self.tabsResults.setTabEnabled(index, self.parent.objectName() == "qsphereMainWindow")

        self.edit = editor(self)
        
        zIcon = getThemeIcon("exportcat.png")
        index = self.tabsResults.addTab(self.edit, QIcon(zIcon), QApplication.translate("QSphere","LOG console"))
        self.tabsResults.setTabToolTip(index, "LOG console")

        self.iconBasesOn = getThemeIcon("sgbdr/baseson.png")
        self.iconBasesOff = getThemeIcon("sgbdr/basesoff.png")
        
        zToolTip = QApplication.translate("QSphere","Database informations, metadata and data on the server")
        self.imgBasesData =MyPushButton(self.groupBoxResults)
        self.imgBasesData.initPushButton(100, 48, 10, 2, "imgBasesData", "", zToolTip, True, self.iconBasesOn, 100, 48, True)

        index = self.tabs.addTab(self.groupBoxViewsDataBase, QIcon(self.pucebase), QApplication.translate("QSphere","Databases"))
        self.tabs.setTabToolTip(index, "Database views")

        self.groupBoxProgressInfos = QGroupBox(self)
        self.groupBoxProgressInfos.setObjectName("groupBoxProgressInfos")
        self.groupBoxProgressInfos.setTitle("")
        self.groupBoxProgressInfos.setStyleSheet("QGroupBox {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.imgProgress = QLabel(self.groupBoxProgressInfos)
        self.fixeImage("bigconnect.png")
       
        self.progressBar = MyProgressBar(self.groupBoxProgressInfos)
        self.progressBar.initTextEdit(160, 15, 240, 55, "progressBar", True, 0, True, Qt.AlignCenter, 0)

        self.groupBoxProgressInfos.setVisible(False)

        self.movie = QMovie(getThemeIcon("sgbdr/attente.gif"))
        self.status_txt = QLabel(self.groupBoxResults)
        self.status_txt.setMovie(self.movie)
        self.status_txt.setLayout(QHBoxLayout())
        self.status_txt.layout().addWidget(QLabel(''))
        self.status_txt.setVisible(False)


        self.widgetCounter = myMdiSubWindow(self.tableview,"widgetCounter", "%s :" % QApplication.translate("QSphere","View"), 340, 80 )
       
        self.seuils = [10, 50, 100, 200]
        self.BComboLimitSQL = QComboBox(self.widgetCounter)
        self.BComboLimitSQL.addItems(["10", "50","100","200"])
        self.BComboLimitSQL.setStyleSheet("QComboBox {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")
        self.BComboLimitSQL.setMinimumSize(100, 30)
        self.BComboLimitSQL.setMaximumSize(100, 30)

        self.LabelCounterSQL = QLabel(self.widgetCounter)
        self.LabelCounterSQL.setText("0")
        self.LabelCounterSQL.setAlignment(Qt.AlignCenter)
        self.LabelCounterSQL.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")
        self.LabelCounterSQL.setMinimumSize(100, 30)
        self.LabelCounterSQL.setMaximumSize(100, 30)        

        self.BComboLimitSQL.setGeometry(10, 30, 100, 30)
        self.LabelCounterSQL.setGeometry(110, 30, 100, 30)

        self.groupTableView = QGroupBox(self.widgetCounter)
        self.groupTableView.setGeometry(220, 30, 105, 30)

        self.BViewAbstractFromBase = MyPushButton(self.groupTableView)
        self.BViewAbstractFromBase.initPushButton(25, 25, 5, 2, "BViewAbstractFromBase", "", QApplication.translate("QSphere","View XML File for the metadata in the base"), True, getThemeIcon("sgbdr/dataabstract.png"), 25, 25, True)

        self.BDownloadDataFromBase = MyPushButton(self.groupTableView)
        self.BDownloadDataFromBase.initPushButton(25, 25, 40, 2, "BDownloadDataFromBase", "", QApplication.translate("QSphere","Download metadata from the base"), True, getThemeIcon("sgbdr/datadownload.png"), 25, 25, True)

        self.BDeleteDataFromBase = MyPushButton(self.groupTableView)
        self.BDeleteDataFromBase.initPushButton(25, 25, 70, 2, "BDeleteDataFromBase", "", QApplication.translate("QSphere","Delete metadata in the base"), True, getThemeIcon("sgbdr/datadelete.png"), 25, 25, True)

        self.enableQSPButtons(False)

        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

        fontchrono = QFont()
        fontchrono.setFamily('Courier')
        fontchrono.setFixedPitch(True)
        fontchrono.setPointSize(8)
        fontchrono.setBold(True)

        self.chronoInfo = QLabel(self.groupBoxResults)
        carIcon = QImage(getThemeIcon("sgbdr/chronoinfo.png")) 
        self.chronoInfo.setPixmap(QPixmap.fromImage(carIcon))
        self.chronoInfo.setVisible(False)
        self.chronoInfo.setToolTip("")

        self.chronoInfoText = QLabel(self.chronoInfo)
        self.chronoInfoText.setAlignment(Qt.AlignLeft)
        self.chronoInfoText.setFont(fontchrono)
        self.chronoInfoText.setText("0")
        self.chronoInfoText.setGeometry(38, 12, 80, 20)
        self.chronoInfoText.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #009900; color: white;}")

        self.stopChronoSQL = MyPushButton(self.chronoInfo)
        zToolTip = QApplication.translate("QSphere","Stop the robot")
        self.stopChronoSQL.initPushButton(25, 25, 100, 10, "stopChronoSQL", "", zToolTip, True, getThemeIcon("sgbdr/stopchrono.png"), 25, 25, True)        

        self.myDockSQL.topLevelChanged.connect(self.FixeMyDock)
        self.myDockSQL.resizeEvent = self.FixeMyDockSize
        self.myDockSQL.moveEvent = self.FixeMyDockSize

        self.buttonPan.clicked.connect(self.pan)
        self.buttonZoomIn.clicked.connect(self.zoomIn)
        self.buttonZoomOut.clicked.connect(self.zoomOut)
        self.buttonZoomAll.clicked.connect(self.ZoomAll)
        self.buttonZoomPrevious.clicked.connect(self.ZoomPrevious)
        self.buttonZoomNext.clicked.connect(self.ZoomNext)
        self.buttonIdentify.clicked.connect(self.identify)
        self.buttonPropertiesLayer.clicked.connect(self.getPropertiesLayer)

        
        self.tabsResults.currentChanged.connect(self.viewInfos)
        self.QGISConnexion.currentIndexChanged.connect(self.fixeInfosConnexionQGIS)
        self.groupParams.collapsedStateChanged.connect(self.resizeCollapsedGroupBox)
        self.groupParamConnexion.collapsedStateChanged.connect(self.changePosGroupOtions)
        self.getSQLResultsButton.clicked.connect(self.executeSQL)

        self.eraseSQLButton.clicked.connect(self.eraseSQL)  
        self.saveSQLFilesButton.clicked.connect(self.saveFilesSQL)
        self.saveSQLFilesAsButton.clicked.connect(self.saveFilesAsSQL)
 
        self.viewUser.clicked.connect(self.enabledPWD)
        self.viewPwd.clicked.connect(self.enabledPWD)

        self.getConButton.clicked.connect(self.getInfosBase)
        self.TreeView.itemDoubleClicked.connect(self.getInfos)
        self.TreeView.itemClicked.connect(self.changeEnableQSPButtons)
        self.BComboLimitSQL.currentIndexChanged.connect(self.lightDataViewer)

        self.BViewAbstractFromBase.clicked.connect(self.viewAbstractData)
        self.BDownloadDataFromBase.clicked.connect(self.downloadData)
        self.BDeleteDataFromBase.clicked.connect(self.deleteData)

        self.BDeleteLog.clicked.connect(self.delLog) 
        self.BReload.clicked.connect(self.reloadTabWidget) 
        self.BPrint.clicked.connect(self.printWidget)      
        self.CloseButton.clicked.connect(self.reject)
        self.imgBasesData.clicked.connect(self.showHideTreeView)
        self.chronoSQL.clicked.connect(self.startStopChronoSQL)
        self.stopChronoSQL.clicked.connect(self.startStopChronoSQL)
        self.searchbutton.clicked.connect(self.findWord)

        self.fixeInfosConnexionQGIS()
        self.setWindowState(Qt.WindowMaximized)


    def findInResults(self):
          self.findDialog = FindDialogToolSearchFiles(self)
          ret = self.findDialog.exec_()
          self.findDialog.lineEdit.setFocus()
          if self.findDialog.ret > 0 : pass

    def findWord(self):
        ret = self.editorSQL.findDialog.exec_()
        self.editorSQL.findDialog.lineEdit.setFocus()
        if self.editorSQL.findDialog.ret > 0 :
          self.editorSQL.find_text = self.editorSQL.findDialog.getFindText()
         
    def startStopChronoSQL(self):
        if not self.isStartChronoSQL :
                self.deltaChrono, ok = QInputDialog.getInt(None, QApplication.translate("QSphere","Params"), "<h2>%s :</h2><br>[%s]" % (QApplication.translate("QSphere","Time interval"),
                                                                                                                                        QApplication.translate("QSphere","minutes")), QLineEdit.Normal)
                if not ok: return
                if self.deltaChrono == 0 : return
                if self.lastSQL == []: return

                self.chronoSQLtoRefresh = list(self.lastSQL)
                self.sendToConsole("sql", self.getTimeStamp(), "%s<br>%s : %s %s." % (self.chronoSQLtoRefresh,
                                                                                  QApplication.translate("QSphere","Time interval"),
                                                                                  self.deltaChrono,
                                                                                  QApplication.translate("QSphere","minutes")), self.ipucestartChrono)                
                self.startTimeChrono = thedatetime.now()
                self.deltaChronoInMinutes = self.deltaChrono 
                self.endTimeChrono = self.startTimeChrono + timedelta(minutes=self.deltaChronoInMinutes)
                self.endTimeChronoText = thedatetime.strptime(self.endTimeChrono.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                
                diff = self.endTimeChronoText - thedatetime.strptime(thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                self.chronoInfoText.setText("%s" % diff)
                
                toolTip = ""
                for line in textwrap.wrap("%s" % self.chronoSQLtoRefresh, 100) : toolTip+= "%s<br>" % line
                self.chronoInfo.setToolTip(toolTip)
                self.deltaChrono = int(self.deltaChrono)*1000*60
                
               
        self.isStartChronoSQL = not(self.isStartChronoSQL)    
        self.chronoSQL.setIcon(QIcon(self.pucestartChrono)) if not self.isStartChronoSQL else self.chronoSQL.setIcon(QIcon(self.pucestopChrono))
        self.chronoSQL.setToolTip(QApplication.translate("QSphere","Start the robot")) if not self.isStartChronoSQL else self.chronoSQL.setToolTip(QApplication.translate("QSphere","Stop the robot"))
        self.chronoInfo.setVisible(self.isStartChronoSQL)
        
    def enabledPWD(self):
        obj = self.pwdUser if self.sender() == self.viewPwd else self.nameUser
        obj.setEchoMode(QLineEdit.Normal) if obj.echoMode() == QLineEdit.Password else obj.setEchoMode(QLineEdit.Password)
        self.sender().setIcon(QIcon(getThemeIcon("show.png"))) if obj.echoMode() == QLineEdit.Password else self.sender().setIcon(QIcon(getThemeIcon("hide.png")))  

    def showHideTreeView(self):
        self.TreeView.setVisible(not self.TreeView.isVisible())

        self.imgBasesData.setIcon(QIcon(self.iconBasesOn)) if self.TreeView.isVisible() else self.imgBasesData.setIcon(QIcon(self.iconBasesOff))
        
        self.majSizetabsResults()   

    def majSizetabsResults(self):
        if not self.TreeView.isVisible(): self.tabsResults.setGeometry(10, 10,self.tabs.width()-20, self.tabs.height()-38)       
        else : self.tabsResults.setGeometry(int(self.tabs.width())/4+10, 10, int(self.tabs.width()*3)/4-18, self.tabs.height()-38)              
     
    def FixeMyDockSize(self, obj):
        self.editorSQL.setGeometry(2, 20, self.myDockSQL.width()-4, self.myDockSQL.height()-22 )
        self.widgetEditorSQL.setGeometry(self.editorSQL.width()-320, self.editorSQL.height()-90, 100, 35)
    
        
    def FixeMyDock(self):
        if not self.myDockSQL.isFloating() :
            self.myDockSQL.setGeometry(320, 10, self.groupParams.width()-340, self.groupParams.height()-15 )
            self.myDockSQL.setVisible(False) if self.groupParams.isCollapsed() else self.myDockSQL.setVisible(True)
            self.groupParams.setCollapsed(False)
        else :
            width, height = QDesktopWidget().screenGeometry().width()/2, QDesktopWidget().screenGeometry().height()/2
            self.myDockSQL.setGeometry(200, 200, width, height)
            self.myDockSQL.setVisible(True)
            self.groupParams.setCollapsed(True)
        
    def reject(self):
        self.killTimer(self.idTimer)    
        self.myDockSQL.close()
        try :
            if self.currentLayer: QgsProject.instance().removeMapLayers([self.currentLayer.id()])
        except : pass
        if self.parent.dialogSGBDR != None : self.parent.dialogSGBDR = None
        self.reject()

    def apply_style(self):
        if self.styledlg:
                self.styledlg.apply()
                self.mapview.refresh()
            
    def getPropertiesLayer(self):
        layer = self.mapview.layer(0)
        dlg = QgsRendererPropertiesDialog(layer, QgsStyle.defaultStyle(), False)
        dlg.setMapCanvas(self.mapview)
        MakeWindowIcon(dlg, "mapview/properties.png")
        dlg.setWindowTitle("%s : %s" % (dlg.windowTitle(), layer.name()))
        buttonBox = None
        for conteneur in dlg.children():
                if conteneur.children()!= []:
                   for child in conteneur.children():
                          for elt in child.children():
                                  if type(elt) == QDialogButtonBox :
                                             buttonBox = elt
                                             break

        if buttonBox :
                self.applystyle = buttonBox.addButton(QApplication.translate("QSphere", "Apply style"), QDialogButtonBox.ActionRole)
                self.applystyle.clicked.connect(self.apply_style)
                buttonBox.rejected.connect(dlg.close)

        self.styledlg = dlg                          
        dlg.exec_()
        self.mapview.refresh()


    def addTag(self):
        textToInsert = {self.act_addparambase : " '%param$' --{votre base}-- ",
                        self.act_addparamschema : " '%param$' --{votre schema}-- ",
                        self.act_addparamschematable : " '%param$' --{votre schema.table}-- ",
                        self.act_addparamtable : " '%param$' --{votre table}-- ",
                        self.act_addparamfunction : " '%param$' --{votre fonction}-- ",
                        self.act_addPARAMbase : " '%PARAM$' --{votre base}-- ",
                        self.act_addPARAMschema : " '%PARAM$' --{votre schema}-- ",
                        self.act_addPARAMschematable : " '%PARAM$' --{votre schema.table}-- ",
                        self.act_addPARAMtable : " '%PARAM$' --{votre table}-- ",
                        self.act_addPARAMfunction : " '%PARAM$' --{votre fonction}-- ",                        
                        }
        self.editorSQL.insertText("%s \n" % textToInsert[self.sender()] if self.checkQuote.isChecked() else "%s \n" % textToInsert[self.sender()].replace("'",""))
        
        
    def makeMenuAddParamButton(self, Obj):
        menuAddParamButton = QMenu() 
        menuAddParamButton.setStyleSheet("QMenu {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")

        if Obj == self.BAddParam :
                pixpuceparam = QPixmap(self.puceparami)
                self.act_addparambase = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Base tag")), self.addTag)
                self.act_addparamschema = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema tag")), self.addTag)
                self.act_addparamschematable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema.Table tag")), self.addTag)
                self.act_addparamtable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Table tag")), self.addTag)
                self.act_addparamfunction = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert function tag")), self.addTag)
        elif Obj == self.BAddPARAM:
                pixpuceparam = QPixmap(self.puceparamc)
                self.act_addPARAMbase = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Base tag")), self.addTag)
                self.act_addPARAMschema = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema tag")), self.addTag)
                self.act_addPARAMschematable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema.Table tag")), self.addTag)
                self.act_addPARAMtable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Table tag")), self.addTag)
                self.act_addPARAMfunction = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert function tag")), self.addTag)
        elif Obj == self.BServer: 
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/expandall.png"))
                self.expandAllNodes = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Expand all nodes")), self.expandTreeAllNodes)
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/reduceall.png"))
                self.closeAllNodes = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Close all nodes")), self.expandTreeAllNodes)
        elif Obj == self.BDataBase: 
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/expandall.png"))
                self.expandAllBaseNodes = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Expand all")), self.expandTreeBaseNodes)
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/reduceall.png"))
                self.closeAllBaseNodes = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Reduce all")), self.expandTreeBaseNodes)
                menuAddParamButton.addSeparator()
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/reload.png"))
                self.BReloadBase = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Reload current base")), self.reloadBase)
                self.BReloadBase.setEnabled(False)
        elif Obj == self.BTableView:
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/vacuum.png"))
                self.BVacuumTable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Vacuum table")), self.vacuumTable)
                menuAddParamButton.addSeparator()
                pixpuceparam = QPixmap(getThemeIcon("sgbdr/adeletetable.png"))
                self.BDeleteTable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Delete table")), self.deleteTable)

        Obj.setMenu(menuAddParamButton)
        
 
    def makeMenuGetSQLButton(self, Obj):
        menuGetSQLButton = QMenu() 
        menuGetSQLButton.setStyleSheet("QMenu {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")
        pixopensql = QPixmap(getThemeIcon("open.png"))
        pixallusers = QPixmap(getThemeIcon("sgbdr/allusers.png"))
        pixallrightsuser = QPixmap(getThemeIcon("sgbdr/rightforuser.png"))
        pixallrightsusers = QPixmap(getThemeIcon("sgbdr/rightforallusers.png"))
        pixexit =  QPixmap(getThemeIcon("close.png"))
        
        self.act_openFilesSQL = menuGetSQLButton.addAction(QIcon(pixopensql), '{:<45}'.format(QApplication.translate("QSphere","Open Files")), self.openFilesSQL)
        self.act_openFilesSQL.setShortcut(QKeySequence("Ctrl+O"))
        menuGetSQLButton.addSeparator()
        self.act_getAllUsersForServer = menuGetSQLButton.addAction(QIcon(pixallusers), '{:<45}'.format(QApplication.translate("QSphere","Get all the users on the server")), self.getAllUsersForServer)
        self.act_getAllPrivilegesForUserDataBase = menuGetSQLButton.addAction(QIcon(pixallrightsuser), '{:<45}'.format(QApplication.translate("QSphere","Get the rights for current profil user on the server")), self.getAllPrivilegesForUserDataBase)
        self.act_getAllPrivilegesForAllUsersDataBase = menuGetSQLButton.addAction(QIcon(pixallrightsusers), '{:<45}'.format(QApplication.translate("QSphere","Get all the rights for all users on the server")), self.getAllPrivilegesForAllUsersDataBase)
        menuGetSQLButton.addSeparator()
        self.act_getAllSchemasDataBase = menuGetSQLButton.addAction(QIcon(QPixmap(self.puceschema)), '{:<45}'.format(QApplication.translate("QSphere","Get list of schemas")), self.getAllSchemasDataBase)
        self.act_getAllSchemasTablesDataBase = menuGetSQLButton.addAction(QIcon(QPixmap(self.pucetable)), '{:<45}'.format(QApplication.translate("QSphere","Get list of schemas and tables")), self.getAllSchemasTablesDataBase)                
        self.act_getAllPrimaryKeysDataBase = menuGetSQLButton.addAction(QIcon(QPixmap(self.pucekey)), '{:<45}'.format(QApplication.translate("QSphere","Get all primary keys on the server for the database")), self.getAllPrimaryKeysDataBase)
        menuGetSQLButton.addSeparator()
        self.act_exit = menuGetSQLButton.addAction(QIcon(pixexit), QApplication.translate("QSphere","Close"), self.passaction) 

        self.act_getAllUsersForServer.setEnabled(False)
        self.act_getAllPrivilegesForUserDataBase.setEnabled(False)
        self.act_getAllPrivilegesForAllUsersDataBase.setEnabled(False)
        self.act_getAllSchemasDataBase.setEnabled(False)
        self.act_getAllSchemasTablesDataBase.setEnabled(False)
        self.act_getAllPrimaryKeysDataBase.setEnabled(False)        

        if Obj != self.getSQLButton : return
        
        Obj.setMenu(menuGetSQLButton)
        self.freeAllOtherButtons(Obj) 

        
    def freeAllOtherButtons(self, senderButton):
        buttons = (self.getSQLButton, self.saveSQLFilesButton, self.saveSQLFilesAsButton, self.getSQLResultsButton)
        for button in buttons :
            if button != senderButton : 
                button.subMenuVisble = False
                button.restoreButton()


    def passaction(self): 
        obj = None
        obj = self.findChild(MyPushButtonFunny, "getSQLButton") 
        if obj != None : obj.subMenuVisble = False

    def zoomIn(self): self.mapview.setMapTool(self.toolZoomIn)
    def zoomOut(self):  self.mapview.setMapTool(self.toolZoomOut)
    def identify(self):  self.mapview.setCursor(self.cursor)
    def pan(self): self.mapview.setMapTool(self.toolPan)

    def ZoomAll(self): self.mapview.zoomToFullExtent()
    def ZoomPrevious(self):self.mapview.zoomToPreviousExtent()
    def ZoomNext(self): self.mapview.zoomToNextExtent()
    def ZoomToLayer(self) :  self.mapview.zoomToActiveLayer()

    def timerEvent(self, e):
        self.LCDNumber.display(QTime.currentTime().toString("hh:mm:ss"))
        if self.status_txt.isVisible() : self.status_txt.repaint()
        if self.isStartChronoSQL :
           if ((thedatetime.now()-self.startTimeChrono).total_seconds() * 1000) >= self.deltaChrono :
                self.startTimeChrono = thedatetime.now()
                self.endTimeChrono = self.startTimeChrono + timedelta(minutes=self.deltaChronoInMinutes)
                self.endTimeChronoText = thedatetime.strptime(self.endTimeChrono.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                self.sendToConsole("sql", self.getTimeStamp(), "%s..." % QApplication.translate("QSphere","Execute SQL"), self.ipucestartChrono)
                self.executeSQL()

           diff = self.endTimeChronoText - thedatetime.strptime(thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
           self.chronoInfoText.setText("%s" % diff)

           
    def changePosGroupOtions(self, state) :
        if self.groupParamConnexion.isCollapsed() : self.groupOptions.setGeometry(10, 40, self.groupOptions.width(), self.groupOptions.height())
        else : self.groupOptions.setGeometry(10, 40+self.groupParamConnexion.height(), self.groupOptions.width(), self.groupOptions.height())
           
        
    def resizeCollapsedGroupBox(self, state) :

        if self.groupParams.isCollapsed() : self.groupBoxResults.setGeometry(5, 30, self.width()-10 , self.height()-40)
        else : self.groupBoxResults.setGeometry(5, 200, self.width()-10 , self.height()-220)
        
        self.tabs.setGeometry(10, 20, self.groupBoxResults.width()-70 , self.groupBoxResults.height()-30)
        self.TreeView.setGeometry(10, 10, int(self.tabs.width())/4-10, self.tabs.height()-40)
        self.majSizetabsResults()
        self.widgetTreeView.setGeometry(self.TreeView.width()-170, 20, 100, 35)

        self.BDeleteLog.setGeometry(self.groupBoxResults.width()-60, self.groupBoxResults.height()-210, 48, 48)
        self.BReload.setGeometry(self.groupBoxResults.width()-60, self.groupBoxResults.height()-160, 48, 48)
        self.BPrint.setGeometry(self.groupBoxResults.width()-60, self.groupBoxResults.height()-110, 48, 48)
        self.CloseButton.setGeometry(self.groupBoxResults.width()-60, self.groupBoxResults.height()-60, 48, 48)
        self.myDockSQL.setVisible(True)

        if self.groupParams.isCollapsed() :
           if not self.myDockSQL.isFloating() : self.myDockSQL.setVisible(False)
           else : self.myDockSQL.setVisible(True)
        else : self.myDockSQL.setVisible(True)    

        

    def resizeEvent(self,ev):
        
         self.groupParams.setMaximumWidth(self.width()-10)
         self.groupParams.setGeometry(5, 5, self.width()-10, 170)

         if not self.myDockSQL.isFloating() :       
                 self.myDockSQL.setGeometry(320, 10, self.groupParams.width()-340, self.groupParams.height()-15)
                 self.editorSQL.setGeometry(2, 20, self.myDockSQL.width()-4, self.myDockSQL.height()-22)

         self.groupBoxProgressInfos.setGeometry(self.width()*2/3-100, self.height()/2-100, 200, 200)
         self.imgProgress.setGeometry(10, 0, self.groupBoxProgressInfos.width()-20, self.groupBoxProgressInfos.height()-20)
         self.progressBar.setGeometry(10, self.groupBoxProgressInfos.height()-40, self.groupBoxProgressInfos.width()-20, 20)

         self.resizeCollapsedGroupBox(self.groupParams.isCollapsed())

         self.imgBasesData.setGeometry(int(self.tabs.width())/4-50, 10, 100, 48)
         self.status_txt.setGeometry(int(self.tabs.width()*2)/3-70, 10, 64, 64)
         self.chronoInfo.setGeometry(self.groupBoxResults.width()-320, 0, 125, 43)
         self.LCDNumber.setGeometry(self.groupBoxResults.width()-180, 8, 120, 30)
         self.barInfo.setGeometry(0, 0, self.width(), 90)

      
    def loadListMDDFiles(self):

        if self.parent.objectName()!= "qsphereMainWindow" : return 
        
        if self.changedMDDFiles :       
            root = self.parent.listWindowsButtons.invisibleRootItem()
            abstract = "No result"
            html = self.html

            html+= "<script type='text/javascript'>"
            zPath = CorrigePath(dirname(__file__)).replace("\\","/")
            with open("%sressources/html/biblio/viewerdata.js" % zPath, 'r') as filejs :
                 LinesJS = filejs.read()
            html+= LinesJS                 
            html+= "</script>"
            
            iconWindowsCount = getThemeIcon("windowcounts.png")
            html+= "<h2><img src='file:///%s'> %s</h2><table width='100%%'>" % (iconWindowsCount,  QApplication.translate("QSphere","Sub mainwindow's"))
            html+= "<tr><th></th><th width='25%%'>%s</th><th>%s</th></tr>" % (QApplication.translate("QSphere","Title : "), QApplication.translate("QSphere","Abstract : "))   
           
            for i in range(root.childCount()):
                mdiSubWindow = self.parent.listSubWindows[root.child(i).whatsThis(0)]
                obj = mdiSubWindow.findChild(MyTextEdit, "resume")
                abstract = obj.toPlainText().replace("'", "\'") if obj != None else "No result"
                abstract = abstract.replace("\n", "<br>")
                html+= "<tr><td><a href='UPLOAD%s'><img src='file:///%s'></a></td><td><h3>%s<h3></td><td>" % (i, getThemeIcon("copyinlist.png"), root.child(i).text(0))
                html+= "<a href='javascript: switchInfoPerso(%s);'><img src='file:///%s'><span id='msg%s'>%s</span></a>" % (i, self.pucearrow, i, QApplication.translate("QSphere", "Hide"))
                html+= "<table><tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'>"
                html+= "<div id='divacacher%s' style='background: #efe; padding: 5px; width: 100%%; height: 100%%;overflow-x: hidden; overflow-y: scroll;'>%s</div></td></tr></table></td></tr>" % (i, abstract)
                
            html+= "</table>"

            html+= "<script type='text/javascript'>"
            html+= "function switchInfoPerso(index)"
            html+= "{"
            html+= "divInfo = document.getElementById('divacacher'+index);"
            html+= "divTexte = document.getElementById('msg'+index);"
            html+= "if (divInfo.style.display == 'none'){divInfo.style.display = 'block';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "Hide")
            html+= "else{divInfo.style.display = 'none';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "View")
            html+= "}\n"                      
            html+= "function init(){"

        
            for i in range(root.childCount()):
               html+= "var TableSorter%s = new TSorter;" %(i)
               html+= "TableSorter%s.init('maTable%s');" %(i, i)
            html+= "}"
            html+= "</script>"
            html+= "<script type='text/javascript'>"    
            html+= "window.onload = init;"
            html+= "</script>"
            html+= "</body></html>"

            self.textBrowserChilds.setHtml(html, self.baseurl) #setText(html)#, self.baseurl)
            self.infosChild = True
            self.changedMDDFiles = False



    def viewInfos(self , index):
        if index == 0 : self.getInfos(self.TreeView.currentItem())     
        elif  index == 1 : self.lightDataViewer()
        elif index == 2 : self.lightMapViewer()
        elif index == 3 : self.loadListMDDFiles()

    def delLog(self): self.edit.setText("")    

    def reloadTabWidget(self):
        index  = self.tabsResults.currentIndex()
        if index == 0 :
           if self.textBrowser.accessibleDescription()!= "sql" :
                self.getInfos(self.TreeView.currentItem())     
           else :     
                   if self.lastSQL != []:
                      self.chronoSQLtoRefresh = list(self.lastSQL)
                      self.executeSQL()
           
        elif  index == 1 : self.tableview.setAccessibleDescription("refresh") ; self.lightDataViewer();
        elif index == 2 : self.mapview.setAccessibleDescription("refresh") ; self.lightMapViewer()
        elif index == 3 : self.changedMDDFiles = True ; self.loadListMDDFiles();        


    def makeImage(self, printer, index):
        fileName = join(dirname(__file__),"xml/tempo.png") 
        self.mapview.saveAsImage(fileName, None, "PNG")
        self.printMapview.clear()
        self.printMapview.setPageSize(QSizeF(printer.pageRect().size()))

        try:
            from osgeo.gdal import Open as gdalopen
            from osgeo.gdal import GA_ReadOnly
        except:
            from gdal import Open as gdalopen
            from gdal import GA_ReadOnly

        sourceWidth, sourceHeight = printer.pageRect().width(), printer.pageRect().height()
        dataset = gdalopen( fileName, GA_ReadOnly )
        if dataset :
            sourceWidth = dataset.RasterXSize
            sourceHeight = dataset.RasterYSize
        xscale = (printer.pageRect().width())/ float(sourceWidth)
        yscale = (printer.pageRect().height()-100) / float(sourceHeight)
        scale = min(xscale, yscale)
        width = int(sourceWidth*scale)
        
        title = "%s : %s" % (QApplication.translate("QSphere", "Print"), self.tabsResults.tabText(index))

        html = """
        <head>
            <title>Report</title>
        </head>
        <body>
            <table width="100%">
                <tr><td><h1>{} {}</h1></td></tr>
            </table>
            <hr>
            <p align=right><img src="{}" width="{}"></p>
            <p align=right>QSphere, le {}</p>
        </body>
        """.format(title, self.mapview.toolTip(), fileName, width, thedatetime.now().strftime("%d/%m/%Y à %Hh%Mm"))
        self.printMapview.setHtml(html)

    def printWidget(self):
        index  = self.tabsResults.currentIndex()
            
        printer = QPrinter() 
        printer.setPageSize(QPrinter.A4)
        printer.setOrientation(QPrinter.Portrait) if index in (0, 4) else printer.setOrientation(QPrinter.Landscape)
        printer.setPageMargins(5, 5, 5, 5, QPrinter.Millimeter)
        printer.setOutputFormat(QPrinter.NativeFormat)
        printDialog = QPrintPreviewDialog(printer)
            
        if  index == 0 : obj = self.textBrowser
        elif index == 1 : pass
        elif index == 2 : self.makeImage(printer, index); obj = self.printMapview; 
        elif index == 3 : obj = self.textBrowserChilds
        elif index == 4 : obj = self.edit

        printDialog.setWindowTitle("%s : %s" % (QApplication.translate("QSphere", "Print"), self.tabsResults.tabText(index)))
        printDialog.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
        printDialog.paintRequested.connect(obj.print_) if index != 1 else printDialog.paintRequested.connect(self.printDataTable)
        printDialog.exec_()                  
        

    def lightMapViewer(self):
        Item = self.changeEnableQSPButtons()
        if not Item : return
        if Item.whatsThis(0) != "table" and Item.whatsThis(0) != "view" : return

        newsource = "%s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0))
        if type(self.sender()) == QTabWidget and self.mapview.accessibleDescription()== newsource : return
        self.mapview.setToolTip("%s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)))

        geomColumn = self.getGeometryColumn(Item, True)
        if geomColumn == None :
           try :
               if self.currentLayer:
                  QgsProject.instance().removeMapLayers([self.currentLayer.id()])
                  QgsProject.instance().addMapLayers([], False)
                  self.mapview.setLayers([]) 
                  self.mapview.refresh()
                  self.currentLayer = None
                  self.mapview.setAccessibleDescription("%s.%s.%s" %  (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)))
                  self.tabsResults.setCurrentIndex(1)
           except : pass  
           return

        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))


        try :
            uri = QgsDataSourceUri()
            uri.setConnection(self.nameService.text(), self.namePort.text(), self.targetDB,  self.nameUser.text(), self.pwdUser.text())

            schema = self.quotedName(Item.parent().text(0), False)
            table = self.quotedName(Item.text(0), False)
            
            req = u"(SELECT row_number() Over() as myid, a.* FROM %s.%s AS a)" % (schema, table) #LIMIT 1000 ??
            uri.setDataSource("", req, geomColumn, "",  "myid")
            vlayer = QgsVectorLayer(uri.uri(), Item.text(0), "postgres")

            if self.currentLayer == vlayer : return

            if vlayer != None :
               if vlayer.isValid() :
                    self.mapview.setLayers([vlayer]) 
                    self.mapview.refresh()
                    self.mapview.zoomToFullExtent()
                    self.currentLayer = vlayer
                    
                    self.mapview.setAccessibleDescription("%s.%s.%s" %  (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)))
            
        except : pass
        QApplication.restoreOverrideCursor()
            

    def lightDataViewer(self):
        Item = self.changeEnableQSPButtons()
        if not Item : return
        if Item.whatsThis(0) != "table" and Item.whatsThis(0) != "view" : return

        newsource = "%s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0))
        if type(self.sender()) == QTabWidget and self.tableview.accessibleDescription()== newsource : return

        if self.sender() == self.BComboLimitSQL : self.tableview.setAccessibleDescription("refresh")
        self.tableview.setToolTip(newsource)

        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
           con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
           cur = con.cursor()
        else : return   


        try :
             if Item.whatsThis(0) == "table" :   req = GET_TABLE_RECORDS % (Item.parent().text(0), Item.text(0), int(self.BComboLimitSQL.currentText()))
             elif Item.whatsThis(0) == "view" :
                    records = self.getRecords(cur, GET_VIEW_SQL % (Item.text(0)), True)
                    req = records[0][0]
        except : return

        self.startMovie()
        self.sendToConsole("sql", self.getTimeStamp(), "%s" % req, "sgbdr/minisql.png")

        
        try :
            records = self.getRecords(cur, req, True)
            fieldNames = [desc[0] for desc in cur.description ] 
            
            model = BaseTableModel(fieldNames, records, self)
            self.tableview.setModel(model)

            self.tableview.setAccessibleDescription("%s.%s.%s" %  (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)))
        except : pass
        self.stopMovie()

        try :
                if self.tableview.selectionModel() : self.tableview.selectionModel().selectionChanged.connect(self.fixeCurrentRow)
                else : self.tableview.selectionModel().disconnect()
        except : pass        


    def quotedName(self, unquotedName, withSimpleQuote):
        if not unquotedName.islower() or unquotedName.find(" ")!= -1 : unquotedName = "\"%s\"" % unquotedName
        if withSimpleQuote : unquotedName = '%s' % unquotedName
        return unquotedName

            
               
    def fixeCurrentRow(self):
        self.currentRecord = -1
        if self.tableview.selectionModel():
            if self.tableview.selectionModel().selectedRows(0)!= [] :
                row = self.tableview.selectionModel().selectedRows(0)[0].row()
                self.currentRecord = row
                self.changeEnableQSPButtons()
                

    def changeEnableQSPButtons(self):
        if self.TreeView.currentItem() == None: return      
        Item = self.TreeView.currentItem()
        if not Item : return
        if Item.whatsThis(0) in ("table", "view") :
            cond = Item.parent().text(0) == "public" and Item.text(0)=="qsp_metadata"
            self.itemQSPMetadata = Item if cond else None
            self.enableQSPButtons(cond)
        return Item    

    def enableQSPButtons(self, cond):
        self.BDeleteTable.setEnabled(cond)
        self.BVacuumTable.setEnabled(cond)
        self.BViewAbstractFromBase.setEnabled(cond and self.tableview.isVisible() and self.currentRecord != -1)
        self.BDownloadDataFromBase.setEnabled(cond and self.tableview.isVisible() and self.currentRecord != -1)
        self.BDeleteDataFromBase.setEnabled(cond and self.tableview.isVisible() and self.currentRecord != -1)
        
    def viewAbstractData(self):
        self.fixeCurrentRow()
        if self.currentRecord != -1 :
            idData = self.tableview.model().getData(self.currentRecord, 0)
            currentTitle = self.tableview.model().getData(self.currentRecord, 1)
            self.goToLinkURL("VIEW%s|%s" % (idData, currentTitle)) 
            
    def downloadData(self):
        self.fixeCurrentRow()
        if self.currentRecord != -1 :
            idData = self.tableview.model().getData(self.currentRecord, 0)
            self.goToLinkURL("DOWNLOAD%s" % idData) #QUrl


    def deleteData(self):
        self.fixeCurrentRow()
        if self.currentRecord != -1 :
            idData = self.tableview.model().getData(self.currentRecord, 0)
            currentTitle = self.tableview.model().getData(self.currentRecord, 1)
            self.goToLinkURL("DEL%s|%s" % (idData, currentTitle))          

    def goToTabResults(self, iLink):
        if type(iLink) != str : iLink = iLink.toString().replace('file:///','')
        try : self.tabsResults.setCurrentIndex(int(iLink))
        except : pass

    def goToLinkURL(self, idData):

        if type(idData) != str : idData = idData.toString().replace('file:///','')

        if idData.startswith("UPLOAD"):
            
            if self.targetDB == None :
                self.errorMsgBase()
                self.sendToConsole("error", self.getTimeStamp(), "%s not defined !" % "targetDB", self.ipucewarning)
                self.tabs.setCurrentIndex(1)
                return
            
            Item = self.TreeView.currentItem()

            if not Item : return
            
            if Item.whatsThis(0)== "base" : self.targetDB = Item.text(0) 
            elif Item.whatsThis(0) in ("table", "view") : self.targetDB = Item.parent().parent().text(0) 
            else : self.targetDB = Item.parent().text(0)
            
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con != None :
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                if not "qsp_metadata" in self.dicSchemas["public"] :
                    result = self.createQSPMETADATATable(con, self.nameUser.text(), True)
                    if result == False : return

            self.startMovie()
            idData = int(idData.replace("UPLOAD", ""))
            root = self.parent.listWindowsButtons.invisibleRootItem()
            
            mdiSubWindow = self.parent.listSubWindows[root.child(idData).whatsThis(0)]
            for child in mdiSubWindow.children():
                if child.objectName()=="DialogMetaData":
                   self.uploadMetadataInBase(child)
                   self.changeSource = True
            self.stopMovie()        

        elif idData.startswith("DOWNLOAD"):
            idData = idData.replace("DOWNLOAD", "")
            if not self.checkMetadatatodownload(idData, QApplication.translate("QSphere","Download metadata from the base")): return
            self.startMovie()

            dataXML = self.metadatatodownload[idData]
            dataXML = unquote(dataXML)
                        
            self.sendToConsole("info", self.getTimeStamp(), dataXML, self.ipucesuccessfull)

            from io import StringIO as bufferData
            from qsphere.xmlISOparser import xmlISOparser

            if self.parent.objectName() != "qsphereMainWindow" :
               self.parent.clickMainWindow()
               self.parent = self.parent.mainWindow
               self.tabsResults.setTabEnabled(3, self.parent.objectName() == "qsphereMainWindow")
               self.parent.dialogSGBDR = self
            
            mdiWindow, child  = self.parent.initMetadataDialog()

            buffer = bufferData(dataXML)
            myISO = xmlISOparser(None, buffer, 'MEDDE', 'fr') 
            myISO.createISOdataStructure(True)
            if myISO.getTagDictionnary() : child.ReloadDataFromXML(myISO, None, 0)

            dns="pgmdd:host=%s port=%s dbname=%s schema=%s table=%s id=%s" % (self.nameService.text(), self.namePort.text(), self.targetDB, "public", "qsp_metadata", idData)

            child.setWindowTitle("%s%s" % (child.racWindowTitle, dns))
            child.connexion, dns, service, port = self.getConnexion(False, False, None, None, None)
            child.show()
            
            self.parent.majSuperviseur(mdiWindow)
            self.parent.fixeModeDialog(0, 0)
            self.parent.changeMetadataMode()
            self.parent.tabWidget.setCurrentIndex(1)

            self.changedMDDFiles = True
            self.loadListMDDFiles()
            self.stopMovie()


        elif idData.startswith("DEL"):
             idData = idData.replace("DEL", "").split("|")
             if self.askForActions("deleterecord", idData[1]) :
                 self.deleteMetadataInBase(idData[0])
                 self.changeSource = True

        elif idData.startswith("VIEW"):
             idData = idData.replace("VIEW", "").split("|")
             if not self.checkMetadatatodownload(idData[0], QApplication.translate("QSphere","View a file (XML ISO 19139, HTML ...)")): return
             dataXML = self.metadatatodownload[idData[0]]#.decode('utf-8')
             d = xmlEditor(self, "")
             MakeWindowIcon(d, "editxml.png")
             d.setWindowTitle("%s : %s" % (d.racTitle, idData[1]))
             d.editor.setText(dataXML)
             d.checkProperties()
             d.BadMyISO()             
             d.exec_()

    def checkMetadatatodownload(self, idData, actionTitle):
             if not DicoHasKey(self.metadatatodownload, idData) :
                 msg = "<b>%s</b> <br>%s - %s <br> %s : %s" % (actionTitle, \
                                                 QApplication.translate("QSphere","XML Params"), \
                                                 QApplication.translate("QSphere","Not found"), \
                                                 QApplication.translate("QSphere","Search for"), \
                                                 idData)
                 self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)
                 return False
             return True   
        

    
    def fixeImage(self, icon):
        theIcon = QImage(getThemeIcon(icon)) 
        self.imgProgress.setPixmap(QPixmap.fromImage(theIcon))


    def printDataTable(self, printer):
        document = QTextDocument()
        cursor = QTextCursor(document)
        model = self.tableview.model()
        if model == None : return
        
        borne = min(model.rowCount(), 100)        

        if self.mode == "html" :
                document.setDefaultStyleSheet("body { color : black; background-color : white; }")
                html = self.html
                Item = self.TreeView.currentItem()
                try :    
                    html+= "<h2><img src='file:///%s'> %s.%s.%s</h2><br><table><tr>" % (self.pucetable, self.targetDB, self.currentItemData.parent().text(0), self.currentItemData.text(0))
                except :
                    html+= "<h2><img src='file:///%s'> %s</h2><br><table><tr>" % (self.pucetable, self.tableview.toolTip())
                html+= "<table style='background-color: #808080;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'><tr>"
                for col in model.columnNames(): html+= "<th>%s</th>" % col
                html+= "</tr>"
                
                for row in range(borne) :
                    html+= "<tr class='comment'>"
                    for column in range(model.columnCount()):
                        data = "%s" % model.getData(row, column)
                        data = netForHTML(data[:100])
                        html+= "<td>%s</td>" % data
                    html+= "</tr>"
                html+= "</table></body>"
                document.setHtml(html)                
        else :
                table = cursor.insertTable(borne+1, model.columnCount())
                for col in model.columnNames(): cursor.insertText("%s" % col);cursor.movePosition(QTextCursor.NextCell)
                for row in range(borne) :
                    for column in range(table.columns()):
                        data = "%s" % model.getData(row, column)
                        cursor.insertText(data[:100])
                        cursor.movePosition(QTextCursor.NextCell)

        document.print_(printer)
        self.act_getAllUsersForServer.setEnabled(False)
        self.act_getAllPrivilegesForUserDataBase.setEnabled(False)
        self.act_getAllPrivilegesForAllUsersDataBase.setEnabled(False)
        self.act_getAllSchemasDataBase.setEnabled(False)
        self.act_getAllSchemasTablesDataBase.setEnabled(False)        
        self.act_getAllPrimaryKeysDataBase.setEnabled(False)
 

    def printRecords(self, schemas, records):
        listBase, listSchema = [], []
        imgpuceschema = getThemeIcon("pgconsole_ico.png")
        html = ""
        for elt in records :
            base, schema, table = elt[0], elt[1], elt[2]
            if schema  not in listSchema :
                listSchema.append(schema)
                html+= "<h2><img src='file:///%s'> %s</h2>" % (imgpuceschema, schema)
            html+= "<table><tr><td width='10%%'></td><td width='5%%'><img src='file:///%s'></td><td width='95%%'> %s</td></tr></table>" % (self.pucetable, table)
            
        for elt in schemas :
            elt = "%s" % elt
            if elt  not in listSchema :
                html+= "<h2><img src='file:///%s'> %s</h2>" % (imgpuceschema, elt)
        return html


    def sendToHTML(self, html): self.textBrowser.setHtml(html, self.baseurl) 
    
    def sendToConsole(self, nature, timestamp, msg, img):
        img = getThemeIcon(img)
        styles = {
                  "maintitle" : "<table style='border: 2px solid grey; border-radius: 5px; background-color: #1a0dc8; color: white;'>",
                  "title"     : "<table style='border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;'>",
                  "error"     : "<table style='background-color: #FF0000;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'>",
                  "info"      : "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "separator" : "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "sql"       : "<table style='background-color: #BEBEBE;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "sqlplus"   : "<table style='background-color: #808080;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'>" 
                  } 
        if DicoHasKey(styles, nature) : style = styles[nature]
        else : style = "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px;'>"

        if nature == "maintitle" : self.edit.append("%s<tr><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h1><font color='white'><b>%s</b></font></h1></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "title" : self.edit.append("%s<tr><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h1><font color='blue'><b>%s</b></font></h1></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "error" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "info" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "sqlplus" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "sql" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "separator" : self.edit.append("%s<tr valign='middle'><td width='12%%'></td><td width='88%%'><hr style='height:2px; color:cyan; border:1px;'></td></tr></table>" % style)
        self.edit.repaint()
        
    def getTimeStamp(self):
        self.startTime = thedatetime.now()
        return self.startTime.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss")
       
    def startMovie(self):
        self.status_txt.setVisible(True)
        self.movie.start()
        self.status_txt.repaint()
        self.setWindowOpacity(0.5)

    def stopMovie(self):
        self.movie.stop()
        self.status_txt.setVisible(False)
        self.setWindowOpacity(1.0)

    def getServerBases(self, cur, terminate):
        records = self.getRecords(cur, GET_SERVER_BASES, True)
        self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s Port:%s]" % (QApplication.translate("QSphere","Get list of databases"), \
                                                 self.QGISConnexion.currentText(), self.nameService.text(), self.namePort.text()), "sgbdr/pgdatabase.png")
        self.sendToConsole("sql", self.getTimeStamp(), "%s" % GET_SERVER_BASES, "sgbdr/minisql.png")
        self.listBases = [elt[0] for elt in sorted(records)]
        self.MakeBasesTreeView()



    #-------------------------------------
    #FUNCTIONS FOR THE TREEWIDGET
    #-------------------------------------
    def MakeBasesTreeView(self):
        self.TreeView.clear()
        for elt in self.listBases :
            itemBase = QTreeWidgetItem()
            itemBase.setIcon(0, QIcon(self.pucebase))
            itemBase.setText(0, "%s" % elt)
            itemBase.setWhatsThis(0, "base")
            self.TreeView.addTopLevelItem(itemBase)
        self.TreeView.expandAll()
        self.TreeView.resizeColumnToContents(0)
        self.TreeView.resizeColumnToContents(1)


    def MakeTreeViewSchemas(self, parentItem, dictBasesSchemasTables):
        listChildren = parentItem.takeChildren()
        for child in listChildren : parentItem.removeChild(child)
        for key in dictBasesSchemasTables.keys() :
            if key != "" :
               itemSchema = QTreeWidgetItem()
               itemSchema.setIcon(0, QIcon(self.puceschema))
               itemSchema.setText(0, "%s" % key)
               itemSchema.setWhatsThis(0, "schema")
               parentItem.addChild(itemSchema)

               listTables = sorted(dictBasesSchemasTables[key])
               for table in listTables :
                   itemTable = QTreeWidgetItem()
                   itemTable.setIcon(0, QIcon(self.pucetable))
                   schema = self.quotedName(key, True)
                   item = self.quotedName(table, True)
                   value = self.schematable['%s.%s' % (schema, item)][0]
                   typegeom = self.schematable['%s.%s' % (schema, item)][3]
                   
                   if value.find("VIEW")!=-1 :
                           if typegeom.lower().find("point")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/pointview.png")))
                           elif typegeom.lower().find("linestring")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/linestringview.png")))
                           elif typegeom.lower().find("polygon")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/polygonview.png")))
                           else : itemTable.setIcon(0, QIcon(self.puceview))
                           itemTable.setWhatsThis(0, "view")
                   else :
                           if typegeom.lower().find("point")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/point.png")))
                           elif typegeom.lower().find("linestring")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/linestring.png")))
                           elif typegeom.lower().find("polygon")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/polygon.png")))
                           else : itemTable.setIcon(0, QIcon(self.pucetable))
                           itemTable.setWhatsThis(0, "table")
                   itemTable.setText(0, "%s" % table)
                   itemSchema.addChild(itemTable)

    def getRecords(self, cursor, req, returnRecords):
            records = None
            try :
                cursor.execute(req)
                if returnRecords : records = cursor.fetchall()
                startTime = self.startTime
                endTime = thedatetime.now() 
                deltaTime = (endTime - startTime).microseconds / 10000
                self.sendToConsole("info", self.getTimeStamp(), "%s<br>%s ! [%s ms]" % (req, self.successfull, deltaTime), self.ipucesuccessfull)
                return records                
            except psycopg2.Error as e:

                if e.pgcode == None and e.pgerror == None :
                          startTime = self.startTime
                          endTime = thedatetime.now() 
                          deltaTime = (endTime - startTime).microseconds / 10000
                          self.sendToConsole("info", self.getTimeStamp(), "%s<br>%s ! [%s ms]" % (req, self.successfull, deltaTime), self.ipucesuccessfull)
                          return True
                else :         
                          self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)
                return None


    def getInfosBase(self):
        self.startMovie()
        targetDB = "postgres"

        self.act_getAllUsersForServer.setEnabled(False)
        self.act_getAllPrivilegesForUserDataBase.setEnabled(False)
        self.act_getAllPrivilegesForAllUsersDataBase.setEnabled(False)
        self.act_getAllSchemasDataBase.setEnabled(False)
        self.act_getAllSchemasTablesDataBase.setEnabled(False)        
        self.act_getAllPrimaryKeysDataBase.setEnabled(False)
        self.BReloadBase.setEnabled(False)

        
        con, dns, service, port = self.getConnexion(True, False, None, None, None)
          
        if con != None :
            self.act_getAllUsersForServer.setEnabled(True)
            self.act_getAllPrivilegesForUserDataBase.setEnabled(True)
            self.act_getAllPrivilegesForAllUsersDataBase.setEnabled(True)

            self.targetDB = None
           
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()
            req = TERMINATE_BACKEND_BASE % targetDB
            try : cur.execute(req)
            except psycopg2.Error as e:
                self.stopMovie()
                if e.pgcode == None and e.pgerror == None :
                          startTime = self.startTime
                          endTime = thedatetime.now() 
                          deltaTime = (endTime - startTime).microseconds / 10000
                          self.sendToConsole("info", self.getTimeStamp(), "%s ! :<hr>%s</hr>[%s ms]" % (self.successfull, req, deltaTime), self.ipucesuccessfull)
                          return True
                else :         
                          self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)
                return 

            con.commit()
            self.oldSource = ""
            self.getServerBases(cur, False)

            records = self.getRecords(cur, GET_PG_PROCEDURES, True)
            self.getSQLDictionnary(records, "functions")

            records = self.getRecords(cur, GET_PG_KEYWORDS, True)
            self.getSQLDictionnary(records, "keywords")

            self.initCompleter()

            records = self.getRecords(cur, GET_SERVER_NUMERIC_TYPES, True)    
            self.numericTypesCodes = [elt[0] for elt in sorted(records)]
            
            cur.close()
            con.close()
            
        else :    
            msg = "%s : %s [%s Port:%s]" % (QApplication.translate("QSphere","Bad result for"), \
                                                 self.QGISConnexion.currentText(), self.nameService.text(), self.namePort.text())
            self.sendToConsole("error", self.getTimeStamp(), msg, "sgbdr/pgdatabase.png")
            self.TreeView.clear()
            self.textBrowser.setHtml("", self.baseurl) 
        self.stopMovie()     

    def getSQLDictionnary(self, records, key):
        self.sqlDictionnary[key] = [proc for proc in records] #[proc[0] for proc in records]

    def initLexer(self, lexer):
        white = QColor(255,255,255,255)
        black = QColor(0,0,0,255)
        comment = QColor(150,150,150,255)
  
        lexer.setDefaultColor(black)
        lexer.setColor(black, lexer.Default)
        lexer.setColor(comment, lexer.Comment)
        lexer.setColor(comment, lexer.CommentLine)
        lexer.setColor(comment, lexer.CommentDoc)
        lexer.setColor(QColor(204,33,33,255), lexer.Number)
        lexer.setColor(QColor(0,0,255,255), lexer.Keyword)
        lexer.setColor(QColor(125,125,255,255), lexer.DoubleQuotedString)
        lexer.setColor(QColor(0,255,0,255), lexer.SingleQuotedString)
        lexer.setColor(QColor(252,163,61,255), lexer.PlusKeyword)
        lexer.setColor(QColor(255,0,0,255), lexer.Operator)
        lexer.setColor(black, lexer.Identifier)
        lexer.setColor(comment, lexer.PlusComment)
        lexer.setColor(comment, lexer.CommentLineHash)
        lexer.setColor(comment, lexer.CommentDocKeyword)
        lexer.setColor(comment, lexer.CommentDocKeywordError)
        lexer.setPaper(white)
        lexer.setDefaultPaper(white)

        return lexer

    def initCompleter(self):
        wordlist = []
        for name, value in self.sqlDictionnary.items(): wordlist += value   
        wordlist = list(set(wordlist))
        lexer = self.initLexer(self.editorSQL.lexer())
        api = QsciAPIs(lexer)
        for word in wordlist:
                try : api.add(word)
                except :
                    try : api.add(word[0])
                    except : pass
        api.prepare()
        self.editorSQL.lexer().setAPIs(api)        

    def saveFilesSQL(self):
        if self.editorSQL.toolTip() != "":
           fileName = self.editorSQL.toolTip()       
           zLOG = open(fileName, "w")
           zLOG.write(self.editorSQL.text()) 
           zLOG.close()
           self.sendToConsole("info", self.getTimeStamp(), "%s" % fileName, "sgbdr/savesql.png")
           zTitle = QApplication.translate("QSphere", "Information" ) 
           zMsg = QApplication.translate("QSphere","The session's file was saved as")
           SendMessage(self, zTitle , "%s :<br><u><i>%s</i></ul>" % (zMsg, fileName), QgsMessageBar.INFO, self.duration_info)           
        else : self.saveFilesAsSQL()   
        
    def saveFilesAsSQL(self):
        zTitle = QApplication.translate("QSphere","Save as a file")
        MyFileDialog = QFileDialog(self, zTitle)
        MyFileDialog.setNameFilters(self.listFiltres)  
        MyFileDialog.selectNameFilter(self.listFiltres[0])
        MyFileDialog.setDefaultSuffix("qry")
        MyFileDialog.setViewMode(QFileDialog.Detail)
        MyFileDialog.setDirectory(self.InitDir)
        MyFileDialog.setAcceptMode(QFileDialog.AcceptSave)

        FixeLabelsFileDialog(self, MyFileDialog, 1, True)
        
        if MyFileDialog.exec_():
            fileName = FileNameWithExtension(self, MyFileDialog.selectedFiles()[0], MyFileDialog.selectedNameFilter())
            self.InitDir = dirname(fileName)
            zLOG = open(fileName, "w")
            zLOG.write(self.editorSQL.text()) 
            zLOG.close()
            self.editorSQL.setToolTip(fileName)
            self.myDockSQL.setWindowTitle("SQL : %s" % fileName)
            self.sendToConsole("info", self.getTimeStamp(), "%s" % fileName, "sgbdr/savesqlas.png")
                

    def eraseSQL(self):
        self.editorSQL.setText("")
        self.editorSQL.setToolTip("")
        self.myDockSQL.setWindowTitle("SQL :")
              
    
    def openFilesSQL(self):
        zTitle = QApplication.translate("QSphere","Load a SQL file")
        MyFileDialog = QFileDialog(self, zTitle)
        MyFileDialog.setNameFilters(self.listFiltres)  
        MyFileDialog.selectNameFilter(self.listFiltres[0])
        MyFileDialog.setDefaultSuffix("qry")
        MyFileDialog.setViewMode(QFileDialog.Detail)
        MyFileDialog.setDirectory(self.InitDir)
        MyFileDialog.setFileMode(QFileDialog.ExistingFiles)  
        MyFileDialog.setAcceptMode(QFileDialog.AcceptOpen)

        FixeLabelsFileDialog(self, MyFileDialog, 0, True)
        
        if MyFileDialog.exec_():
            fileName = "%s" % (MyFileDialog.selectedFiles()[0])
            if fileName!="" :
                self.InitDir = dirname(fileName)
                zLOG = open(fileName, "r")
                reqSQL = zLOG.read().replace("\n\n", "\n")
                zLOG.close()
                self.myDockSQL.setWindowTitle("SQL : %s" % fileName)
                self.editorSQL.setToolTip(fileName)
                self.editorSQL.setText(reqSQL)
                self.editorSQL.setCursorPosition(self.editorSQL.lines() , 1)
                self.editorSQL.setFocus()
       
    def executeSQL(self):

        automate = (self.sender() == None or self.sender() == self.BReload)

        if self.editorSQL.text() != "" or automate :
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con != None :
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = con.cursor()

                if not automate :
                        req = self.editorSQL.selectedText() if self.editorSQL.hasSelectedText() else self.editorSQL.text()
                        
                html = self.html
                html+= "<script type='text/javascript'>"
                zPath = CorrigePath(dirname(__file__)).replace("\\","/")
                with open("%sressources/html/biblio/viewerdata.js" % zPath, 'r') as filejs :
                        LinesJS = filejs.read()
                html+= LinesJS                 
                html+= "</script>"

             
                html+= "<h2><img src='file:///%s'>[%s:%s] %s - %s : %s <img src='file:///%s'></h2>" % (getThemeIcon("mime/sql.png"), service, port, QApplication.translate("QSphere","Execute SQL"), QApplication.translate("QSphere", "source"), self.targetDB, self.pucebase)
                html+= "<pre>%s %s</pre><br>" % (thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), QApplication.translate("QSphere", "Automate") if automate else "")

                if not automate :
                        oldreq = req
                        collectionReq = req.split(";")
                else : collectionReq = self.chronoSQLtoRefresh
                        
                idreq, listidreq, self.lastSQL = 0, [], []
                schema, PARAMValue = "", ""

                for req in collectionReq :
    
                    if req != "" :      
                          req = "%s ;" % req
    
                          textSQL = ""
                          for line in textwrap.wrap(req, 100) : textSQL+= "%s<br>" % line
                          textSQL = textSQL.replace("--{", "<b><u>").replace("}--", "</u></b>")
                          newreq = ""
                          
                          if req.find("%param$")!=-1 :

                              listreq = req.split( "%param$")
                              
                              for i in range(len(listreq)-1):
                                     if self.condSQL(listreq, i+1):        
                                             paramValue, ok = QInputDialog.getText(None, QApplication.translate("QSphere","Params"), "<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSQL, QApplication.translate("QSphere","Params"), listreq[i+1].replace("\n", "<br>")), QLineEdit.Normal)
                                             if not ok: return
                                             newreq+= "%s%s" % (listreq[i], paramValue)
                                     else :
                                             self.getParamSQL(listreq, i+1, textSQL, schema)
                                             if self.dialogInput.exec_() :
                                                 if listreq[i+1].find('schema')!=-1 : schema = self.dialogInput.textValue()
                                                 newreq+= "%s%s" % (listreq[i], self.dialogInput.textValue())
                                             else : return    
                              newreq+= "%s" % listreq[len(listreq)-1]
                              req = newreq 

                                  
                          if req.find("%PARAM$")!=-1 :
                                  
                              listreq = req.split( "%PARAM$")
                             
                              if PARAMValue == "" :
                                  if self.condSQL(listreq, 1):        
                                             PARAMValue, ok = QInputDialog.getText(None, QApplication.translate("QSphere","Params"), "<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSQL, QApplication.translate("QSphere","Params"), listreq[1].replace("\n", "<br>")), QLineEdit.Normal)
                                             if not ok: return
                                  else :
                                             self.getParamSQL(listreq, 1, textSQL, schema)
                                             if self.dialogInput.exec_() :  PARAMValue = self.dialogInput.textValue()    
                                             else : return                                      

                              req = req.replace("%PARAM$", PARAMValue).strip()

                          if req.strip() != ";" and req.strip() != "":
                                  self.startTime = thedatetime.now()
                                  records = self.getRecords(cur, req, True)
                                  html+= "<table class='sql' cellspacing=0 cellpadding=0>"
                                  html+= "<tr><th class='title' colspan=2><h3>SQL</h3></th></tr>"
                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><td bgcolor='white' bordercolor='black' borderwidth='1px'>%s</td></tr>" % (self.pucequery, self.makeColorHTML(req))  

                                  if cur.description != None :
                                          if self.checkSQLComments.isChecked() :
                                                  i=0 
                                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s %s</h3></th></tr>" % (self.pucetable, QApplication.translate("QSphere", "Structure"), QApplication.translate("QSphere", "Results").lower())
                                                  html+= "<tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'><table>"
                                                  for desc in cur.description : html+= "<tr><th>col%s</th><td>%s</td></tr>" % (i, str(desc)); i+= 1;
                                                  html+= "</table></td></tr>"
    
                                          self.description = list(cur.description)
                                          
                                          if records == True :
                                                  html+= "<tr><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a></td></tr>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))
                                                  self.reloadBase()
                                          else :
                                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s : %s</h3><a href='javascript: switchInfoPerso(%s);'><img src='file:///%s'><span id='msg%s'>%s</span></a></th></tr>" % (self.pucerecords, self.labelRecords.title(), len(records), idreq, self.pucearrow, idreq, QApplication.translate("QSphere", "Hide"))
                                                  html+= "<tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'>"
                                                  html+= "<div id='divacacher%s' style='background: #efe; padding: 5px; width: 100%%; height: 100%%;overflow-x: hidden; overflow-y: scroll;'>" % idreq
                                                  html+= self.getResultsInHtml(cur, records, True, 'maTable%s' % idreq)
                                                  html+= "</div></td></tr>"
                                                  listidreq.append([idreq, len(records)])
                                                  idreq+= 1
                                                  self.lastSQL.append(req)
                                  else :
                                       if records == True :
                                           html+= "<tr><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))
                                           self.reloadBase()
                                       else :        
                                           html+= "<tr valign='top'><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a><br><center><img src='file:///%s'></center>" % (self.pucewarning, self.failed, QApplication.translate("QSphere","SQL Commands"), self.sphery)
                                  html+= "</td></tr></table><br>"


                html+= "<script type='text/javascript'>"
                html+= "function switchInfoPerso(index)"
                html+= "{"
                html+= "divInfo = document.getElementById('divacacher'+index);"
                html+= "divTexte = document.getElementById('msg'+index);"
                html+= "if (divInfo.style.display == 'none'){divInfo.style.display = 'block';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "Hide")
                html+= "else{divInfo.style.display = 'none';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "View")
                html+= "}\n"                      
                html+= "function init(){"

                
                for i in range(len(listidreq)):
                    if listidreq[i][1] <= 500:
                       idreq = listidreq[i][0]
                       html+= "var TableSorter%s = new TSorter;" %(idreq)
                       html+= "TableSorter%s.init('maTable%s');" %(idreq, idreq)
                html+= "}"
                html+= "</script>"
                html+= "<script type='text/javascript'>"    
                html+= "window.onload = init;"
                html+= "</script>"

                                  
                html+= "<br><br><br><br>"
                
                self.sendToHTML(html)
                self.tabsResults.setCurrentIndex(0)
                
                if not self.editorSQL.hasSelectedText() and not automate : self.editorSQL.setText(oldreq)
                
                self.oldSource = "sql"
                self.textBrowser.setAccessibleDescription("sql")
                self.tableview.setAccessibleDescription("sql")
                self.mapview.setAccessibleDescription("sql")
                
            else :

                self.errorMsgBase()
                            

    def condSQL(self, listreq, index):
        if listreq[index].find('schema')==-1 and listreq[index].find('table')==-1 and listreq[index].find('base')==-1 and  listreq[index].find('function')==-1 and  listreq[index].find('fonction')==-1 : return True
        else : return False
            

    def getParamSQL(self, listreq, index, textSQL, schema):
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("QSphere","Params"))
        self.dialogInput.setLabelText("<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSQL, QApplication.translate("QSphere","Params"), listreq[index].replace("\n", "<br>")))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        if listreq[index].find('schema.table')!=-1 : self.dialogInput.setComboBoxItems(sorted(self.schematable.keys()))
        elif listreq[index].find('base')!=-1 : self.dialogInput.setComboBoxItems(self.listBases)
        elif listreq[index].find('function')!=-1 or listreq[index].find('fonction')!=-1 : self.dialogInput.setComboBoxItems(self.listFunctions)
        elif listreq[index].find('schema')!=-1 : self.dialogInput.setComboBoxItems(self.listSchemas)
        elif listreq[index].find('table')!=-1 and schema != "" : self.dialogInput.setComboBoxItems(self.dicSchemas[schema])
        else : self.dialogInput.setComboBoxItems(self.listTables)
        for child in self.dialogInput.children():
            if type(child) == QListView :
                    child.doubleClicked.connect(self.dialogInput.accept)
  

    def createQSPMETADATATable(self, con, user, isSystemAction):
        if self.checkQSphereAsk.isChecked() and not isSystemAction : return   
        
        cur = con.cursor()
        if self.askForActions("createtable", None) :

            msg = "%s ..." % QApplication.translate("QSphere","Must create qsp_metadata")
            self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)
            self.dicSchemas["public"].append("qsp_metadata")
        
            req = """CREATE TABLE qsp_metadata (
                     id SERIAL PRIMARY KEY,
                     title character varying(255),
                     abstract character varying,
                     identificator character varying(255),
                     metadata xml,
                     datasource character varying(255),
                     parentid character varying(100)
                )
                WITH (OIDS=FALSE);
            """
            reqall=req
            cur.execute(req)
            req = """ALTER TABLE public.\"%s\" OWNER TO %s;
                     GRANT ALL ON TABLE public.\"%s\" TO %s;
                     GRANT SELECT ON TABLE public.\"%s\" TO public;
                     """ % ("qsp_metadata", user, "qsp_metadata", user, "qsp_metadata")
            cur.execute(req)
            cur.close()

            reqall+= "<br>%s" % req

            self.successFullMsg(reqall, False)
            self.reloadBase()
            return True
           

        else : return False
            

    def uploadMetadataInBase(self, currentMetadataWindow):
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            if "qsp_metadata" not in self.dicSchemas["public"] :
                result = self.createQSPMETADATATable(con, self.nameUser.text(), True)
                if result == False : return
                Item = self.TreeView.currentItem()
                if not Item : return
            
            dataXML = ExportToXML(currentMetadataWindow, None)
            dataXML = dataXML.decode("utf-8").replace("'", "''")

            fileName = currentMetadataWindow.windowTitle().replace(currentMetadataWindow.racWindowTitle, "")
            obj = currentMetadataWindow.findChild(QLineEdit, "intitule")
            title = obj.text().replace("'", "''") if obj != None else "No result"
            obj = currentMetadataWindow.findChild(MyTextEdit, "resume")
            abstract = obj.toPlainText().replace("'", "''") if obj != None else "No result"
            obj = currentMetadataWindow.findChild(QLineEdit, "identificator")
            identificator = obj.text() if obj != None else "-1"

            zLayer = GetzLayerCombo(currentMetadataWindow, currentMetadataWindow.ComboLayers, True)
            comboLayersCurrentLayerId = currentMetadataWindow.ComboLayers.model().item(currentMetadataWindow.ComboLayers.currentIndex(),1).text()
            if not zLayer :  datasource, parentid = None, -1
            else :
               typedatasource = FixeLayerType(currentMetadataWindow, zLayer.type())
               datasource = zLayer.source()
 
            msg = title
            self.sendToConsole("info", self.getTimeStamp(), "%s : %s [%s:%s]<br>%s<br>%s" % ("uploadMetadataInBase", self.targetDB, service, port, fileName, msg), self.ipuceschema)
      
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            try :
                    req = """INSERT INTO qsp_metadata 
                            (title, abstract, identificator, metadata, datasource, parentid)
                             VALUES ('%s','%s', '%s','%s', '%s', '%s');
                          """ % (title, abstract, identificator, dataXML, datasource, fileName)
                    cur.execute(req)
                    self.successFullMsg(req, True)

            except psycopg2.Error as e:
                    zTitle = QApplication.translate("QSphere","Warning")
                    zMsg = "%s : %s [%s]" % (QApplication.translate("QSphere","Error"), e.pgerror, e.pgcode)
                    SendMessage(self, zTitle , "<u><i>%s</i></u>" % zMsg, QgsMessageBar.WARNING, self.duration_warning)
                    self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)

                

    def deleteMetadataInBase(self, idData):
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
            msg = QApplication.translate("QSphere","Delete the current line")
            self.sendToConsole("info", self.getTimeStamp(), "%s : %s [%s:%s]<br>%s : %s" % ("deleteMetadataInBase", self.targetDB, service, port, msg, idData), self.ipuceschema)
      
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()            

            req = "DELETE FROM qsp_metadata WHERE id=%s ;" % int(idData)
            cur.execute(req)
            self.successFullMsg(req, True)


    def majIconItem(self, Item):
        counter = self.TreeView.topLevelItemCount()
        for i in range(counter):
            item = self.TreeView.topLevelItem(i)
            item.setIcon(0, QIcon(self.pucebase)) if item != Item else item.setIcon(0, QIcon(self.pucebaseopen))
            

    def deleteTable(self):
        if self.targetDB == None : return
        if self.itemQSPMetadata == None : return

        if self.askForActions("deletetable", None) :
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con != None :

                req = "DROP TABLE qsp_metadata ;"
                msg = QApplication.translate("QSphere","Delete all entries")
                self.sendToConsole("title", self.getTimeStamp(), "%s %s : %s [%s:%s]" % (msg, "qsp_metadata", \
                                                     self.targetDB, service, port), "sgbdr/deletetable.png")
                        
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = con.cursor()
                self.getRecords(cur, req, False)

                cur.close()
                con.close()

                self.textBrowser.setHtml("", self.baseurl)
                self.TreeView.currentItem().parent().removeChild(self.itemQSPMetadata)
                self.itemQSPMetadata = None

                try:
                    listTables = self.dicSchemas['public']
                    newlist = [elt for elt in listTables if elt!="qsp_metadata"]
                    del self.dicSchemas['public']
                    self.dicSchemas['public'] = newlist
                except KeyError: pass

                self.enableQSPButtons(False)
                self.successFullMsg(req, False)
                self.tableview.setVisible(False)


        
    def errorMsgBase(self):
        zTitle = QApplication.translate("QSphere","Warning")
        zMsg = QApplication.translate("QSphere","No object focused !")
        SendMessage(self, zTitle , "<u><i>%s</i></u>" % zMsg, QgsMessageBar.WARNING, self.duration_warning) 

    def successFullMsg(self, req, reloadData):
        self.sendToConsole("info", self.getTimeStamp(), "%s ! :<hr>%s</hr>" % (self.successfull, req) , self.ipucesuccessfull)
        if reloadData :
           if self.itemQSPMetadata != None :
              self.getInfosStructureTable(self.itemQSPMetadata, False) 
              if self.tabsResults.currentIndex()!= 1 and self.tableview.toolTip().endswith("public.qsp_metadata") :
                      self.tableview.setAccessibleDescription("refresh") ; self.lightDataViewer();

              
    def vacuumTable(self):
        if self.targetDB == None : return
        if self.itemQSPMetadata == None : return

        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
            req = "VACUUM qsp_metadata ;"
            self.sendToConsole("title", self.getTimeStamp(), "%s %s : %s [%s:%s]" % (QApplication.translate("QSphere","Vacuum table"), "qsp_metadata", \
                                                 self.targetDB, service, port), "sgbdr/vacuumtable.png")
                    
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()
            self.getRecords(cur, req, False)

            cur.close()
            con.close()

            self.successFullMsg(req, True)

    def reloadBase(self):
        Item = self.TreeView.currentItem()
        if not Item : return

        if Item.whatsThis(0) in ("table", "view") : Item = Item.parent().parent()
        elif Item.whatsThis(0)== "schema" : Item = Item.parent()
        self.TreeView.setCurrentItem(Item)
        parent = self.TreeView.currentIndex()
        model = parent.model()

        stateExpand = self.getExpandChild(Item, model, False, False)
        self.oldSource = ""
        self.itemQSPMetadata = None
        self.enableQSPButtons(False)
        self.getInfos(Item)

        self.TreeView.setExpanded(parent, True)
        self.expandChild(parent, model, stateExpand)


    def expandTreeBaseNodes(self):
        Item = self.TreeView.currentItem()
        if not Item : return

        cond = True if self.sender() == self.expandAllBaseNodes else False
        
        if Item.whatsThis(0) in ("table", "view")  : Item = Item.parent().parent()
        elif Item.whatsThis(0)== "schema" : Item = Item.parent()

        self.TreeView.setCurrentItem(Item)        
        indexBase = self.TreeView.currentIndex()
        model = indexBase.model()
        stateExpand = self.getExpandChild(Item, model, True, cond)

        self.TreeView.setExpanded(indexBase, cond)
        self.expandChild(indexBase, model, stateExpand)
        self.enableQSPButtons(False)

 
    def expandTreeAllNodes(self):
          if self.sender() == self.expandAllNodes : self.TreeView.expandAll()
          else :
             self.TreeView.collapseAll()
             self.TreeView.expandItem(self.TreeView.itemAt(0, 0))

    def getExpandChild(self, parent, model, forcestate, state):
            expandChildState = {}
            for i in range( parent.childCount()):
                    child = parent.child(i)
                    expandChildState[child.text(0)] = child.isExpanded() if not forcestate else state
            return expandChildState
        
    def expandChild(self, parent, model, expand):
            i = 0
            for key in expand.keys():
                    child = self.TreeView.findItems(key, Qt.MatchFixedString| Qt.MatchRecursive)
                    if child :
                        indexChild = model.index(i,0, parent)
                        self.TreeView.setExpanded(indexChild, expand[key])
                    i+= 1

    def getInfos(self, Item):
        if type(self.sender()) == QTabWidget : return
        
        if not Item : return      
        expandState = Item.isExpanded()
        self.currentItemData = None
        if Item.whatsThis(0)== "base" :
             newSource = "%s" %  Item.text(0)  
             if self.oldSource != newSource :
                 self.majIconItem(Item)
                 self.getInfosBaseSchemas(Item)
                 if self.infosChild == False :
                    if self.textBrowser.accessibleDescription() != "sqlresults" :
                            self.textBrowser.setHtml("", self.baseurl) 

        elif Item.whatsThis(0) in ("table", "view") :
             newSource = "%s.%s.%s" %  (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0))
             if self.oldSource != newSource :
                 self.getInfosStructureTable(Item, False)
                 if  self.tabsResults.currentIndex() == 1 : self.lightDataViewer()
                 elif self.tabsResults.currentIndex() == 2 : self.lightMapViewer()
                 else : self.tabsResults.setCurrentIndex(0)
                 self.currentItemData = Item
                 self.infosChild = False
        else :
             newSource = "%s.%s" %  (Item.parent().text(0), Item.text(0))
             if self.oldSource != newSource :
                 msg = "%s : %s [%s - %s]" % (QApplication.translate("QSphere","Bad result for"), self.targetDB, Item.whatsThis(0), Item.text(0))
                 self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)
             

        if Item.whatsThis(0)== "table" and self.oldSource != newSource : self.changeSource = True
        self.oldSource = newSource
        Item.setExpanded(not expandState)
        
    
    def getInfosBaseSchemas(self, Item): 
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDB = Item.text(0) if Item.text(0)!="" else "postgres"
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        if con != None :

            self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s:%s]" % (QApplication.translate("QSphere","Get list of schemas and tables"), \
                                                 self.targetDB, service, port), "sgbdr/schemastables.png")
                    
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            self.listSchemas = [elt[0] for elt in sorted(self.getRecords(cur, GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, True))]
            self.getSQLDictionnary(self.listSchemas, "schemas")

            if self.checkSystemObjects.isChecked() :
                    listOuterSchemas = str(tuple(self.listSchemas))
                    records = self.getRecords(cur, GET_ALL_SCHEMAS_SYSTEM_DATABASE % listOuterSchemas, True)
                    if records :    
                            systemSchemas = [elt[0] for elt in sorted(records)]
                            for schema in systemSchemas :
                                if schema not in self.listSchemas : self.listSchemas.append(schema)   
            
            tempoListTables = self.getRecords(cur, GET_ALL_SCHEMAS_TABLES_DATABASE, True)
            if tempoListTables :
                    self.listTables = [elt[2] for elt in sorted(tempoListTables)] if tempoListTables else []

            if self.checkSystemObjects.isChecked() :
                    tempoListSystemSchemas = self.getRecords(cur, GET_SERVER_SYSTEM_SCHEMAS % (self.targetDB, listOuterSchemas), True)
                    if tempoListSystemSchemas : tempoListTables.extend(tempoListSystemSchemas)
                        
            if self.listSchemas != []: self.getListTablesForSchemas(tempoListTables)

            self.listFunctions = [elt[0] for elt in sorted(self.getRecords(cur, GET_PG_PROCEDURES, True))]
            self.getSQLDictionnary(self.listFunctions, "functions")


            recordsTables = [elt[0] for elt in sorted(self.getRecords(cur, GET_TABLE_NAMES, True))]

            if DicoHasKey(self.dicSchemas, "public"):
                    if "qsp_metadata" not in self.dicSchemas["public"] : self.createQSPMETADATATable(con, user, False)
                    else :

                        req = "SELECT column_name, data_type, character_maximum_length FROM information_schema.columns WHERE table_schema = '%s' AND table_name = '%s';" % ("public", "qsp_metadata") 
                        records = self.getRecords(cur, req, True)

                        def_qsp_metadata = [(u'id', u'integer', None), (u'title', u'character varying', 255), (u'abstract', u'character varying', None), (u'identificator', u'character varying', 255), (u'metadata', u'xml', None), (u'datasource', u'character varying', 255), (u'parentid', u'character varying', 100)]
                        
                        if  records == def_qsp_metadata : self.sendToConsole("info", self.getTimeStamp(), "%s : <b><u>%s</u></b><br>%s" % (QApplication.translate("QSphere","Find"), "qsp_metadata", QApplication.translate("QSphere","Successfull")), self.ipucetable) 
                        else : self.sendToConsole("error", self.getTimeStamp(), "<b><u>%s</u></b> :<br>%s" % ("qsp_metadata", QApplication.translate("QSphere","Bad format !")), self.ipucewarning)


            self.getSQLDictionnary(self.listTables, "tables")
            self.initCompleter()
            self.MakeTreeViewSchemas(Item, self.dicSchemas)
            self.BReloadBase.setEnabled(True)
            self.act_getAllSchemasDataBase.setEnabled(True)
            self.act_getAllSchemasTablesDataBase.setEnabled(True)            
            self.act_getAllPrimaryKeysDataBase.setEnabled(True)
            
            cur.close()
            con.close()
        
        else :

            if Item.parent() != None :
               msg = "%s : %s" % (QApplication.translate("QSphere","Error"), Item.text(0))
            else :
               msg = "%s : %s" % (QApplication.translate("QSphere","Bad result for"), self.targetDB)
            self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)


    def askForActions(self, action, idData):
                if action == "createtable" :
                    zTitle = "%s ..." % QApplication.translate("QSphere","Confirm create table")
                    zMsg = "%s ?" % QApplication.translate("QSphere","Are you sure you want to create the metadata table")
                elif action == "deletetable" :
                    zTitle = "%s ..." % QApplication.translate("QSphere","Confirm delete table")
                    zMsg = "%s ?" % QApplication.translate("QSphere","Are you sure you want to delete the metadata table")
                elif action == "deleterecord" :
                    zTitle = "%s ..." % QApplication.translate("QSphere","Confirm delete record")
                    zMsg = "%s<br><b>%s</b> ?" % (QApplication.translate("QSphere","Are you sure you want to delete the metadata record"),
                                        idData)

                MyMsgBoxDialog = myMsgBoxDialog(self, zTitle, zMsg, "qsphere_stop.png")
                ret = MyMsgBoxDialog.exec_()    

                if ret == 1 : return True
                else : return False
        

    def getInfosStructureTable(self, Item, console):
        self.enableQSPButtons(False)

        if not Item : return
        if not Item.whatsThis(0) in ("table", "view") : return
        
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
                
        self.targetDB = Item.parent().parent().text(0) if Item.parent().parent().text(0)!= "" and Item.parent().parent()!= None else "postgres"
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        self.majIconItem(Item.parent().parent())

        if Item.whatsThis(0) == "table" :
                self.ipuce = self.ipucetable 
                self.puce = self.pucetable 
        else :        
                self.ipuce = self.ipuceview
                self.puce = self.puceview

        self.sendToConsole("info", self.getTimeStamp(), "Function getInfosStructureTable for %s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)), self.ipuce)
        html = self.html

        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            req = GET_TABLE_FIELDS % Item.text(0)
            
            records = self.getRecords(cur, req, True)
            html+= "<h2><img src='file:///%s'> %s.%s.%s</h2><table><tr>" % (self.puce, self.targetDB, Item.parent().text(0), Item.text(0))
            for desc in cur.description : html+= "<th>%s</th>" %  desc[0]
            html+= "</tr>"

            listFields = []    
            for record in records :
                html+= "<tr>"
                i = 0
                for elt in record :
                     html+= "<td>%s</td>" %  elt
                     if i == 0 : listFields.append(elt)
                     i+= 1
                html+= "</tr>"
            html+= "</table>"

            self.sqlDictionnary["fieldstable"] = listFields
            self.initCompleter()
            

            records = None
            try :
                req = GET_TABLE_COUNT_RECORDS % (Item.parent().text(0), Item.text(0))
                records = self.getRecords(cur, req, True)
                html+= "<pre><img src='file:///%s'> %s %s</pre>" % (self.pucerecords, records[0][0], self.labelRecords)
                maxLimit = int("%s" % records[0])

                self.majComboLimitSQL(maxLimit, False, 0)

            except : pass

            self.itemQSPMetadata = None
            if Item.parent().text(0) == "public" and Item.text(0)=="qsp_metadata" :
                self.enableQSPButtons(True)
                self.itemQSPMetadata = Item
                self.startMovie()
                self.getDataFromBase(Item, cur, -1)
                self.stopMovie()
                

            if Item.whatsThis(0) == "table" :    
                    records = None
                    html+= "<h2><img src='file:///%s'> %s : </h2>" % (getThemeIcon("sgbdr/pucetablestats.png"), QApplication.translate("QSphere","Statistics"))
                    try :
                        req = GET_TABLE_STATISTICS % (Item.text(0))
                        records = self.getRecords(cur, req, True)
                    except : pass

                    
                    if records :
                       records = records[0]
                       html+= "<table>"
                       for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                       html+= "</table>"
                    else :
                       html+= "%s." % QApplication.translate("QSphere","No statistics")
            else :
                    req = GET_VIEW_SQL_PLUS % (Item.text(0))

                    records = self.getRecords(cur, req, True)
                    
                    if records :
                       records = records[0]
                       html+= "<table>"
                       for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                       html+= "</table>"
                    else :
                       html+= "%s." % QApplication.translate("QSphere","No definition")

                    

            geomColumn = self.getGeometryColumn(Item, True)
            if geomColumn != None :
                req = """SELECT srid, srtext, proj4text FROM spatial_ref_sys WHERE srid IN (SELECT Find_SRID('%s', '%s', '%s')) ;""" % (Item.parent().text(0), Item.text(0), geomColumn)
                records = self.getRecords(cur, req, True)
                if records :
                         records = records[0]
                         html+= """<br><h2><img src='file:///%s'> %s</h2><table>""" % (getThemeIcon("sgbdr/pucetablesrid.png"), QApplication.translate("QSphere","Coordinate reference system(s) : "))
                         for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                         html+= "</table>"


            req = GET_TABLE_JOINS % (Item.parent().text(0), Item.text(0))           
            records = self.getRecords(cur, req, True)
            
            if records :
                    html+= "<br><h2><img src='file:///%s'> %s : %s </h2>" % (getThemeIcon("sgbdr/pucetablelinked.png"), QApplication.translate("QSphere","Linked table(s)"), len(records))
                    html+= self.getResultsInHtml(cur, records, False, 0)

            html+= "<br><br><br><br></BODY></HTML>"          
               
        
        if not console :
                self.textBrowser.setAccessibleDescription("%s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)))
                self.textBrowser.setHtml(html, self.baseurl) 
        else : self.sendToConsole("info", self.getTimeStamp(), html, "html.png")


    def getGeometryColumn(self, Item, console):
        if not Item : return
        if Item.whatsThis(0) != "table" and Item.whatsThis(0) != "view" : return
        
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDB = Item.parent().parent().text(0) if Item.text(0)!= "" and Item.parent().parent()!= None else "postgres"
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        
        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()
            self.majIconItem(Item.parent().parent())
            self.sendToConsole("info", self.getTimeStamp(), "Function getGeometryColumn for %s.%s.%s" % (Item.parent().parent().text(0), Item.parent().text(0), Item.text(0)), self.ipucetable)
            req = GET_TABLE_GEOMETRY_COLUMN % (Item.parent().text(0), Item.text(0))
            records = self.getRecords(cur, req, True)
            self.sendToConsole("sql", self.getTimeStamp(), "%s" % req, "sgbdr/minisql.png")

            if records :
               try : return "%s" % records[0][0]
               except : pass 

        return None



    def getDataFromBase(self, Item, cur, offset):
        try :
            req = GET_TABLE_RECORDS % (Item.parent().text(0), Item.text(0), int(self.BComboLimitSQL.currentText()))
        except : return
        
        records = self.getRecords(cur, req, True)
        self.metadatatodownload = {}

        indexdataxml = -1
        i = 0
        for desc in cur.description :
            if desc[0] == "metadata" :
                indexdataxml = i
                break
            i+= 1

        if indexdataxml == -1 : return
     
        for i in range(len(records)): self.metadatatodownload['%s' % records[i][0]] = records[i][indexdataxml].replace("''", "'")
    

    def remoteCtrlSQL(self, req, title):
        self.editorSQL.setText(req)
        self.editorSQL.setToolTip("")
        self.myDockSQL.setWindowTitle("SQL : %s" % title)        
        self.executeSQL()
        
    def getAllUsersForServer(self):
        self.remoteCtrlSQL(GET_ALL_SERVER_USERS, self.sender().text())

    def getAllPrimaryKeysDataBase(self):
        self.remoteCtrlSQL(GET_ALL_PRIMARYKEYS_DATABASE, self.sender().text())           

    def getAllSchemasDataBase(self):    
        self.remoteCtrlSQL(GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, self.sender().text())

    def getAllSchemasTablesDataBase(self):
        self.remoteCtrlSQL(GET_ALL_SCHEMAS_TABLES_DATABASE, self.sender().text())
        
    def getAllPrivilegesForAllUsersDataBase(self):
        self.remoteCtrlSQL("SELECT table_schema, table_name, grantee, string_agg(privilege_type, ', ') AS privileges FROM information_schema.role_table_grants GROUP BY table_schema, table_name, grantee ORDER BY table_schema;", \
                           self.sender().text())
       

    def getAllPrivilegesForUserDataBase(self):
        self.remoteCtrlSQL(GET_ALL_PRIVILEGES_FOR_USER % self.nameUser.text(), self.sender().text())           

    def getConnexion(self, test, hasmsg, typemsg, msg, img):
        service = self.nameService.text()
        port = self.namePort.text()
        user = self.nameUser.text()
        pwd = self.pwdUser.text()
        targetDB = self.targetDB if not test else 'postgres'
        con, dns = None, ""

        if hasmsg :
            msg = "%s : %s [%s:%s]" % (msg, targetDB, service, port)
            self.sendToConsole(typemsg, self.getTimeStamp(), msg , img)    
        
        if user == "" or pwd == "" :
           self.sendToConsole("error", self.getTimeStamp(), QApplication.translate("QSphere","User or password can't be empty"), self.ipucewarning)
        else : 
           dns="host=%s dbname=%s user=%s password=%s port=%s" % (service, targetDB, user, pwd, port)
           try : con = connect(dns)
           except OperationalError : pass
        return con, dns, service, port

    def majComboLimitSQL(self, maxLimit, fixeIndex, index):
        steps = fixeRanges(maxLimit)
        seuils = []
        for seuil in steps :
           elt = "%s" % int(seuil)
           if int(seuil)>0 and elt not in seuils : seuils.append("%s" % int(seuil))
        if seuils == []: seuils = ["%s" % maxLimit]

        self.BComboLimitSQL.blockSignals(True)
        self.BComboLimitSQL.clear()
        self.BComboLimitSQL.addItems(seuils)
        if fixeIndex :
           if index == -1 : self.BComboLimitSQL.setCurrentIndex(len(seuils)-1)
           else : self.BComboLimitSQL.setCurrentIndex(index)
        self.BComboLimitSQL.blockSignals(False)
        self.LabelCounterSQL.setText("%s" % maxLimit)
              
 
    def getResultsInHtml(self, cur, records, fromSQL, idtable):
        if records == None : return "<img src='file:///%s'><br><h1>%s %s</h1>" % (self.pucewarning, self.failed, QApplication.translate("QSphere","SQL Commands"))
        elif records == True : return "<img src='file:///%s'><br><h1>%s %s</h1>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))

        html = self.html
        maxLimit = len(records)

        if len(records) > 500 :
                  self.startMovie()
                  
                  html+= "<a href='1'><h2>%s</h2></a>" % QApplication.translate("QSphere","Too many records, sending results in Data viewer.")

                  fieldNames = [desc[0] for desc in cur.description ]
                  model = BaseTableModel(fieldNames, records, self)
                 
                  self.textBrowser.setAccessibleDescription("sqlresults")
                  self.tableview.setAccessibleDescription("sqlresults")
                  self.mapview.setAccessibleDescription("sqlresults")

                  self.majComboLimitSQL(maxLimit, True, -1)
                 
                  self.tableview.setModel(model)
                  
                  self.stopMovie()
                  self.tabsResults.setCurrentIndex(1)
                  if self.tableview.selectionModel() : self.tableview.selectionModel().selectionChanged.connect(self.fixeCurrentRow)
                  else : self.tableview.selectionModel().disconnect()

        else :
                  if not fromSQL : html+= "<pre><img src='file:///%s'> %s %s</pre><br>" % (self.pucerecords, len(records), self.labelRecords)
                  html+= "<table class='browse' id='%s'><thead><tr>" % idtable
                  k = 0

                  for desc in cur.description :
                         typeField = self.fixeTypeField(desc[0]) 
                         if k == 0 : html+= "<th class='ascend' abbr='%s'>%s</th>" %  (typeField, desc[0])
                         else :  html+= "<th abbr='%s'>%s</th>" %  (typeField, desc[0])
                         k+= 1
                  html+= "</tr></thead><tbody>"
                  for record in records :
                      html+= "<tr>"
                      k = 0
                      for elt in record :
                              if type(elt)==str :
                                  if k == 0 : html+= "<td class='browse.col0'>%s</td>" %  elt.replace("\n", "<br>")
                                  else : html+= "<td class='browse.input'>%s</td>" %  elt.replace("\n", "<br>")
                              else : html+= "<td class='browse.gel'>%s</td>" %  elt
                              k+= 1
                      html+= "</tr>"
                  html+= "</tbody></table><br>"    
        return html


    def fixeTypeField(self, nameField):
        if self.description == [] : return ""
        typeField = ""        
        for field in self.description :
            if field[0] == nameField :
               typeColumn = field[1]
               if typeColumn in self.numericTypesCodes : typeField =  "number"
               break     
        return typeField     


    def getListTablesForSchemas(self, listAllInfos):
        self.dicSchemas = {}
        self.schematable = {}
        for elt in self.listSchemas : self.dicSchemas["%s" % elt]= []
        if not listAllInfos : return
        for elt in sorted(listAllInfos): 
                self.dicSchemas["%s" % elt[1]].append("%s" %elt[2])
                schema = self.quotedName(elt[1], True)
                table = self.quotedName(elt[2], True)
                self.schematable['%s.%s' % (schema, table)] = ('"%s"' % elt[3], "%s" % elt[4], "%s" % elt[5], "%s" % elt[6], "%s" % elt[7])


    def fixeInfosConnexionQGIS(self):
        if self.QGISConnexion.currentText()!= "" :
            params = getParamsConnexionQGIS(self.QGISConnexion.currentText())
            if params!= [] and len(params)== 4:
                self.nameService.setText(params[0])
                self.namePort.setText(params[1])
                self.nameUser.setText(params[2])
                self.pwdUser.setText(params[3])


    def makeColorHTML(self, SQL):
        keywords = ["ALTER","AND","AS","BEGIN","BY", "CALL","CASE","COMMENT","COMMIT","CONNECT","CREATE","DELETE","DROP","ELSE","END", \
                    "EXPLAIN","EXPORT","FROM","GRANT", "GROUP", "IMPORT","IN", "INNER","INSERT","LEFT","LIKE","!=","JOIN","LOAD","LOCK",\
                    "MERGE","NOT","NULL","ON","ORDER", "RECURSIVE", "REFRESH","RENAME","REPLACE","REVOKE", "RIGHT","ROLLBACK","SELECT","SET", \
                    "THEN","TRUNCATE","UNLOAD","UNSET","UPDATE","UPSERT","WHEN","WHERE", "WITH"]
        sql = SQL.replace("\n", " ").split(" ")
        sqlFormated = ""
        openchr = False
        for word in sql :
            if word.strip().upper() in keywords :  sqlFormated+= " <font color='blue'><b>%s</b></font>" % word.strip().upper()
            elif word.find("'")!= -1 :
                 posd = word.find("'")
                 posf = word.find("'", posd+1)
                 if posf!=-1 : sqlFormated+= " %s<font color='#66FF3A'><b>%s</b></font>%s" % (word[0:posd], word[posd:posf+1], word[posf+1:])
                 else :
                       if not openchr : sqlFormated+= " %s<font color='#66FF3A'><b>%s" % (word[0:posd], word[posd:]); openchr = True ;
                       else : sqlFormated+= " %s</b></font>%s" % (word[0:posd+1], word[posd+1:]); openchr = False ;
                        
            elif  word.find("--{")!= -1 : sqlFormated+= " <font color='#BA96A8'><b>%s" % word
            elif  word.find("}--")!= -1 : sqlFormated+= " %s</b></font>" % word
            else :  sqlFormated+= " %s" % word       
        return sqlFormated


    def getDatabasePrivileges(self):
        Item = self.TreeView.currentItem()
        if Item.whatsThis(0)== "table" : Item = Item.parent().parent()
        elif Item.whatsThis(0)== "schema" : Item = Item.parent()
        self.TreeView.setCurrentItem(Item)

        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDB = Item.text(0) if Item.text(0)!="" else "postgres"
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        result = False
        
        if con != None :

            self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s:%s]" % (QApplication.translate("QSphere","Get list of schemas and tables"), \
                                                 self.targetDB, service, port), "sgbdr/schemastables.png")
                    
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            req = "SELECT has_database_privilege(current_database(), 'CREATE'), has_database_privilege(current_database(), 'TEMP')"
            records = self.getRecords(cur, req, True)

            if records : result = bool(records[0][0])

        return result    
        
            
